@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -29,7 +29,7 @@ enum class PlayerColor : uint8_t;
 
 namespace ICN
 {
-    enum : int
+    enum : int32_t
     {
         UNKNOWN,
         ADVBORDE,
@@ -1018,8 +1018,8 @@ namespace ICN
         BUTTONS_FILE_DIALOG_EVIL,
         BUTTONS_EDITOR_FILE_DIALOG_GOOD,
         BUTTONS_EDITOR_FILE_DIALOG_EVIL,
-        BUTTON_INFO_GOOD,
-        BUTTON_INFO_EVIL,
+        BUTTONS_ADVENTURE_OPTIONS_DIALOG_GOOD,
+        BUTTONS_ADVENTURE_OPTIONS_DIALOG_EVIL,
         BUTTON_QUIT_GOOD,
         BUTTON_QUIT_EVIL,
 
@@ -1074,6 +1074,18 @@ namespace ICN
         BUTTON_MAPSIZE_XLARGE,
         BUTTON_MAPSIZE_ALL,
 
+        BUTTON_1_GOOD,
+        BUTTON_2_GOOD,
+        BUTTON_3_GOOD,
+        BUTTON_4_GOOD,
+        BUTTON_5_GOOD,
+
+        BUTTON_1_EVIL,
+        BUTTON_2_EVIL,
+        BUTTON_3_EVIL,
+        BUTTON_4_EVIL,
+        BUTTON_5_EVIL,
+
         BUTTON_MAP_SELECT_GOOD,
         BUTTON_MAP_SELECT_EVIL,
 
@@ -1084,11 +1096,25 @@ namespace ICN
         BUTTON_VERTICAL_DISMISS,
         BUTTON_VERTICAL_EXIT,
         BUTTON_VERTICAL_PATROL,
-        BUTTON_HSCORES_VERTICAL_CAMPAIGN,
-        BUTTON_HSCORES_VERTICAL_EXIT,
-        BUTTON_HSCORES_VERTICAL_STANDARD,
+        BUTTON_HSCORES_VERTICAL_CAMPAIGN_GOOD,
+        BUTTON_HSCORES_VERTICAL_CAMPAIGN_EVIL,
+        BUTTON_HSCORES_VERTICAL_EXIT_GOOD,
+        BUTTON_HSCORES_VERTICAL_EXIT_EVIL,
+        BUTTON_HSCORES_VERTICAL_STANDARD_GOOD,
+        BUTTON_HSCORES_VERTICAL_STANDARD_EVIL,
 
+        BUTTON_SELECT_GOOD,
+        BUTTON_SELECT_EVIL,
+
+        BUTTON_TOGGLE_ALL_ON_GOOD,
+        BUTTON_TOGGLE_ALL_OFF_GOOD,
+        BUTTON_TOGGLE_ALL_ON_EVIL,
+        BUTTON_TOGGLE_ALL_OFF_EVIL,
+
+        EMPTY_OPTION_ICON_BACKGROUND,
         GAME_OPTION_ICON,
+        RESOLUTION_ICON,
+        ARMY_ESTIMATION_ICON,
 
         DIFFICULTY_ICON_EASY,
         DIFFICULTY_ICON_NORMAL,
