@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -114,11 +114,11 @@ namespace
     // Never cache the value of this function as it depends on hero's path and location.
     bool AIIsShowAnimationForHero( const Heroes & hero, const PlayerColorsSet colors )
     {
-        if ( Settings::Get().AIMoveSpeed() == 0 ) {
+        if ( colors == 0 ) {
             return false;
         }
 
-        if ( colors == 0 ) {
+        if ( Settings::Get().AIMoveSpeed() == 0 ) {
             return false;
         }
 
@@ -143,11 +143,11 @@ namespace
 
     bool AIIsShowAnimationForTile( const Maps::Tile & tile, const PlayerColorsSet colors )
     {
-        if ( Settings::Get().AIMoveSpeed() == 0 ) {
+        if ( colors == 0 ) {
             return false;
         }
 
-        if ( colors == 0 ) {
+        if ( Settings::Get().AIMoveSpeed() == 0 ) {
             return false;
         }
 
@@ -265,14 +265,34 @@ namespace
         // Shuffle spells as all of them seem to be equal in power.
         Rand::Shuffle( spells );
 
-        const int32_t spellMultiplier = Difficulty::getGuardianSpellMultiplier( Game::getDifficulty() );
+        double spellMultiplier = Difficulty::getGuardianSpellMultiplier( Game::getDifficulty() );
+
+        // Adjust spell multiplier based on hero's AI role.
+        switch ( hero.getAIRole() ) {
+        case Heroes::Role::SCOUT:
+            // A hero with almost no army risking his life and most likely going to be killed.
+            spellMultiplier = std::min( 1.0, spellMultiplier / 5 );
+            break;
+        case Heroes::Role::COURIER:
+            // A hero who usually delivers army. Might have some army to battle with but he is not a fighter and can be easily defeated.
+            spellMultiplier = std::min( 1.0, spellMultiplier / 3 );
+            break;
+        case Heroes::Role::HUNTER:
+            // A normal hero with normal stuff to do. Not so strong and not so weak.
+            spellMultiplier = std::min( 1.0, spellMultiplier / 2 );
+            break;
+        default:
+            // The rest of heroes are fighters and should keep their spell points for battles.
+            break;
+        }
 
         for ( const Spell spell : spells ) {
             if ( hero.CanCastSpell( spell ) && hero.GetSpellPoints() > spellMultiplier * spell.spellPoints( &hero ) ) {
                 // Looks like this hero knows the spell and casting it won't take too many spell points.
                 // So, let's do it!
-                hero.ActionSpellCast( spell );
-                return;
+                if ( hero.ActionSpellCast( spell ) ) {
+                    return;
+                }
             }
         }
     }
@@ -298,7 +318,8 @@ namespace
 
         assert( hero.CanCastSpell( spellToUse ) );
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        const auto allianceColors = AIGetAllianceColors();
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
             hero.FadeOut( Game::AIHeroAnimSpeedMultiplier() );
         }
@@ -308,7 +329,7 @@ namespace
         hero.SpellCasted( spellToUse );
         hero.GetPath().Reset();
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
             hero.FadeIn( Game::AIHeroAnimSpeedMultiplier() );
         }
@@ -320,7 +341,7 @@ namespace
 
     void AIBattleLose( Heroes & hero, const Battle::Result & res, bool attacker, const fheroes2::Point * centerOn = nullptr, const bool playSound = false )
     {
-        const uint32_t reason = attacker ? res.AttackerResult() : res.DefenderResult();
+        const uint32_t reason = attacker ? res.getAttackerResult() : res.getDefenderResult();
 
         if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
             if ( centerOn != nullptr ) {
@@ -441,10 +462,10 @@ namespace
             // Human controlled castle or hero in this castle was in the battle. Update icons.
             Interface::AdventureMap::Get().renderWithFadeInOrPlanRender( Interface::REDRAW_ICONS );
 
-            castle->ActionAfterBattle( res.AttackerWins() );
+            castle->ActionAfterBattle( res.isAttackerWin() );
 
             // The defender was defeated
-            if ( !res.DefenderWins() ) {
+            if ( !res.isDefenderWin() ) {
                 if ( defender ) {
                     AIBattleLose( *defender, res, false, nullptr, isDefenderHuman );
                 }
@@ -456,19 +477,19 @@ namespace
             }
 
             // The attacker was defeated
-            if ( !res.AttackerWins() ) {
+            if ( !res.isAttackerWin() ) {
                 AIBattleLose( hero, res, true, &( castle->GetCenter() ), isDefenderHuman );
             }
 
             // The attacker won
-            if ( res.AttackerWins() ) {
+            if ( res.isAttackerWin() ) {
                 captureCastle();
 
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+                hero.IncreaseExperience( res.getAttackerExperience() );
             }
             // The defender won
-            else if ( res.DefenderWins() && defender ) {
-                defender->IncreaseExperience( res.GetExperienceDefender() );
+            else if ( res.isDefenderWin() && defender ) {
+                defender->IncreaseExperience( res.getDefenderExperience() );
             }
 
             return;
@@ -531,7 +552,7 @@ namespace
         Interface::AdventureMap::Get().renderWithFadeInOrPlanRender( Interface::REDRAW_HEROES );
 
         // The defender was defeated
-        if ( !res.DefenderWins() ) {
+        if ( !res.isDefenderWin() ) {
             AIBattleLose( *otherHero, res, false, nullptr, isDefenderHuman );
 
             if ( isDefenderHuman ) {
@@ -541,17 +562,17 @@ namespace
         }
 
         // The attacker was defeated
-        if ( !res.AttackerWins() ) {
+        if ( !res.isAttackerWin() ) {
             AIBattleLose( hero, res, true, ( otherHero->isActive() ? &( otherHero->GetCenter() ) : nullptr ), isDefenderHuman );
         }
 
         // The attacker won
-        if ( res.AttackerWins() ) {
-            hero.IncreaseExperience( res.GetExperienceAttacker() );
+        if ( res.isAttackerWin() ) {
+            hero.IncreaseExperience( res.getAttackerExperience() );
         }
         // The defender won
-        else if ( res.DefenderWins() ) {
-            otherHero->IncreaseExperience( res.GetExperienceDefender() );
+        else if ( res.isDefenderWin() ) {
+            otherHero->IncreaseExperience( res.getDefenderExperience() );
         }
     }
 
@@ -616,8 +637,8 @@ namespace
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, dst_index );
 
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
 
                 destroy = true;
             }
@@ -729,7 +750,7 @@ namespace
                         // If AI is extremely low on gold consider taking it
                         if ( kingdomGold < 3000 ) {
                             // Safeguard the calculation since we're working with unsigned values
-                            return ( std::max( exp, 500U ) - 500 ) / 15;
+                            return ( std::max<uint32_t>( exp, 500U ) - 500 ) / 15;
                         }
 
                         // Otherwise Champion always picks experience
@@ -740,7 +761,7 @@ namespace
                         return ( role == Heroes::Role::FIGHTER && exp >= 1500 ) ? 10 : 0;
                     }
 
-                    uint32_t value = std::max( exp, 500U ) - 500;
+                    uint32_t value = std::max<uint32_t>( exp, 500U ) - 500;
                     if ( role == Heroes::Role::FIGHTER ) {
                         value += 500;
                     }
@@ -816,8 +837,8 @@ namespace
 
                 const Battle::Result result = Battle::Loader( hero.GetArmy(), army, dstIndex );
 
-                if ( result.AttackerWins() ) {
-                    hero.IncreaseExperience( result.GetExperienceAttacker() );
+                if ( result.isAttackerWin() ) {
+                    hero.IncreaseExperience( result.getAttackerExperience() );
 
                     captureObject();
                 }
@@ -964,7 +985,8 @@ namespace
 
         assert( world.getTile( indexTo ).getMainObjectType() != MP2::OBJ_HERO );
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        const auto allianceColors = AIGetAllianceColors();
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             // AI-controlled hero cannot activate Stone Liths from the same tile, but should move to this tile from some
             // other tile first, so there is no need to re-center the game area on the hero before his disappearance
             hero.FadeOut( Game::AIHeroAnimSpeedMultiplier() );
@@ -974,7 +996,7 @@ namespace
         hero.Move2Dest( indexTo );
         hero.GetPath().Reset();
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
             hero.FadeIn( Game::AIHeroAnimSpeedMultiplier() );
         }
@@ -1029,7 +1051,8 @@ namespace
             return;
         }
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        const auto allianceColors = AIGetAllianceColors();
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             // AI-controlled hero cannot activate Whirlpool from the same tile, but should move to this tile from some
             // other tile first, so there is no need to re-center the game area on the hero before his disappearance
             hero.FadeOut( Game::AIHeroAnimSpeedMultiplier() );
@@ -1041,7 +1064,7 @@ namespace
 
         AIWhirlpoolTroopLoseEffect( hero );
 
-        if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+        if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
             Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
             hero.FadeIn( Game::AIHeroAnimSpeedMultiplier() );
         }
@@ -1353,8 +1376,8 @@ namespace
             Army army( tile );
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, dst_index );
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
                 complete = true;
 
                 Artifact art;
@@ -1405,8 +1428,8 @@ namespace
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, dst_index );
 
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
 
                 // check magic book
                 if ( hero.HaveSpellBook() &&
@@ -1483,8 +1506,8 @@ namespace
             Army army( tile );
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, dst_index );
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
                 // Daemon Cave always gives 2500 Gold after a battle
                 hero.GetKingdom().AddFundsResource( Funds( Resource::GOLD, 2500 ) );
             }
@@ -1593,8 +1616,8 @@ namespace
             Army army( tile );
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, tileIndex );
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
 
                 // Set ownership of the dwelling to a Neutral (gray) player so that any player can recruit troops without a fight.
                 setColorOnTile( tile, PlayerColor::UNUSED );
@@ -1649,8 +1672,8 @@ namespace
 
         const Battle::Result result = Battle::Loader( hero.GetArmy(), army, dstIndex );
 
-        if ( result.AttackerWins() ) {
-            hero.IncreaseExperience( result.GetExperienceAttacker() );
+        if ( result.isAttackerWin() ) {
+            hero.IncreaseExperience( result.getAttackerExperience() );
 
             Maps::restoreAbandonedMine( tile, Resource::GOLD );
             hero.setObjectTypeUnderHero( MP2::OBJ_MINE );
@@ -1738,8 +1761,8 @@ namespace
             Army army( tile );
 
             const Battle::Result res = Battle::Loader( hero.GetArmy(), army, dst_index );
-            if ( res.AttackerWins() ) {
-                hero.IncreaseExperience( res.GetExperienceAttacker() );
+            if ( res.isAttackerWin() ) {
+                hero.IncreaseExperience( res.getAttackerExperience() );
                 result = true;
             }
             else {
@@ -2377,7 +2400,8 @@ void AI::HeroesCastDimensionDoor( Heroes & hero, const int32_t targetIndex )
         return;
     }
 
-    if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+    const auto allianceColors = AIGetAllianceColors();
+    if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
         Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
         hero.FadeOut( Game::AIHeroAnimSpeedMultiplier() );
     }
@@ -2388,7 +2412,7 @@ void AI::HeroesCastDimensionDoor( Heroes & hero, const int32_t targetIndex )
     hero.setDimensionDoorUsage( hero.getDimensionDoorUses() + 1 );
     hero.GetPath().Reset();
 
-    if ( AIIsShowAnimationForHero( hero, AIGetAllianceColors() ) ) {
+    if ( AIIsShowAnimationForHero( hero, allianceColors ) ) {
         Interface::AdventureMap::Get().getGameArea().SetCenter( hero.GetCenter() );
         hero.FadeIn( Game::AIHeroAnimSpeedMultiplier() );
     }
@@ -2418,7 +2442,8 @@ int32_t AI::HeroesCastSummonBoat( Heroes & hero, const int32_t boatDestinationIn
 
     Maps::Tile & tileSource = world.getTile( boatSource );
 
-    if ( AIIsShowAnimationForTile( tileSource, AIGetAllianceColors() ) ) {
+    const auto allianceColors = AIGetAllianceColors();
+    if ( AIIsShowAnimationForTile( tileSource, allianceColors ) ) {
         gameArea.SetCenter( Maps::GetPoint( boatSource ) );
         gameArea.runSingleObjectAnimation( std::make_shared<Interface::ObjectFadingOutInfo>( tileSource.getMainObjectPart()._uid, boatSource, MP2::OBJ_BOAT ) );
     }
@@ -2427,12 +2452,12 @@ int32_t AI::HeroesCastSummonBoat( Heroes & hero, const int32_t boatDestinationIn
     }
 
     Maps::Tile & tileDest = world.getTile( boatDestinationIndex );
-    assert( tileDest.getMainObjectType() == MP2::OBJ_NONE );
+    assert( tileDest.isSuitableForSummoningBoat() );
 
     tileDest.setBoat( Direction::RIGHT, heroColor );
     tileSource.resetBoatOwnerColor();
 
-    if ( AIIsShowAnimationForTile( tileDest, AIGetAllianceColors() ) ) {
+    if ( AIIsShowAnimationForTile( tileDest, allianceColors ) ) {
         gameArea.SetCenter( Maps::GetPoint( boatDestinationIndex ) );
         gameArea.runSingleObjectAnimation( std::make_shared<Interface::ObjectFadingInInfo>( tileDest.getMainObjectPart()._uid, boatDestinationIndex, MP2::OBJ_BOAT ) );
     }
