@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -28,9 +28,9 @@
 #include <cmath>
 #include <map>
 #include <numeric>
-#include <random>
 #include <set>
 #include <sstream>
+#include <unordered_map>
 #include <utility>
 
 #include "army_troop.h"
@@ -84,6 +84,12 @@ namespace
         ARMY_LEGION = 1000
     };
 
+    const std::unordered_map<ArmySize, std::string> troopSizeNumbers{
+        { ArmySize::ARMY_FEW, "1-4" },       { ArmySize::ARMY_SEVERAL, "5-9" },    { ArmySize::ARMY_PACK, "10-19" },
+        { ArmySize::ARMY_LOTS, "20-49" },    { ArmySize::ARMY_HORDE, "50-99" },    { ArmySize::ARMY_THRONG, "100-249" },
+        { ArmySize::ARMY_SWARM, "250-499" }, { ArmySize::ARMY_ZOUNDS, "500-999" }, { ArmySize::ARMY_LEGION, "1000+" },
+    };
+
     ArmySize getArmySize( const uint32_t count )
     {
         const ArmySize countAsEnum = static_cast<ArmySize>( count );
@@ -149,39 +155,48 @@ namespace
 std::string Army::TroopSizeString( const Troop & troop )
 {
     std::string str;
+    const ArmySize armySize = getArmySize( troop.GetCount() );
 
-    switch ( getArmySize( troop.GetCount() ) ) {
-    case ArmySize::ARMY_FEW:
-        str = _( "A few\n%{monster}" );
-        break;
-    case ArmySize::ARMY_SEVERAL:
-        str = _( "Several\n%{monster}" );
-        break;
-    case ArmySize::ARMY_PACK:
-        str = _( "A pack of\n%{monster}" );
-        break;
-    case ArmySize::ARMY_LOTS:
-        str = _( "Lots of\n%{monster}" );
-        break;
-    case ArmySize::ARMY_HORDE:
-        str = _( "A horde of\n%{monster}" );
-        break;
-    case ArmySize::ARMY_THRONG:
-        str = _( "A throng of\n%{monster}" );
-        break;
-    case ArmySize::ARMY_SWARM:
-        str = _( "A swarm of\n%{monster}" );
-        break;
-    case ArmySize::ARMY_ZOUNDS:
-        str = _( "Zounds...\n%{monster}" );
-        break;
-    case ArmySize::ARMY_LEGION:
-        str = _( "A legion of\n%{monster}" );
-        break;
-    default:
-        // Are you passing the correct value?
-        assert( 0 );
-        break;
+    // Numeric estimates
+    if ( Settings::Get().isArmyEstimationViewNumeric() ) {
+        str = _( "%{monster}\n%{range}" );
+        StringReplace( str, "%{range}", troopSizeNumbers.at( armySize ) );
+    }
+    // Verbal estimates
+    else {
+        switch ( armySize ) {
+        case ArmySize::ARMY_FEW:
+            str = _( "A few\n%{monster}" );
+            break;
+        case ArmySize::ARMY_SEVERAL:
+            str = _( "Several\n%{monster}" );
+            break;
+        case ArmySize::ARMY_PACK:
+            str = _( "A pack of\n%{monster}" );
+            break;
+        case ArmySize::ARMY_LOTS:
+            str = _( "Lots of\n%{monster}" );
+            break;
+        case ArmySize::ARMY_HORDE:
+            str = _( "A horde of\n%{monster}" );
+            break;
+        case ArmySize::ARMY_THRONG:
+            str = _( "A throng of\n%{monster}" );
+            break;
+        case ArmySize::ARMY_SWARM:
+            str = _( "A swarm of\n%{monster}" );
+            break;
+        case ArmySize::ARMY_ZOUNDS:
+            str = _( "Zounds...\n%{monster}" );
+            break;
+        case ArmySize::ARMY_LEGION:
+            str = _( "A legion of\n%{monster}" );
+            break;
+        default:
+            // Are you passing the correct value?
+            assert( 0 );
+            break;
+        }
     }
 
     StringReplaceWithLowercase( str, "%{monster}", troop.GetMultiName() );
@@ -190,6 +205,12 @@ std::string Army::TroopSizeString( const Troop & troop )
 
 std::string Army::SizeString( uint32_t size )
 {
+    // Numeric estimates
+    if ( Settings::Get().isArmyEstimationViewNumeric() ) {
+        return troopSizeNumbers.at( getArmySize( size ) );
+    }
+
+    // Verbal estimates
     switch ( getArmySize( size ) ) {
     case ArmySize::ARMY_FEW:
         return _( "army|Few" );
@@ -1923,7 +1944,7 @@ void Army::ArrangeForBattle( const Monster & monster, const uint32_t monstersCou
         stacksCount = maximumTroopCount;
     }
     else {
-        std::mt19937 seededGen( world.GetMapSeed() + static_cast<uint32_t>( tileIndex ) );
+        Rand::PCG32 seededGen( world.GetMapSeed() + static_cast<uint32_t>( tileIndex ) );
 
         stacksCount = Rand::GetWithGen( 3, 5, seededGen );
     }
@@ -1938,7 +1959,7 @@ void Army::ArrangeForBattle( const Monster & monster, const uint32_t monstersCou
         assert( troopToUpgrade != nullptr );
 
         if ( troopToUpgrade->isValid() && troopToUpgrade->isAllowUpgrade() ) {
-            std::mt19937 seededGen( world.GetMapSeed() + static_cast<uint32_t>( tileIndex ) + static_cast<uint32_t>( monster.GetID() ) );
+            Rand::PCG32 seededGen( world.GetMapSeed() + static_cast<uint32_t>( tileIndex ) + static_cast<uint32_t>( monster.GetID() ) );
 
             // 50% chance to get an upgraded stack
             if ( Rand::GetWithGen( 0, 1, seededGen ) == 1 ) {
@@ -1985,7 +2006,7 @@ IStreamBase & operator>>( IStreamBase & stream, Army & army )
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1109_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1109_RELEASE ) {
-        int temp;
+        int32_t temp;
         stream >> temp;
         army._color = static_cast<PlayerColor>( temp );
     }
