@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -59,6 +59,7 @@ namespace Battle
     class Actions;
     class Arena;
     class Cell;
+    class Interface;
     class Position;
     class StatusListBox;
     class Tower;
@@ -68,6 +69,7 @@ namespace Battle
     struct TargetsInfo;
 
     enum class CastleDefenseStructure : int;
+    enum class CellDirection;
 
     void DialogBattleSettings();
     bool DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdom & kingdom );
@@ -98,6 +100,18 @@ namespace Battle
         CAPTAIN
     };
 
+    enum ArmyColor : uint8_t
+    {
+        ARMY_COLOR_BLACK = 0x00,
+        ARMY_COLOR_BLUE = 0x47,
+        ARMY_COLOR_GREEN = 0x67,
+        ARMY_COLOR_RED = 0xbd,
+        ARMY_COLOR_YELLOW = 0x70,
+        ARMY_COLOR_ORANGE = 0xcd,
+        ARMY_COLOR_PURPLE = 0x87,
+        ARMY_COLOR_GRAY = 0x10
+    };
+
     // Sprite data to render over the unit (spell effect animation)
     struct UnitSpellEffectInfo
     {
@@ -162,13 +176,13 @@ namespace Battle
         };
 
     private:
-        HeroBase * _heroBase;
+        HeroBase * _heroBase{ nullptr };
         AnimationSequence _currentAnim;
         int _animationType{ OP_STATIC };
         RandomizedDelay _idleTimer{ 8000 };
 
         int _heroIcnId{ ICN::UNKNOWN };
-        bool _isFlippedHorizontally;
+        bool _isFlippedHorizontally{ false };
         fheroes2::Rect _area;
         fheroes2::Point _offset;
     };
@@ -212,7 +226,7 @@ namespace Battle
         StatusListBox * _battleStatusLog{ nullptr };
     };
 
-    class TurnOrder final : public fheroes2::Rect
+    class TurnOrder final
     {
     public:
         TurnOrder() = default;
@@ -223,36 +237,45 @@ namespace Battle
 
         void set( const fheroes2::Rect & roi, const std::shared_ptr<const Units> & units, const PlayerColor opponentColor )
         {
-            _area = roi;
+            _battleRoi = roi;
             _orderOfUnits = units;
             _opponentColor = opponentColor;
         }
 
-        void redraw( const Unit * current, const uint8_t currentUnitColor, fheroes2::Image & output );
-        bool queueEventProcessing( std::string & msg, const fheroes2::Point & offset ) const;
+        void redraw( const Unit * current, const uint8_t currentUnitColor, const Unit * underCursor, fheroes2::Image & output, const fheroes2::Rect & dialogRoi );
+
+        bool queueEventProcessing( Interface & interface, std::string & msg, const fheroes2::Point & offset, const bool highlightUnitMomevementArea ) const;
+
+        const fheroes2::Rect & getRenderingRoi() const
+        {
+            return _renderingRoi;
+        }
+
+        void restore()
+        {
+            if ( _restorer ) {
+                _restorer->restore();
+            }
+        }
+
+        void clear()
+        {
+            _restorer.reset();
+        }
 
     private:
-        enum ArmyColor : uint8_t
-        {
-            ARMY_COLOR_BLACK = 0x00,
-            ARMY_COLOR_BLUE = 0x47,
-            ARMY_COLOR_GREEN = 0x67,
-            ARMY_COLOR_RED = 0xbd,
-            ARMY_COLOR_YELLOW = 0x70,
-            ARMY_COLOR_ORANGE = 0xcd,
-            ARMY_COLOR_PURPLE = 0x87,
-            ARMY_COLOR_GRAY = 0x10
-        };
-
         using UnitPos = std::pair<const Unit *, fheroes2::Rect>;
 
-        void _redrawUnit( const fheroes2::Rect & pos, const Battle::Unit & unit, const bool revert, const bool isCurrentUnit, const uint8_t currentUnitColor,
-                          fheroes2::Image & output ) const;
+        static void _redrawUnit( const fheroes2::Rect & pos, const Battle::Unit & unit, const bool revert, const uint8_t currentUnitColor, fheroes2::Image & output );
 
         std::weak_ptr<const Units> _orderOfUnits;
         PlayerColor _opponentColor{ PlayerColor::NONE };
-        fheroes2::Rect _area;
+        fheroes2::Rect _renderingRoi;
+        fheroes2::Rect _battleRoi;
         std::vector<UnitPos> _rects;
+
+        std::unique_ptr<fheroes2::ImageRestorer> _restorer;
+        bool _isInsideBattleField{ false };
     };
 
     class PopupDamageInfo : public Dialog::FrameBorder
@@ -323,6 +346,17 @@ namespace Battle
         fheroes2::Point getRelativeMouseCursorPos() const;
 
         void setStatus( const std::string & message, const bool top );
+
+        void setUnitTobeHighlighted( const Unit * unit )
+        {
+            _unitToHighlight = unit;
+        }
+
+        void setUnitToShowMovementArea( const Unit * unit )
+        {
+            _highlightUnitMovementArea = unit;
+        }
+
         void SetOrderOfUnits( const std::shared_ptr<const Units> & units );
         void FadeArena( const bool clearMessageLog );
 
@@ -385,6 +419,9 @@ namespace Battle
 
         void RedrawTroopCount( const Unit & unit );
 
+        bool _drawTroopSpriteWithMoatMask( const Unit & unit, const fheroes2::Sprite & sprite, const fheroes2::Point & offset, const fheroes2::Point & movementDelta,
+                                           const CellDirection movementDirection );
+
         void _redrawActionArmageddonSpell();
         void _redrawActionArrowSpell( const Unit & target );
         void _redrawActionBloodLustSpell( const Unit & target );
@@ -431,7 +468,7 @@ namespace Battle
         void MouseLeftClickBoardAction( const int themes, const Cell & cell, const bool isConfirmed, Actions & actions );
         bool MousePressRightBoardAction( const Cell & cell ) const;
 
-        int GetBattleCursor( std::string & statusMsg ) const;
+        int GetBattleCursor( std::string & statusMsg, const bool highlightUnitMomevementArea );
         int GetBattleSpellCursor( std::string & statusMsg ) const;
 
         void _startAutoCombat( const Unit & unit, Actions & actions );
@@ -448,6 +485,7 @@ namespace Battle
         fheroes2::Image _hexagonShadow;
         fheroes2::Image _hexagonGridShadow;
         fheroes2::Image _hexagonCursorShadow;
+        fheroes2::Image _hexagonHighlightShadow;
 
         int _battleGroundIcn{ ICN::UNKNOWN };
         int _borderObjectsIcn{ ICN::UNKNOWN };
@@ -457,8 +495,8 @@ namespace Battle
         fheroes2::Button _buttonSkip;
         Status status;
 
-        std::unique_ptr<OpponentSprite> _opponent1;
-        std::unique_ptr<OpponentSprite> _opponent2;
+        std::unique_ptr<OpponentSprite> _attackingOpponent;
+        std::unique_ptr<OpponentSprite> _defendingOpponent;
 
         Spell humanturn_spell{ Spell::NONE };
         bool humanturn_exit{ true };
@@ -480,6 +518,8 @@ namespace Battle
         const Unit * _currentUnit{ nullptr };
         const Unit * _movingUnit{ nullptr };
         const Unit * _flyingUnit{ nullptr };
+        const Unit * _unitToHighlight{ nullptr };
+        const Unit * _highlightUnitMovementArea{ nullptr };
         const fheroes2::Sprite * _spriteInsteadCurrentUnit{ nullptr };
         fheroes2::Point _movingPos;
         fheroes2::Point _flyingPos;
@@ -505,9 +545,9 @@ namespace Battle
                 DESTROYED = 24
             };
 
-            bool animationIsRequired;
+            bool animationIsRequired{ false };
 
-            uint32_t currentFrameId;
+            uint32_t currentFrameId{ 0 };
         };
 
         BridgeMovementAnimation _bridgeAnimation{ false, BridgeMovementAnimation::UP_POSITION };
@@ -618,7 +658,7 @@ namespace Battle
 
         private:
             BoardActionIntent & _storedIntent;
-            const bool _isFromTouchpad;
+            const bool _isFromTouchpad{ false };
             std::optional<BoardActionIntent> _intent;
         };
     };
