@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2020 - 2024                                             *
+ *   Copyright (C) 2020 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -51,7 +51,7 @@ namespace Battle
 
         // Passability of the board cells can change during the unit's turn even without its intervention (for example, because of a hero's spell cast),
         // we need to keep track of this
-        std::array<bool, ARENASIZE> boardStatus{};
+        std::array<bool, Board::sizeInCells> boardStatus{};
         for ( const Cell & cell : *board ) {
             const int32_t cellIdx = cell.GetIndex();
             assert( Board::isValidIndex( cellIdx ) );
@@ -90,16 +90,13 @@ namespace Battle
                 const int32_t headCellIdx = pos.GetHead()->GetIndex();
                 const int32_t tailCellIdx = pos.GetTail() ? pos.GetTail()->GetIndex() : -1;
 
-                const BattleNodeIndex newNodeIdx = { headCellIdx, tailCellIdx };
-                if ( newNodeIdx == _pathStart ) {
-                    continue;
+                if ( const auto [iter, inserted] = _cache.try_emplace( { headCellIdx, tailCellIdx } ); inserted ) {
+                    // Wide units can occupy overlapping positions, the distance between which is actually zero,
+                    // but since the movement takes place, we will consider the distance equal to 1 in this case
+                    const uint32_t distance = std::max<uint32_t>( Board::GetDistance( unit.GetPosition(), pos ), 1U );
+
+                    iter->second.update( _pathStart, 1, distance );
                 }
-
-                // Wide units can occupy overlapping positions, the distance between which is actually zero,
-                // but since the movement takes place, we will consider the distance equal to 1 in this case
-                const uint32_t distance = std::max( Board::GetDistance( unit.GetPosition(), pos ), 1U );
-
-                _cache.try_emplace( newNodeIdx, _pathStart, 1, distance );
             }
 
             return;
@@ -107,7 +104,7 @@ namespace Battle
 
         // The index of that of the cells of the initial unit's position, which is located
         // in the moat (-1, if there is none)
-        const int32_t pathStartMoatCellIdx = [this, &unit, isMoatBuilt]() {
+        const int32_t pathStartMoatCellIdx = [this, &unit, isMoatBuilt]() -> int32_t {
             if ( !isMoatBuilt ) {
                 return -1;
             }
@@ -130,8 +127,7 @@ namespace Battle
         }();
 
         std::vector<BattleNodeIndex> nodesToExplore;
-
-        nodesToExplore.reserve( ARENASIZE * 2 );
+        nodesToExplore.reserve( Board::sizeInCells * 2 );
         nodesToExplore.push_back( _pathStart );
 
         for ( size_t nodesToExploreIdx = 0; nodesToExploreIdx < nodesToExplore.size(); ++nodesToExploreIdx ) {
@@ -147,7 +143,7 @@ namespace Battle
                 const Cell * currentHeadCell = Board::GetCell( currentHeadCellIdx );
                 assert( currentHeadCell != nullptr );
 
-                const bool isCurrentLeftDirection = ( ( Board::GetDirection( currentTailCellIdx, currentHeadCellIdx ) & LEFT_SIDE ) != 0 );
+                const bool isCurrentLeftDirection = isLeftSide( Board::GetDirection( currentTailCellIdx, currentHeadCellIdx ) );
                 // The moat restrictions can be ignored if the wide unit originally occupied a moat cell, and at the current step any part
                 // of this unit occupies the same moat cell
                 const bool isIgnoreMoat = ( currentHeadCellIdx == pathStartMoatCellIdx || currentTailCellIdx == pathStartMoatCellIdx );
@@ -162,7 +158,7 @@ namespace Battle
                         continue;
                     }
 
-                    const int32_t tailCellIdx = ( Board::GetDirection( currentHeadCellIdx, headCellIdx ) & LEFT_SIDE ) ? headCellIdx + 1 : headCellIdx - 1;
+                    const int32_t tailCellIdx = isLeftSide( Board::GetDirection( currentHeadCellIdx, headCellIdx ) ) ? headCellIdx + 1 : headCellIdx - 1;
 
                     const BattleNodeIndex newNodeIdx = { headCellIdx, tailCellIdx };
                     if ( newNodeIdx == _pathStart ) {
@@ -175,9 +171,7 @@ namespace Battle
 
                     BattleNode & newNode = _cache[newNodeIdx];
                     if ( newNode._from == BattleNodeIndex{ -1, -1 } || newNode._cost > cost ) {
-                        newNode._from = currentNodeIdx;
-                        newNode._cost = cost;
-                        newNode._distance = distance;
+                        newNode.update( currentNodeIdx, cost, distance );
 
                         nodesToExplore.push_back( newNodeIdx );
                     }
@@ -214,9 +208,7 @@ namespace Battle
 
                     BattleNode & newNode = _cache[newNodeIdx];
                     if ( newNode._from == BattleNodeIndex{ -1, -1 } || newNode._cost > cost ) {
-                        newNode._from = currentNodeIdx;
-                        newNode._cost = cost;
-                        newNode._distance = distance;
+                        newNode.update( currentNodeIdx, cost, distance );
 
                         nodesToExplore.push_back( newNodeIdx );
                     }
