@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2021 - 2024                                             *
+ *   Copyright (C) 2021 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -29,6 +29,8 @@
 #include "army.h"
 #include "campaign_data.h"
 #include "difficulty.h"
+#include "game_io.h"
+#include "save_format_version.h"
 #include "serialize.h"
 
 namespace Campaign
@@ -39,17 +41,17 @@ namespace Campaign
         return instance;
     }
 
-    void CampaignSaveData::addCampaignAward( const int awardID )
+    void CampaignSaveData::addCampaignAward( const int32_t awardID )
     {
         _obtainedCampaignAwards.emplace_back( awardID );
     }
 
-    void CampaignSaveData::removeCampaignAward( const int awardID )
+    void CampaignSaveData::removeCampaignAward( const int32_t awardID )
     {
         _obtainedCampaignAwards.erase( std::remove( _obtainedCampaignAwards.begin(), _obtainedCampaignAwards.end(), awardID ), _obtainedCampaignAwards.end() );
     }
 
-    void CampaignSaveData::setEnemyDefeatedAward( const int heroId )
+    void CampaignSaveData::setEnemyDefeatedAward( const int32_t heroId )
     {
         const ScenarioInfoId & currentScenarioInfo = getCurrentScenarioInfoId();
         const std::vector<CampaignAwardData> obtainableAwards = CampaignAwardData::getCampaignAwardData( currentScenarioInfo );
@@ -89,19 +91,21 @@ namespace Campaign
 
     void CampaignSaveData::addDaysPassed( const uint32_t days )
     {
-        _daysPassed += days;
+        _daysPassed.emplace_back( days );
+        assert( _daysPassed.size() == _finishedMaps.size() );
     }
 
     void CampaignSaveData::reset()
     {
         _finishedMaps.clear();
+        _daysPassed.clear();
         _bonusesForFinishedMaps.clear();
         _obtainedCampaignAwards.clear();
         _carryOverTroops.clear();
         _currentScenarioInfoId = { -1, -1 };
         _currentScenarioBonusId = -1;
-        _daysPassed = 0;
         _difficulty = CampaignDifficulty::Normal;
+        _minDifficulty = CampaignDifficulty::Normal;
     }
 
     void CampaignSaveData::setCarryOverTroops( const Troops & troops )
@@ -121,7 +125,7 @@ namespace Campaign
 
     uint32_t CampaignSaveData::getCampaignDifficultyPercent() const
     {
-        switch ( _difficulty ) {
+        switch ( _minDifficulty ) {
         case CampaignDifficulty::Easy:
             return 125;
         case CampaignDifficulty::Normal:
@@ -161,7 +165,8 @@ namespace Campaign
     OStreamBase & operator<<( OStreamBase & stream, const CampaignSaveData & data )
     {
         return stream << data._currentScenarioInfoId.campaignId << data._currentScenarioInfoId.scenarioId << data._currentScenarioBonusId << data._finishedMaps
-                      << data._bonusesForFinishedMaps << data._daysPassed << data._obtainedCampaignAwards << data._carryOverTroops << data._difficulty;
+                      << data._bonusesForFinishedMaps << data._daysPassed << data._obtainedCampaignAwards << data._carryOverTroops << data._difficulty
+                      << data._minDifficulty;
     }
 
     IStreamBase & operator>>( IStreamBase & stream, CampaignSaveData & data )
@@ -172,7 +177,36 @@ namespace Campaign
         // Make sure that the number of elements in the vector of map bonuses matches the number of elements in the vector of finished maps
         data._bonusesForFinishedMaps.resize( data._finishedMaps.size(), -1 );
 
-        return stream >> data._daysPassed >> data._obtainedCampaignAwards >> data._carryOverTroops >> data._difficulty;
+        static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1111_RELEASE, "Remove the logic below." );
+        if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1111_RELEASE ) {
+            uint32_t daysPassed{ 0 };
+            stream >> daysPassed;
+
+            data._daysPassed.assign( data._finishedMaps.size(), 1 );
+
+            if ( !data._daysPassed.empty() && daysPassed > data._daysPassed.size() ) {
+                data._daysPassed.back() += static_cast<uint32_t>( daysPassed - data._daysPassed.size() );
+            }
+        }
+        else {
+            stream >> data._daysPassed;
+
+            // Make sure that the number of elements in the vector of the number of days spent completing individual maps matches the number of elements in the vector of
+            // finished maps
+            data._daysPassed.resize( data._finishedMaps.size(), 1 );
+        }
+
+        stream >> data._obtainedCampaignAwards >> data._carryOverTroops >> data._difficulty;
+
+        static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE1_1108_RELEASE, "Remove the logic below." );
+        if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE1_1108_RELEASE ) {
+            data._minDifficulty = data._difficulty;
+        }
+        else {
+            stream >> data._minDifficulty;
+        }
+
+        return stream;
     }
 
     ScenarioVictoryCondition getCurrentScenarioVictoryCondition()
