@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2021 - 2025                                             *
+ *   Copyright (C) 2021 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -20,7 +20,9 @@
 
 #pragma once
 
+#include <algorithm>
 #include <cstdint>
+#include <numeric>
 #include <optional>
 #include <vector>
 
@@ -81,7 +83,7 @@ namespace Campaign
 
         uint32_t getDaysPassed() const
         {
-            return _daysPassed;
+            return std::accumulate( _daysPassed.cbegin(), _daysPassed.cend(), static_cast<uint32_t>( 0 ) );
         }
 
         int32_t getDifficulty() const
@@ -89,9 +91,15 @@ namespace Campaign
             return _difficulty;
         }
 
-        void setDifficulty( const int32_t difficulty )
+        int32_t getMinDifficulty() const
+        {
+            return _minDifficulty;
+        }
+
+        void setDifficulty( const int32_t difficulty, const bool resetMinDifficulty )
         {
             _difficulty = difficulty;
+            _minDifficulty = ( resetMinDifficulty ? difficulty : std::min( difficulty, _minDifficulty ) );
         }
 
         // Get the campaign difficulty in percents for rating calculations.
@@ -106,13 +114,13 @@ namespace Campaign
 
         void setCurrentScenarioInfo( const ScenarioInfoId & scenarioInfoId, const int32_t bonusId = -1 );
         void addCurrentMapToFinished();
-        void addCampaignAward( const int awardID );
+        void addCampaignAward( const int32_t awardID );
         void setCarryOverTroops( const Troops & troops );
         void reset();
         void addDaysPassed( const uint32_t days );
-        void removeCampaignAward( const int awardID );
+        void removeCampaignAward( const int32_t awardID );
 
-        void setEnemyDefeatedAward( const int heroId );
+        void setEnemyDefeatedAward( const int32_t heroId );
 
         void removeAllAwards()
         {
@@ -128,15 +136,16 @@ namespace Campaign
         CampaignSaveData() = default;
 
         std::vector<ScenarioInfoId> _finishedMaps;
+        std::vector<uint32_t> _daysPassed;
         std::vector<int32_t> _bonusesForFinishedMaps;
-        std::vector<int> _obtainedCampaignAwards;
+        std::vector<int32_t> _obtainedCampaignAwards;
         std::vector<Troop> _carryOverTroops;
 
         ScenarioInfoId _currentScenarioInfoId;
         int32_t _currentScenarioBonusId{ -1 };
 
-        uint32_t _daysPassed{ 0 };
         int32_t _difficulty{ CampaignDifficulty::Normal };
+        int32_t _minDifficulty{ CampaignDifficulty::Normal };
     };
 
     // Call this function only when playing campaign scenario.
