@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -28,7 +28,6 @@
 #include <cassert>
 #include <iterator>
 #include <sstream>
-#include <utility>
 
 #include "agg_image.h"
 #include "ai_planner.h"
@@ -389,6 +388,9 @@ void Castle::LoadFromMP2( const std::vector<uint8_t> & data )
 
     // Skip the rest of 29 bytes.
 
+    // MageGuild
+    _mageGuild.initialize( _race, HaveLibraryCapability() );
+
     _postLoad();
 }
 
@@ -417,6 +419,9 @@ void Castle::loadFromResurrectionMap( const Maps::Map_Format::CastleMetadata & m
         _name = metadata.customName;
     }
 
+    // MageGuild
+    _mageGuild.initialize( _race, HaveLibraryCapability(), metadata.mustHaveSpells, metadata.bannedSpells );
+
     _postLoad();
 }
 
@@ -497,8 +502,6 @@ void Castle::_postLoad()
         _captain.SetSpellPoints( _captain.GetMaxSpellPoints() );
     }
 
-    // MageGuild
-    _mageGuild.initialize( _race, HaveLibraryCapability() );
     _trainGuestHeroAndCaptainInMageGuild();
 
     // AI troops auto pack for gray towns
@@ -680,7 +683,7 @@ double Castle::getVisitValue( const Heroes & hero ) const
     const int mageGuildLevel = GetLevelMageGuild();
     if ( mageGuildLevel > 0 ) {
         const int spellPower = hero.GetPower();
-        const SpellStorage & guildSpells = _mageGuild.GetSpells( GetLevelMageGuild(), isLibraryBuild() );
+        const SpellStorage & guildSpells = _mageGuild.GetSpells( GetLevelMageGuild(), isLibraryBuilt() );
         for ( const Spell & spell : guildSpells ) {
             if ( hero.CanLearnSpell( spell ) && !hero.HaveSpell( spell, true ) ) {
                 spellValue += spell.getStrategicValue( heroArmyStrength, hero.GetMaxSpellPoints(), spellPower );
@@ -844,8 +847,9 @@ void Castle::ActionNewWeek()
             DWELLING_UPGRADE2, DWELLING_UPGRADE3, DWELLING_UPGRADE4, DWELLING_UPGRADE5, DWELLING_UPGRADE6, DWELLING_UPGRADE7 };
 
     const bool isNeutral = ( GetColor() == PlayerColor::NONE );
-    const bool isPlagueWeek = ( world.GetWeekType().GetType() == WeekName::PLAGUE );
-    const bool isMonsterWeek = ( world.GetWeekType().GetType() == WeekName::MONSTERS );
+    const WeekName weekType = world.GetWeekType().GetType();
+    const bool isPlagueWeek = ( weekType == WeekName::PLAGUE );
+    const bool isMonsterWeek = ( weekType == WeekName::MONSTERS );
 
     if ( !isPlagueWeek ) {
         static const std::array<uint32_t, 6> basicDwellings
