@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -31,6 +31,7 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "army.h"
@@ -140,8 +141,10 @@ public:
         Close, // Close the dialog.
         NextCastle, // Open main dialog of the next castle.
         PreviousCastle, // Open main dialog of the previous castle.
-        NextCostructionWindow, // Open construction dialog of the next castle.
-        PreviousCostructionWindow // Open construction dialog of the previous castle.
+        NextConstructionWindow, // Open construction dialog of the next castle.
+        PreviousConstructionWindow, // Open construction dialog of the previous castle.
+        NextMageGuildWindow, // Open Mage Guild dialog of the next castle.
+        PreviousMageGuildWindow, // Open Mage Guild dialog of the previous castle.
     };
 
     Castle() = default;
@@ -287,7 +290,7 @@ public:
 
     void DrawImageCastle( const fheroes2::Point & pt ) const;
 
-    CastleDialogReturnValue OpenDialog( const bool openConstructionWindow, const bool fade, const bool renderBackgroundDialog );
+    CastleDialogReturnValue OpenDialog( const bool openConstructionWindow, const bool openMageGuildWindow, const bool fade, const bool renderBackgroundDialog );
 
     int GetAttackModificator( const std::string * /* unused */ ) const
     {
@@ -389,6 +392,13 @@ private:
         RecruitHero // Recruit a hero.
     };
 
+    enum class MageGuildDialogResult : uint8_t
+    {
+        DoNothing,
+        NextMageGuildWindow, // Open Mage Guild dialog for the next castle.
+        PrevMageGuildWindow, // Open Mage Guild dialog for the previous castle.
+    };
+
     // Checks whether this particular building is currently built in the castle (unlike
     // the isBuild(), upgraded versions of the same building are not taken into account)
     bool _isExactBuildingBuilt( const uint32_t buildingToCheck ) const;
@@ -400,7 +410,7 @@ private:
 
     void _openTavern() const;
     void _openWell();
-    void _openMageGuild( const Heroes * hero ) const;
+    MageGuildDialogResult _openMageGuild( const Heroes * hero ) const;
     void _joinRNDArmy();
     void _postLoad();
 
@@ -416,7 +426,7 @@ private:
     friend OStreamBase & operator<<( OStreamBase & stream, const Castle & castle );
     friend IStreamBase & operator>>( IStreamBase & stream, Castle & castle );
 
-    int _race{ Race::NONE };
+    int32_t _race{ Race::NONE };
     uint32_t _constructedBuildings{ 0 };
     uint32_t _disabledBuildings{ 0 };
 
@@ -486,11 +496,11 @@ namespace CastleDialog
         bool _isOnlyBoat{ false };
     };
 
-    struct BuildingRenderInfo
+    struct BuildingRenderInfo final
     {
-        BuildingRenderInfo( const BuildingType buildingType, const fheroes2::Rect & buildingRect )
+        BuildingRenderInfo( const BuildingType buildingType, std::vector<fheroes2::Rect> buildingAreas )
             : id( buildingType )
-            , coord( buildingRect )
+            , areas( std::move( buildingAreas ) )
         {
             // Do nothing.
         }
@@ -500,8 +510,8 @@ namespace CastleDialog
             return buildingType == static_cast<uint32_t>( id );
         }
 
-        BuildingType id;
-        fheroes2::Rect coord;
+        BuildingType id{ BUILD_NOTHING };
+        std::vector<fheroes2::Rect> areas;
     };
 
     struct BuildingsRenderQueue : std::vector<BuildingRenderInfo>
