@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -20,8 +20,8 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifndef H2DIALOG_H
-#define H2DIALOG_H
+
+#pragma once
 
 #include <cstddef>
 #include <cstdint>
@@ -116,9 +116,12 @@ namespace Dialog
     bool SelectGoldOrExp( const std::string &, const std::string &, uint32_t gold, uint32_t expr, const Heroes & );
     int SelectSkillFromArena();
     bool SelectCount( std::string header, const int32_t min, const int32_t max, int32_t & selectedValue, const int32_t step = 1,
-                      const fheroes2::DialogElement * uiElement = nullptr );
+                      const fheroes2::DialogElement * topUiElement = nullptr, const fheroes2::DialogElement * bottomUiElement = nullptr );
+
+    // If character limit is set to 0, then no limitation for the resulting string will be applied.
     bool inputString( const fheroes2::TextBase & title, const fheroes2::TextBase & body, std::string & result, const size_t charLimit, const bool isMultiLine,
                       const std::optional<fheroes2::SupportedLanguage> & textLanguage );
+
     Troop RecruitMonster( const Monster & monster0, const uint32_t available, const bool allowDowngradedMonster, const int32_t windowOffsetY );
     void DwellingInfo( const Monster &, const uint32_t available );
     int ArmyInfo( const Troop & troop, int flags, bool isReflected = false, const int32_t windowOffsetY = 0 );
@@ -126,7 +129,7 @@ namespace Dialog
     int ArmyJoinWithCost( const Troop &, const uint32_t join, const uint32_t gold );
     int ArmySplitTroop( const int32_t freeSlots, const int32_t redistributeMax, int32_t & redistributeCount, bool & useFastSplit, const std::string & troopName );
     void Marketplace( Kingdom & kingdom, bool fromTradingPost );
-    void MakeGiftResource( Kingdom & kingdom );
+    int MakeGiftResource( Kingdom & kingdom );
     int BuyBoat( bool enable );
     void ThievesGuild( const bool oracle );
     void GameInfo();
@@ -147,6 +150,8 @@ namespace Dialog
 
         void redraw();
 
+        static int32_t getButtonAreaHeight();
+
     protected:
         std::unique_ptr<fheroes2::ImageRestorer> _restorer;
         fheroes2::Rect area;
@@ -160,7 +165,7 @@ namespace Dialog
     class FrameBox : public NonFixedFrameBox
     {
     public:
-        FrameBox( int height, bool buttons = false )
+        explicit FrameBox( int height, bool buttons = false )
             : Dialog::NonFixedFrameBox( height, -1, buttons )
         {
             // Do nothing.
@@ -219,5 +224,3 @@ namespace Dialog
         int border;
     };
 }
-
-#endif
