@@ -20,7 +20,6 @@
 
 #include "dialog_interface_settings.h"
 
-#include <cstdint>
 #include <string>
 #include <utility>
 
@@ -35,9 +34,9 @@
 #include "settings.h"
 #include "translations.h"
 #include "ui_button.h"
-#include "ui_constants.h"
 #include "ui_dialog.h"
 #include "ui_option_item.h"
+#include "ui_window.h"
 
 namespace
 {
@@ -47,17 +46,19 @@ namespace
         InterfaceType,
         InterfacePresence,
         CursorType,
+        ArmyEstimationMode,
         Exit
     };
 
-    const fheroes2::Size offsetBetweenOptions{ 118, 110 };
-    const fheroes2::Point optionOffset{ 69, 47 };
-    const int32_t optionWindowSize{ 65 };
+    const fheroes2::Size offsetBetweenOptions{ fheroes2::threeOptionsStepX, fheroes2::optionsStepY };
+    const fheroes2::Point optionOffset{ fheroes2::threeOptionsOffsetX, fheroes2::optionsOffsetY };
 
-    const fheroes2::Rect interfaceTypeRoi{ optionOffset.x, optionOffset.y, optionWindowSize, optionWindowSize };
-    const fheroes2::Rect interfacePresenceRoi{ optionOffset.x + offsetBetweenOptions.width, optionOffset.y, optionWindowSize, optionWindowSize };
-    const fheroes2::Rect cursorTypeRoi{ optionOffset.x, optionOffset.y + offsetBetweenOptions.height, optionWindowSize, optionWindowSize };
-    const fheroes2::Rect scrollSpeedRoi{ optionOffset.x + offsetBetweenOptions.width, optionOffset.y + offsetBetweenOptions.height, optionWindowSize, optionWindowSize };
+    const fheroes2::Rect interfaceTypeRoi{ optionOffset.x, optionOffset.y, fheroes2::optionIconSize, fheroes2::optionIconSize };
+    const fheroes2::Rect interfacePresenceRoi{ optionOffset.x + offsetBetweenOptions.width, optionOffset.y, fheroes2::optionIconSize, fheroes2::optionIconSize };
+    const fheroes2::Rect armyEstimationModeRoi{ optionOffset.x + offsetBetweenOptions.width * 2, optionOffset.y, fheroes2::optionIconSize, fheroes2::optionIconSize };
+    const fheroes2::Rect cursorTypeRoi{ fheroes2::twoOptionsOffsetX, optionOffset.y + offsetBetweenOptions.height, fheroes2::optionIconSize, fheroes2::optionIconSize };
+    const fheroes2::Rect scrollSpeedRoi{ fheroes2::twoOptionsOffsetX + offsetBetweenOptions.width, optionOffset.y + offsetBetweenOptions.height, fheroes2::optionIconSize,
+                                         fheroes2::optionIconSize };
 
     void drawInterfacePresence( const fheroes2::Rect & optionRoi )
     {
@@ -76,48 +77,50 @@ namespace
             value = _( "Show" );
         }
 
-        fheroes2::drawOption( optionRoi, interfaceStateIcon, _( "Interface" ), std::move( value ), fheroes2::UiOptionTextWidth::TWO_ELEMENTS_ROW );
+        fheroes2::drawOption( optionRoi, interfaceStateIcon, _( "Interface" ), std::move( value ), fheroes2::UiOptionTextWidth::THREE_ELEMENTS_ROW );
+    }
+
+    void drawArmyNumberEstimationOption( const fheroes2::Rect & optionRoi )
+    {
+        const bool isArmyEstimationNumeric = Settings ::Get().isArmyEstimationViewNumeric();
+
+        fheroes2::drawOption( optionRoi, fheroes2::AGG::GetICN( ICN::ARMY_ESTIMATION_ICON, isArmyEstimationNumeric ? 1 : 0 ), _( "Army Estimation" ),
+                              isArmyEstimationNumeric ? _( "Numeric" ) : _( "Canonical" ), fheroes2::UiOptionTextWidth::THREE_ELEMENTS_ROW );
     }
 
     SelectedWindow showConfigurationWindow( bool & saveConfiguration )
     {
         fheroes2::Display & display = fheroes2::Display::instance();
 
-        Settings & conf = Settings::Get();
-        const bool isEvilInterface = conf.isEvilInterfaceEnabled();
-        const fheroes2::Sprite & dialog = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::ESPANBKG_EVIL : ICN::ESPANBKG ), 0 );
-        const fheroes2::Sprite & dialogShadow = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::CSPANBKE : ICN::CSPANBKG ), 1 );
+        fheroes2::StandardWindow background( 289, fheroes2::optionsStepY * 2 + 52, true, display );
 
-        const fheroes2::Point dialogOffset( ( display.width() - dialog.width() ) / 2, ( display.height() - dialog.height() ) / 2 );
-        const fheroes2::Point shadowOffset( dialogOffset.x - fheroes2::borderWidthPx, dialogOffset.y );
-
-        const fheroes2::ImageRestorer restorer( display, shadowOffset.x, shadowOffset.y, dialog.width() + fheroes2::borderWidthPx,
-                                                dialog.height() + fheroes2::borderWidthPx );
-        const fheroes2::Rect windowRoi{ dialogOffset.x, dialogOffset.y, dialog.width(), dialog.height() };
-
-        fheroes2::Blit( dialogShadow, display, windowRoi.x - fheroes2::borderWidthPx, windowRoi.y + fheroes2::borderWidthPx );
-        fheroes2::Blit( dialog, display, windowRoi.x, windowRoi.y );
-
-        fheroes2::ImageRestorer emptyDialogRestorer( display, windowRoi.x, windowRoi.y, windowRoi.width, windowRoi.height );
+        const fheroes2::Rect windowRoi = background.activeArea();
+        fheroes2::ImageRestorer emptyDialogRestorer( display, windowRoi.x, windowRoi.y, windowRoi.width, windowRoi.height - 30 );
 
         const fheroes2::Rect windowInterfaceTypeRoi( interfaceTypeRoi + windowRoi.getPosition() );
         const fheroes2::Rect windowInterfacePresenceRoi( interfacePresenceRoi + windowRoi.getPosition() );
         const fheroes2::Rect windowCursorTypeRoi( cursorTypeRoi + windowRoi.getPosition() );
         const fheroes2::Rect windowScrollSpeedRoi( scrollSpeedRoi + windowRoi.getPosition() );
+        const fheroes2::Rect windowArmyEstimationModeRoi( armyEstimationModeRoi + windowRoi.getPosition() );
 
-        const auto drawOptions = [&conf, &windowInterfaceTypeRoi, &windowInterfacePresenceRoi, &windowCursorTypeRoi, &windowScrollSpeedRoi]() {
-            drawInterfaceType( windowInterfaceTypeRoi, conf.getInterfaceType() );
-            drawInterfacePresence( windowInterfacePresenceRoi );
-            drawCursorType( windowCursorTypeRoi, conf.isMonochromeCursorEnabled() );
-            drawScrollSpeed( windowScrollSpeedRoi, conf.ScrollSpeed() );
-        };
+        Settings & conf = Settings::Get();
+
+        const auto drawOptions
+            = [&conf, &windowInterfaceTypeRoi, &windowInterfacePresenceRoi, &windowCursorTypeRoi, &windowScrollSpeedRoi, &windowArmyEstimationModeRoi]() {
+                  drawInterfaceType( windowInterfaceTypeRoi, conf.getInterfaceType(), fheroes2::UiOptionTextWidth::TWO_ELEMENTS_ROW );
+                  drawInterfacePresence( windowInterfacePresenceRoi );
+                  drawCursorType( windowCursorTypeRoi, conf.isMonochromeCursorEnabled(), fheroes2::UiOptionTextWidth::TWO_ELEMENTS_ROW );
+                  drawScrollSpeed( windowScrollSpeedRoi, conf.ScrollSpeed() );
+                  drawArmyNumberEstimationOption( windowArmyEstimationModeRoi );
+              };
 
         drawOptions();
 
-        const fheroes2::Point buttonOffset( 112 + windowRoi.x, 252 + windowRoi.y );
-        fheroes2::Button buttonOk( buttonOffset.x, buttonOffset.y, isEvilInterface ? ICN::BUTTON_SMALL_OKAY_EVIL : ICN::BUTTON_SMALL_OKAY_GOOD, 0, 1 );
+        const bool isEvilInterface = conf.isEvilInterfaceEnabled();
 
-        buttonOk.draw();
+        fheroes2::Button buttonOk;
+        const int buttonOkIcnId = isEvilInterface ? ICN::BUTTON_SMALL_OKAY_EVIL : ICN::BUTTON_SMALL_OKAY_GOOD;
+        background.renderButton( buttonOk, buttonOkIcnId, 0, 1, { 0, 5 }, fheroes2::StandardWindow::Padding::BOTTOM_CENTER );
 
         const auto refreshWindow = [&drawOptions, &emptyDialogRestorer, &buttonOk, &display]() {
             emptyDialogRestorer.restore();
@@ -146,6 +149,9 @@ namespace
             if ( le.MouseClickLeft( windowCursorTypeRoi ) ) {
                 return SelectedWindow::CursorType;
             }
+            if ( le.MouseClickLeft( windowArmyEstimationModeRoi ) ) {
+                return SelectedWindow::ArmyEstimationMode;
+            }
 
             if ( le.MouseClickLeft( windowScrollSpeedRoi ) ) {
                 saveConfiguration = true;
@@ -178,9 +184,15 @@ namespace
             else if ( le.isMouseRightButtonPressedInArea( windowCursorTypeRoi ) ) {
                 fheroes2::showStandardTextMessage( _( "Mouse Cursor" ), _( "Toggle colored cursor on or off. This is only an aesthetic choice." ), 0 );
             }
-            if ( le.isMouseRightButtonPressedInArea( windowScrollSpeedRoi ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( windowScrollSpeedRoi ) ) {
                 fheroes2::showStandardTextMessage( _( "Scroll Speed" ), _( "Sets the speed at which you scroll the window." ), 0 );
             }
+            else if ( le.isMouseRightButtonPressedInArea( windowArmyEstimationModeRoi ) ) {
+                fheroes2::showStandardTextMessage(
+                    _( "Army Estimation" ),
+                    _( "Toggle how army sizes are displayed when right-clicking armies on the adventure map. \n\nCanonical: Army sizes are shown as descriptive text (e.g. \"Few\").\n\nNumeric: Army sizes are shown as numeric ranges (e.g. \"1-4\")." ),
+                    0 );
+            }
             else if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Okay" ), _( "Exit this menu." ), 0 );
             }
@@ -217,15 +229,7 @@ namespace fheroes2
                 windowType = showConfigurationWindow( saveConfiguration );
                 break;
             case SelectedWindow::InterfaceType:
-                if ( conf.getInterfaceType() == InterfaceType::DYNAMIC ) {
-                    conf.setInterfaceType( InterfaceType::GOOD );
-                }
-                else if ( conf.getInterfaceType() == InterfaceType::GOOD ) {
-                    conf.setInterfaceType( InterfaceType::EVIL );
-                }
-                else {
-                    conf.setInterfaceType( InterfaceType::DYNAMIC );
-                }
+                conf.switchToNextInterfaceType();
                 updateUI();
                 saveConfiguration = true;
 
@@ -242,6 +246,12 @@ namespace fheroes2
                 conf.setMonochromeCursor( !conf.isMonochromeCursorEnabled() );
                 saveConfiguration = true;
 
+                windowType = SelectedWindow::Configuration;
+                break;
+            case SelectedWindow::ArmyEstimationMode:
+                conf.setNumericArmyEstimationView( !conf.isArmyEstimationViewNumeric() );
+                saveConfiguration = true;
+
                 windowType = SelectedWindow::Configuration;
                 break;
             default:
