@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2021 - 2024                                             *
+ *   Copyright (C) 2021 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -24,8 +24,10 @@
 #include <cassert>
 #include <cstddef>
 #include <cstdint>
+#include <map>
 #include <memory>
 #include <string>
+#include <utility>
 
 #include "agg_image.h"
 #include "cursor.h"
@@ -43,7 +45,6 @@
 #include "ui_button.h"
 #include "ui_dialog.h"
 #include "ui_language.h"
-#include "ui_scrollbar.h"
 #include "ui_text.h"
 #include "ui_window.h"
 
@@ -54,7 +55,7 @@ namespace
     const int32_t scrollBarAreaWidth = 48;
     const int32_t paddingLeftSide = 24;
 
-    class LanguageList : public Interface::ListBox<fheroes2::SupportedLanguage>
+    class LanguageList final : public Interface::ListBox<fheroes2::SupportedLanguage>
     {
     public:
         using Interface::ListBox<fheroes2::SupportedLanguage>::ActionListSingleClick;
@@ -70,10 +71,9 @@ namespace
 
         void RedrawItem( const fheroes2::SupportedLanguage & language, int32_t offsetX, int32_t offsetY, bool isSelected ) override
         {
-            fheroes2::LanguageSwitcher languageSwitcher( language );
-            const fheroes2::Text languageName( fheroes2::getLanguageName( language ),
-                                               isSelected ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() );
-            languageName.draw( ( textAreaWidth - languageName.width() ) / 2 + offsetX, offsetY, fheroes2::Display::instance() );
+            // This method is supposed to do rendering but since the language resource generation is too performance extensive,
+            // we cache the information needed to render here and perform the actual rendering in the Redraw() method.
+            _languagesRenderingInfo[fheroes2::getCodePage( language )].emplace_back( language, offsetX, offsetY, isSelected );
         }
 
         void RedrawBackground( const fheroes2::Point & /* unused */ ) override
@@ -111,28 +111,73 @@ namespace
             return _isDoubleClicked;
         }
 
-        int getCurrentId() const
-        {
-            return _currentId;
-        }
-
         void initListBackgroundRestorer( fheroes2::Rect roi )
         {
             _listBackground = std::make_unique<fheroes2::ImageRestorer>( fheroes2::Display::instance(), roi.x, roi.y, roi.width, roi.height );
         }
 
-        void updateScrollBarImage()
+        void Redraw() override
         {
-            const int32_t scrollBarWidth = _scrollbar.width();
+            // Do not call this method!
+            // Use the overloaded method below!
+            assert( 0 );
+        }
 
-            setScrollBarImage( fheroes2::generateScrollbarSlider( _scrollbar, false, _scrollbar.getArea().height, VisibleItemCount(), _size(),
-                                                                  { 0, 0, scrollBarWidth, 8 }, { 0, 7, scrollBarWidth, 8 } ) );
-            _scrollbar.moveToIndex( _topId );
+        void Redraw( const fheroes2::SupportedLanguage mainLanguage )
+        {
+            // Since we are setting languages for each item to be rendered
+            // we cannot afford to switch the language back to the current one using fheroes2::LanguageSwitcher.
+            // This is why we set language switcher here and then initiate real rendering.
+            const fheroes2::LanguageSwitcher languageSwitcher( mainLanguage );
+
+            // Rendering of items in the base class is strictly in the top-to-bottom order.
+            // However, this class cannot afford doing the same due to extensive calculations
+            // for the language switching operation.
+            // This is why we first check the location of each language,
+            // then group them by code page,
+            // and only afterward render all languages, minimizing the number of resource switching operations.
+            _languagesRenderingInfo.clear();
+
+            Interface::ListBox<fheroes2::SupportedLanguage>::Redraw();
+
+            // Now we can do the real rendering.
+            fheroes2::Display & display = fheroes2::Display::instance();
+
+            for ( const auto & [codePage, languages] : _languagesRenderingInfo ) {
+                for ( const auto & info : languages ) {
+                    Settings::Get().setGameLanguage( fheroes2::getLanguageAbbreviation( info.language ) );
+
+                    const fheroes2::Text languageName( fheroes2::getLanguageName( info.language ),
+                                                       info.isSelected ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() );
+                    languageName.draw( ( textAreaWidth - languageName.width() ) / 2 + info.offset.x, info.offset.y, display );
+                }
+            }
         }
 
     private:
+        struct LanguageInfo
+        {
+            LanguageInfo() = delete;
+
+            LanguageInfo( const fheroes2::SupportedLanguage language_, int32_t offsetX, int32_t offsetY, bool isSelected_ )
+                : language( language_ )
+                , offset( offsetX, offsetY )
+                , isSelected( isSelected_ )
+            {
+                // Do nothing.
+            }
+
+            fheroes2::SupportedLanguage language{ fheroes2::SupportedLanguage::English };
+
+            fheroes2::Point offset;
+
+            bool isSelected{ false };
+        };
+
         bool _isDoubleClicked;
         std::unique_ptr<fheroes2::ImageRestorer> _listBackground;
+
+        std::map<fheroes2::CodePage, std::vector<LanguageInfo>> _languagesRenderingInfo;
     };
 
     void redrawDialogInfo( const fheroes2::Rect & listRoi, const fheroes2::SupportedLanguage language, const bool isGameLanguage )
@@ -164,7 +209,7 @@ namespace
         // If we don't have many languages, we reduce the maximum dialog height,
         // but not less than enough for 11 elements.
         // We also limit the maximum list height to 22 lines.
-        const int32_t maxDialogHeight = fheroes2::getFontHeight( fheroes2::FontSize::NORMAL ) * std::clamp( static_cast<int32_t>( languages.size() ), 11, 22 )
+        const int32_t maxDialogHeight = fheroes2::getFontHeight( fheroes2::FontSize::NORMAL ) * std::clamp<int32_t>( static_cast<int32_t>( languages.size() ), 11, 22 )
                                         + listAreaOffsetY + listAreaHeightDeduction + listHeightDeduction;
 
         fheroes2::Display & display = fheroes2::Display::instance();
@@ -191,15 +236,16 @@ namespace
         listBox.initListBackgroundRestorer( listRoi );
 
         Settings & conf = Settings::Get();
-        const bool isEvilInterface = conf.isEvilInterfaceEnabled();
 
         // Prepare OKAY and CANCEL buttons and render their shadows.
         fheroes2::Button buttonOk;
         fheroes2::Button buttonCancel;
-        background.renderOkayCancelButtons( buttonOk, buttonCancel, isEvilInterface );
+        background.renderOkayCancelButtons( buttonOk, buttonCancel );
 
         listBox.SetAreaItems( { listRoi.x, listRoi.y + 3, listRoi.width - 3, listRoi.height - 4 } );
 
+        const bool isEvilInterface = conf.isEvilInterfaceEnabled();
+
         int32_t scrollbarOffsetX = roi.x + roi.width - 35;
         background.renderScrollbarBackground( { scrollbarOffsetX, listRoi.y, listRoi.width, listRoi.height }, isEvilInterface );
 
@@ -223,7 +269,7 @@ namespace
             }
         }
 
-        listBox.Redraw();
+        listBox.Redraw( chosenLanguage );
 
         redrawDialogInfo( listRoi, chosenLanguage, isGameLanguage );
 
@@ -232,10 +278,10 @@ namespace
         LocalEvent & le = LocalEvent::Get();
         while ( le.HandleEvents() ) {
             if ( buttonOk.isEnabled() ) {
-                buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+                buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
             }
 
-            buttonCancel.drawOnState( le.isMouseLeftButtonPressedInArea( buttonCancel.area() ) );
+            buttonCancel.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonCancel.area() ) );
 
             if ( le.isMouseRightButtonPressedInArea( listRoi ) ) {
                 continue;
@@ -278,11 +324,11 @@ namespace
                     selectedLangBackground.restore();
                     redrawDialogInfo( listRoi, chosenLanguage, isGameLanguage );
                     buttonsBackground.restore();
-                    background.renderOkayCancelButtons( buttonOk, buttonCancel, isEvilInterface );
+                    background.renderOkayCancelButtons( buttonOk, buttonCancel );
                 }
             }
 
-            listBox.Redraw();
+            listBox.Redraw( chosenLanguage );
             display.render( roi );
         }
 
