@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -71,7 +71,7 @@ namespace
         const fheroes2::Sprite & originalBackground = fheroes2::AGG::GetICN( ICN::RECRBKG, 0 );
         const fheroes2::Sprite & recruitWindowTitle = fheroes2::AGG::GetICN( ICN::BLDGXTRA, 0 );
 
-        fheroes2::Image recruitWindow( 132, 67 );
+        fheroes2::Sprite recruitWindow( 132, 67 );
         // Reset the transparent layer.
         recruitWindow.reset();
 
@@ -172,7 +172,7 @@ namespace
 
         const fheroes2::Sprite & smon = fheroes2::AGG::GetICN( monster.GetMonsterSprite(), monsterInfo.animationFrames[Bin_Info::MonsterAnimInfo::STATIC][0] );
         dst_pt.x = pos.x + 64 + smon.x() - ( monster.isWide() ? 22 : 0 );
-        const int32_t monsterExtraOffsetY = std::max( 0, smon.height() - 96 );
+        const int32_t monsterExtraOffsetY = std::max<int32_t>( 0, smon.height() - 96 );
         dst_pt.y = pos.y + 119 - smon.height() + monsterExtraOffsetY;
 
         if ( monsterId == Monster::CHAMPION ) {
@@ -212,7 +212,7 @@ namespace
         str = _( "Available: %{count}" );
         StringReplace( str, "%{count}", available );
         text.set( std::move( str ), fheroes2::FontType::smallWhite() );
-        text.draw( pos.x + 64 - text.width() / 2, pos.y + 120 + std::max( monsterExtraOffsetY, 2 ), display );
+        text.draw( pos.x + 64 - text.width() / 2, pos.y + 120 + std::max<int32_t>( monsterExtraOffsetY, 2 ), display );
 
         if ( showTotalSum ) {
             text.set( _( "Number to buy:" ), fheroes2::FontType::smallWhite() );
@@ -282,7 +282,7 @@ Troop Dialog::RecruitMonster( const Monster & monster0, const uint32_t available
     const fheroes2::Sprite & originalBackground = fheroes2::AGG::GetICN( ICN::RECRBKG, 0 );
 
     // Render the recruit count background from original recruit dialog ICN.
-    fheroes2::Image background( 68, 19 );
+    fheroes2::Sprite background( 68, 19 );
     fheroes2::Copy( originalBackground, 134, 159, background, 0, 0, background.width(), background.height() );
 
     const fheroes2::Rect recruitCountInputArea( dialogOffset.x + 118, dialogOffset.y + 147, background.width(), background.height() );
