@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -232,7 +232,7 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
     // setup cursor
     const CursorRestorer cursorRestorer( true, Cursor::POINTER );
 
-    result.reserve( charLimit == 0 ? 48 : charLimit );
+    result.reserve( ( charLimit == 0 ) ? 48 : charLimit );
     size_t charInsertPos = result.size();
 
     const bool hasTitle = !title.empty();
@@ -358,14 +358,36 @@ bool Dialog::inputString( const fheroes2::TextBase & title, const fheroes2::Text
             charInsertPos = result.size();
             redraw = true;
         }
-        else if ( le.isAnyKeyPressed() && ( charLimit == 0 || charLimit > result.size() || le.getPressedKeyValue() == fheroes2::Key::KEY_BACKSPACE ) ) {
+        else if ( le.isAnyKeyPressed()
+                  && ( charLimit == 0 || charLimit > result.size() || le.getPressedKeyValue() == fheroes2::Key::KEY_BACKSPACE
+                       || le.getPressedKeyValue() == fheroes2::Key::KEY_UP || le.getPressedKeyValue() == fheroes2::Key::KEY_DOWN
+                       || le.getPressedKeyValue() == fheroes2::Key::KEY_DELETE || le.getPressedKeyValue() == fheroes2::Key::KEY_LEFT
+                       || le.getPressedKeyValue() == fheroes2::Key::KEY_RIGHT ) ) {
             // Handle new line input for multi-line texts only.
             if ( isMultiLine && le.getPressedKeyValue() == fheroes2::Key::KEY_ENTER ) {
-                result.insert( charInsertPos, 1, '\n' );
-                ++charInsertPos;
+                // We should verify the height of the text before allowing to enter one more line.
+                std::string tmp = result;
+                tmp.insert( charInsertPos, 1, '\n' );
+                if ( textInput.height( tmp ) <= textInputArea.height ) {
+                    result = std::move( tmp );
+                    ++charInsertPos;
+                }
+            }
+            else if ( isMultiLine && ( le.getPressedKeyValue() == fheroes2::Key::KEY_UP || le.getPressedKeyValue() == fheroes2::Key::KEY_DOWN ) ) {
+                const size_t newPos = textInput.getCursorPositionInAdjacentLine( charInsertPos, le.getPressedKeyValue() == fheroes2::Key::KEY_UP );
+                if ( newPos == charInsertPos ) {
+                    continue;
+                }
+                charInsertPos = newPos;
             }
             else {
-                charInsertPos = InsertKeySym( result, charInsertPos, le.getPressedKeyValue(), LocalEvent::getCurrentKeyModifiers() );
+                // We should verify the height of the text before allowing to enter one more line.
+                std::string tmp = result;
+                const size_t tempCharInsertPos = InsertKeySym( tmp, charInsertPos, le.getPressedKeyValue(), LocalEvent::getCurrentKeyModifiers() );
+                if ( textInput.height( tmp ) <= textInputArea.height ) {
+                    result = std::move( tmp );
+                    charInsertPos = tempCharInsertPos;
+                }
             }
             redraw = true;
         }
@@ -434,7 +456,7 @@ int Dialog::ArmySplitTroop( const int32_t freeSlots, const int32_t redistributeM
     // setup cursor
     const CursorRestorer cursorRestorer( true, Cursor::POINTER );
 
-    const int32_t redistributeMin = std::min( 1, redistributeMax );
+    const int32_t redistributeMin = std::min<int32_t>( 1, redistributeMax );
     const int spacer = 10;
     const fheroes2::Text header( troopName, fheroes2::FontType::normalYellow() );
     const int32_t headerHeight = header.height() + 6;
