@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2011 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -99,12 +99,12 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             renderItem( getImage( index ), Monster{ index }.GetName(), { dstx, dsty }, 45 / 2, 50, _offsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             const Monster monster( index );
             if ( !monster.isValid() ) {
@@ -131,15 +131,15 @@ namespace
     public:
         using SelectEnumMonster::SelectEnumMonster;
 
-        using Interface::ListBox<int>::ActionListDoubleClick;
-        using Interface::ListBox<int>::ActionListSingleClick;
-        using Interface::ListBox<int>::ActionListPressRight;
+        using Interface::ListBox<int32_t>::ActionListDoubleClick;
+        using Interface::ListBox<int32_t>::ActionListSingleClick;
+        using Interface::ListBox<int32_t>::ActionListPressRight;
 
-        void setup( std::vector<int> allowed, const std::vector<int> & selected )
+        void setup( std::vector<int32_t> allowed, const std::vector<int32_t> & selected )
         {
             _ids = std::move( allowed );
 
-            for ( const int id : selected ) {
+            for ( const int32_t id : selected ) {
                 if ( std::find( _ids.begin(), _ids.end(), id ) != _ids.end() ) {
                     _selected.emplace( id );
                 }
@@ -151,22 +151,22 @@ namespace
 
             SetListContent( _ids );
             // For multi-selection we don't have any current item.
-            SetCurrent( 0 );
+            SetCurrent( static_cast<int32_t>( 0 ) );
 
             enableToggleButtons();
         }
 
-        std::vector<int> getSelected() const
+        std::vector<int32_t> getSelected() const
         {
             return { _selected.begin(), _selected.end() };
         }
 
-        void ActionListSingleClick( int & id ) override
+        void ActionListSingleClick( int32_t & id ) override
         {
             updateStatus( id );
         }
 
-        void ActionListDoubleClick( int & id ) override
+        void ActionListDoubleClick( int32_t & id ) override
         {
             updateStatus( id );
         }
@@ -176,7 +176,7 @@ namespace
         {
             _selected = {};
 
-            for ( const int id : _ids ) {
+            for ( const int32_t id : _ids ) {
                 _selected.emplace( id );
             }
 
@@ -191,10 +191,10 @@ namespace
         }
 
     private:
-        std::vector<int> _ids;
-        std::set<int> _selected;
+        std::vector<int32_t> _ids;
+        std::set<int32_t> _selected;
 
-        void updateStatus( const int id )
+        void updateStatus( const int32_t id )
         {
             assert( std::find( _ids.begin(), _ids.end(), id ) != _ids.end() );
 
@@ -238,14 +238,14 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             const fheroes2::Sprite & port = Heroes::GetPortrait( index, PORT_SMALL );
 
             renderItem( port, Heroes::getDefaultName( index ), { dstx, dsty }, 45 / 2, 50, _offsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             Dialog::QuickInfo( *world.GetHeroes( index ) );
         }
@@ -265,7 +265,7 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             const Artifact art( index );
             const fheroes2::Sprite & artifactSprite = fheroes2::AGG::GetICN( ICN::ARTFX, art.IndexSprite32() );
@@ -273,7 +273,7 @@ namespace
             renderItem( artifactSprite, art.GetName(), { dstx, dsty }, 45 / 2, 50, _offsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             fheroes2::ArtifactDialogElement( Artifact( index ) ).showPopup( Dialog::ZERO );
         }
@@ -293,7 +293,7 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             const Spell spell( index );
             const fheroes2::Sprite & spellSprite = fheroes2::AGG::GetICN( ICN::SPELLS, spell.IndexSprite() );
@@ -301,7 +301,7 @@ namespace
             renderItem( spellSprite, spell.GetName(), { dstx, dsty }, 75 / 2, 80, _offsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             fheroes2::SpellDialogElement( Spell( index ), nullptr ).showPopup( Dialog::ZERO );
         }
@@ -331,7 +331,7 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             const Skill::Secondary skill( getSkillFromListIndex( index ), getLevelFromListIndex( index ) );
             const fheroes2::Sprite & skillSprite = fheroes2::AGG::GetICN( ICN::MINISS, skill.GetIndexSprite2() );
@@ -339,7 +339,7 @@ namespace
             renderItem( skillSprite, skill.GetName(), { dstx, dsty }, 45 / 2, 50, _offsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             fheroes2::SecondarySkillDialogElement( Skill::Secondary( getSkillFromListIndex( index ), getLevelFromListIndex( index ) ), Heroes() )
                 .showPopup( Dialog::ZERO );
@@ -361,7 +361,7 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & index, int32_t dstx, int32_t dsty, bool current ) override
+        void RedrawItem( const int32_t & index, int32_t dstx, int32_t dsty, bool current ) override
         {
             const Castle * castle = world.getCastleEntrance( Maps::GetPoint( index ) );
 
@@ -373,7 +373,7 @@ namespace
             renderItem( castleIcon, castle->GetName(), { dstx, dsty }, 35, 75, itemsOffsetY / 2, current );
         }
 
-        void ActionListPressRight( int & index ) override
+        void ActionListPressRight( int32_t & index ) override
         {
             Dialog::QuickInfoWithIndicationOnRadar( *world.getCastleEntrance( Maps::GetPoint( index ) ), getBackgroundArea() );
         }
@@ -401,7 +401,7 @@ namespace
 
         using Dialog::ItemSelectionWindow::ActionListPressRight;
 
-        void RedrawItem( const int & objectId, int32_t posX, int32_t posY, bool isSelected ) override
+        void RedrawItem( const int32_t & objectId, int32_t posX, int32_t posY, bool isSelected ) override
         {
             // If this assertion blows up then you are setting different number of items.
             assert( objectId >= 0 && objectId < static_cast<int>( _objectInfo.size() ) );
@@ -422,7 +422,7 @@ namespace
             }
         }
 
-        void ActionListPressRight( int & objectId ) override
+        void ActionListPressRight( int32_t & objectId ) override
         {
             // If this assertion blows up then you are setting different number of items.
             assert( objectId >= 0 && objectId < static_cast<int>( _objectInfo.size() ) );
@@ -668,18 +668,18 @@ namespace
         }
     };
 
-    int selectObjectType( const int objectType, const size_t objectCount, ObjectTypeSelection & objectSelection )
+    int selectObjectType( const int32_t objectType, const size_t objectCount, ObjectTypeSelection & objectSelection )
     {
         if ( objectCount == 0 ) {
             fheroes2::showStandardTextMessage( _( "Warning" ), _( "There is nothing to select from." ), Dialog::OK );
             return -1;
         }
 
-        std::vector<int> objects( objectCount, 0 );
+        std::vector<int32_t> objects( objectCount, 0 );
         std::iota( objects.begin(), objects.end(), 0 );
         objectSelection.SetListContent( objects );
 
-        objectSelection.SetCurrent( std::max( objectType, 0 ) );
+        objectSelection.SetCurrent( std::max<int32_t>( objectType, 0 ) );
 
         const int32_t result = objectSelection.selectItemsEventProcessing();
         return result == Dialog::OK && objectSelection.IsValid() ? objectSelection.GetCurrent() : -1;
@@ -986,9 +986,9 @@ namespace Dialog
     }
 }
 
-Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int skillId /* = Skill::Secondary::UNKNOWN */ )
+Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int32_t skillId /* = Skill::Secondary::UNKNOWN */ )
 {
-    std::vector<int> skills;
+    std::vector<int32_t> skills;
     skills.reserve( static_cast<size_t>( Skill::numOfSecondarySkills ) * 3 );
 
     for ( int i = 0; i < Skill::numOfSecondarySkills * 3; ++i ) {
@@ -1019,10 +1019,10 @@ Skill::Secondary Dialog::selectSecondarySkill( const Heroes & hero, const int sk
     return {};
 }
 
-Spell Dialog::selectSpell( const int spellId, const bool includeRandomSpells, const std::set<int32_t> & excludeSpellsList /* = {} */,
+Spell Dialog::selectSpell( const int32_t spellId, const bool includeRandomSpells, const std::set<int32_t> & excludeSpellsList /* = {} */,
                            const int32_t spellsLevel /* = -1 */ )
 {
-    std::vector<int> spells = Spell::getAllSpellIdsSuitableForSpellBook( spellsLevel, excludeSpellsList );
+    std::vector<int32_t> spells = Spell::getAllSpellIdsSuitableForSpellBook( spellsLevel, excludeSpellsList );
 
     if ( includeRandomSpells ) {
         // We add random spell items to the end of the list.
@@ -1053,9 +1053,9 @@ Spell Dialog::selectSpell( const int spellId, const bool includeRandomSpells, co
     return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Spell::NONE };
 }
 
-Artifact Dialog::selectArtifact( const int artifactId, const bool isForVictoryConditions )
+Artifact Dialog::selectArtifact( const int32_t artifactId, const bool isForVictoryConditions )
 {
-    std::vector<int> artifacts;
+    std::vector<int32_t> artifacts;
     artifacts.reserve( Artifact::ARTIFACT_COUNT - 2 );
 
     const GameVersion version = Settings::Get().getCurrentMapInfo().version;
@@ -1107,9 +1107,9 @@ Artifact Dialog::selectArtifact( const int artifactId, const bool isForVictoryCo
     return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Artifact::UNKNOWN };
 }
 
-Monster Dialog::selectMonster( const int monsterId )
+Monster Dialog::selectMonster( const int32_t monsterId )
 {
-    std::vector<int> monsters( Monster::MONSTER_COUNT - 1, Monster::UNKNOWN );
+    std::vector<int32_t> monsters( Monster::MONSTER_COUNT - 1, Monster::UNKNOWN );
 
     // Skip Monster::UNKNOWN and start from the next one.
     std::iota( monsters.begin(), monsters.end(), Monster::UNKNOWN + 1 );
@@ -1132,12 +1132,12 @@ Monster Dialog::selectMonster( const int monsterId )
     return { ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Monster::UNKNOWN };
 }
 
-int Dialog::selectHeroes( const int heroId /* = Heroes::UNKNOWN */ )
+int Dialog::selectHeroes( const int32_t heroId /* = Heroes::UNKNOWN */ )
 {
     const GameVersion version = Settings::Get().getCurrentMapInfo().version;
     const bool isPoLHeroesAllowed = ( version == GameVersion::PRICE_OF_LOYALTY || version == GameVersion::RESURRECTION );
 
-    std::vector<int> heroes( static_cast<int>( isPoLHeroesAllowed ? Heroes::JARKONAS : Heroes::BRAX ), Heroes::UNKNOWN );
+    std::vector<int32_t> heroes( static_cast<int>( isPoLHeroesAllowed ? Heroes::JARKONAS : Heroes::BRAX ), Heroes::UNKNOWN );
 
     if ( heroes.empty() ) {
         fheroes2::showStandardTextMessage( _( "Warning" ), _( "There are no heroes to select from." ), Dialog::OK );
@@ -1158,7 +1158,7 @@ int Dialog::selectHeroes( const int heroId /* = Heroes::UNKNOWN */ )
     return ( result == Dialog::OK && listbox.IsValid() ) ? listbox.GetCurrent() : Heroes::UNKNOWN;
 }
 
-void Dialog::multiSelectMonsters( std::vector<int> allowed, std::vector<int> & selected )
+void Dialog::multiSelectMonsters( std::vector<int32_t> allowed, std::vector<int32_t> & selected )
 {
     MultiMonsterSelection monsterList( { 380, fheroes2::Display::instance().height() - dialogHeightDeduction }, _( "Select Monsters:" ) );
     monsterList.setup( std::move( allowed ), selected );
@@ -1397,7 +1397,7 @@ int Dialog::selectLandscapeOceanObjectType( const int objectType )
     return selectObjectType( objectType, objectInfo.size(), listbox );
 }
 
-void Dialog::selectTownType( int & type, int & color )
+void Dialog::selectTownType( int32_t & type, int32_t & color )
 {
     fheroes2::Display & display = fheroes2::Display::instance();
     fheroes2::StandardWindow background( 520, 370, true, display );
@@ -1818,8 +1818,14 @@ int32_t Dialog::selectMineType( const int32_t type )
     auto updateListContent = [&listbox, &objectInfo, &allObjectInfo, &objectInfoIndexes, &listIcnId, &getObjectTypeByResource]( const uint32_t resourceId ) {
         const MP2::MapObjectType objectType = getObjectTypeByResource( resourceId );
 
-        // Mine appearance is the same for different mine resources so we keep the selection.
-        const bool keepSelection = ( objectType == MP2::OBJ_MINE ) && !objectInfo.empty() && ( objectInfo.front().objectType == MP2::OBJ_MINE );
+        // Mine and Abandoned Mine appearance is the same for different mine resources so we keep the selection.
+        bool keepSelection = !objectInfo.empty();
+        if ( keepSelection ) {
+            const bool isMineObject = ( objectType == MP2::OBJ_MINE ) || ( objectType == MP2::OBJ_ABANDONED_MINE );
+            const bool isMineObjectPart = ( objectInfo.front().objectType == MP2::OBJ_MINE ) || ( objectInfo.front().objectType == MP2::OBJ_ABANDONED_MINE );
+            keepSelection = isMineObject && isMineObjectPart;
+        }
+
         const int selectedId = keepSelection ? listbox.getCurrentId() : 0;
         const int topId = keepSelection ? listbox.getTopId() : 0;
 
