@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2023 - 2025                                             *
+ *   Copyright (C) 2023 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -37,6 +37,7 @@
 #include "icn.h"
 #include "image.h"
 #include "interface_base.h"
+#include "interface_gamearea.h"
 #include "localevent.h"
 #include "maps_tiles.h"
 #include "monster.h"
@@ -213,29 +214,31 @@ namespace Interface
         _selectedLandscapeObjectType.fill( -1 );
         _selectedAdventureObjectType.fill( -1 );
         _selectedKingdomObjectType.fill( -1 );
+
+        updateUndoRedoButtonsStates( false, false );
     }
 
     fheroes2::Rect EditorPanel::getBrushArea() const
     {
         // Roads and streams are placed using only 1x1 brush.
-        if ( _selectedInstrument == Instrument::STREAM || _selectedInstrument == Instrument::ROAD || _selectedInstrument == Instrument::DETAIL
-             || _selectedInstrument == Instrument::MONSTERS ) {
+        if ( _selectedInstrument == Instrument::STREAM || _selectedInstrument == Instrument::ROAD || _selectedInstrument == Instrument::DETAIL ) {
             return { 0, 0, 1, 1 };
         }
 
+        if ( _selectedInstrument == Instrument::MONSTERS ) {
+            const int32_t objectType = getSelectedObjectType();
+            if ( objectType >= 0 ) {
+                return { 0, 0, 1, 1 };
+            }
+
+            return {};
+        }
+
         if ( _selectedInstrument == Instrument::LANDSCAPE_OBJECTS || _selectedInstrument == Instrument::ADVENTURE_OBJECTS
              || _selectedInstrument == Instrument::KINGDOM_OBJECTS ) {
             const int32_t objectType = getSelectedObjectType();
             if ( objectType >= 0 ) {
                 const Maps::ObjectGroup objectGroup = getSelectedObjectGroup();
-                if ( objectGroup == Maps::ObjectGroup::ADVENTURE_MINES ) {
-                    // For mine we need to decode the objectType.
-                    int32_t type = -1;
-                    int32_t color = -1;
-                    getMineObjectProperties( type, color );
-
-                    return getObjectOccupiedArea( objectGroup, type );
-                }
 
                 return getObjectOccupiedArea( objectGroup, objectType );
             }
@@ -374,7 +377,7 @@ namespace Interface
 
         // Brush size buttons position. Shown on the terrain and erasure instrument panels.
         offsetX = displayX + 14;
-        int32_t offsetY = displayY + std::min( instrumentPanelHeight - 27, 135 );
+        int32_t offsetY = displayY + std::min<int32_t>( instrumentPanelHeight - 27, 135 );
         for ( size_t i = 0; i < _brushSizeButtonsRect.size(); ++i ) {
             _brushSizeButtons[i].setPosition( offsetX, offsetY );
             _brushSizeButtonsRect[i] = _brushSizeButtons[i].area();
@@ -799,11 +802,11 @@ namespace Interface
             return;
         case Instrument::LANDSCAPE_OBJECTS:
             switch ( _selectedLandscapeObject ) {
+            case LandscapeObjectBrush::LANDSCAPE_MISC:
             case LandscapeObjectBrush::MOUNTAINS:
             case LandscapeObjectBrush::ROCKS:
             case LandscapeObjectBrush::TREES:
             case LandscapeObjectBrush::WATER_OBJECTS:
-            case LandscapeObjectBrush::LANDSCAPE_MISC:
                 _interface.setCursorUpdater(
                     [type = getSelectedObjectType(), group = getSelectedObjectGroup()]( const int32_t /*tileIndex*/ ) { setCustomCursor( group, type ); } );
                 return;
@@ -813,35 +816,16 @@ namespace Interface
             break;
         case Instrument::ADVENTURE_OBJECTS:
             switch ( _selectedAdventureObject ) {
+            case AdventureObjectBrush::ADVENTURE_MISC:
             case AdventureObjectBrush::ARTIFACTS:
             case AdventureObjectBrush::DWELLINGS:
+            case AdventureObjectBrush::MINES:
             case AdventureObjectBrush::POWER_UPS:
             case AdventureObjectBrush::TREASURES:
             case AdventureObjectBrush::WATER_ADVENTURE:
-            case AdventureObjectBrush::ADVENTURE_MISC:
                 _interface.setCursorUpdater(
                     [type = getSelectedObjectType(), group = getSelectedObjectGroup()]( const int32_t /*tileIndex*/ ) { setCustomCursor( group, type ); } );
                 return;
-            case AdventureObjectBrush::MINES:
-                _interface.setCursorUpdater( [this]( const int32_t /*tileIndex*/ ) {
-                    int32_t type = -1;
-                    int32_t color = -1;
-                    getMineObjectProperties( type, color );
-
-                    if ( type == -1 || color == -1 ) {
-                        // The object type is not set. We show the POINTER cursor for this case.
-                        Cursor::Get().SetThemes( Cursor::POINTER );
-                        return;
-                    }
-
-                    assert( Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES ).size() > static_cast<size_t>( type ) );
-
-                    // TODO: Implement a function to render also the owner flag after ownership selection is implemented.
-                    const fheroes2::Sprite & image = getObjectImage( Maps::ObjectGroup::ADVENTURE_MINES, type );
-
-                    Cursor::Get().setCustomImage( image, { image.x(), image.y() } );
-                } );
-                return;
             default:
                 break;
             }
@@ -894,6 +878,25 @@ namespace Interface
         _interface.setCursorUpdater( {} );
     }
 
+    void EditorPanel::updateUndoRedoButtonsStates( const bool isUndoAvailable, const bool isRedoAvailable )
+    {
+        auto updateButtonState = []( fheroes2::ButtonBase & button, const bool isAvailable ) {
+            if ( isAvailable ) {
+                if ( button.isDisabled() ) {
+                    button.enable();
+                    button.draw();
+                }
+            }
+            else if ( button.isEnabled() ) {
+                button.disable();
+                button.draw();
+            }
+        };
+
+        updateButtonState( _buttonUndo, isUndoAvailable );
+        updateButtonState( _buttonRedo, isRedoAvailable );
+    }
+
     fheroes2::GameMode EditorPanel::queueEventProcessing()
     {
         LocalEvent & le = LocalEvent::Get();
@@ -1092,15 +1095,7 @@ namespace Interface
                 return res;
             }
             if ( le.MouseClickLeft( _adventureObjectButtonsRect[AdventureObjectBrush::MINES] ) ) {
-                handleObjectMouseClick( [this]( const int32_t /* type */ ) {
-                    int32_t type = -1;
-                    int32_t color = -1;
-
-                    getMineObjectProperties( type, color );
-                    Dialog::selectMineType( type, color );
-
-                    return _generateMineObjectProperties( type, color );
-                } );
+                handleObjectMouseClick( Dialog::selectMineType );
                 return res;
             }
             if ( le.MouseClickLeft( _adventureObjectButtonsRect[AdventureObjectBrush::DWELLINGS] ) ) {
@@ -1185,23 +1180,21 @@ namespace Interface
             }
         }
 
-        _buttonMagnify.drawOnState( le.isMouseLeftButtonPressedInArea( _rectMagnify ) );
-        _buttonUndo.drawOnState( le.isMouseLeftButtonPressedInArea( _rectUndo ) );
-        _buttonRedo.drawOnState( le.isMouseLeftButtonPressedInArea( _rectRedo ) );
-        _buttonSpecs.drawOnState( le.isMouseLeftButtonPressedInArea( _rectSpecs ) );
-        _buttonFile.drawOnState( le.isMouseLeftButtonPressedInArea( _rectFile ) );
-        _buttonSystem.drawOnState( le.isMouseLeftButtonPressedInArea( _rectSystem ) );
+        _buttonMagnify.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectMagnify ) );
+        _buttonUndo.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectUndo ) );
+        _buttonRedo.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectRedo ) );
+        _buttonSpecs.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectSpecs ) );
+        _buttonFile.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectFile ) );
+        _buttonSystem.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _rectSystem ) );
 
         if ( le.MouseClickLeft( _rectMagnify ) ) {
             _interface.eventViewWorld();
         }
         else if ( _buttonUndo.isEnabled() && le.MouseClickLeft( _rectUndo ) ) {
             _interface.undoAction();
-            // TODO: Disable Undo button when there are no actions left to undo.
         }
         else if ( _buttonRedo.isEnabled() && le.MouseClickLeft( _rectRedo ) ) {
             _interface.redoAction();
-            // TODO: Disable Redo button when there are no actions left to redo.
         }
         else if ( le.MouseClickLeft( _rectSpecs ) ) {
             EditorInterface::Get().openMapSpecificationsDialog();
@@ -1287,7 +1280,9 @@ namespace Interface
             }
         }
         _setCursor();
-        _interface.updateCursor( 0 );
+
+        // The cursor must be set according to the position of mouse cursor.
+        _interface.updateCursor( _interface.getGameArea().GetValidTileIdFromPoint( LocalEvent::Get().getMouseCursorPos() ) );
     }
 
     void EditorPanel::getTownObjectProperties( int32_t & type, int32_t & color ) const
@@ -1316,32 +1311,4 @@ namespace Interface
 
         return color * static_cast<int32_t>( townObjects.size() ) + type;
     }
-
-    void EditorPanel::getMineObjectProperties( int32_t & type, int32_t & color ) const
-    {
-        const auto & mineObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES );
-        if ( mineObjects.empty() ) {
-            // How is it even possible?
-            assert( 0 );
-            type = -1;
-            color = -1;
-            return;
-        }
-
-        type = _selectedAdventureObjectType[AdventureObjectBrush::MINES] % static_cast<int32_t>( mineObjects.size() );
-        color = _selectedAdventureObjectType[AdventureObjectBrush::MINES] / static_cast<int32_t>( mineObjects.size() );
-    }
-
-    int32_t EditorPanel::_generateMineObjectProperties( const int32_t type, const int32_t color )
-    {
-        const auto & mineObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES );
-        if ( mineObjects.empty() ) {
-            // How is it even possible?
-            assert( 0 );
-            return -1;
-        }
-
-        const int32_t objectType = ( color * static_cast<int32_t>( mineObjects.size() ) + type );
-        return ( objectType < 0 ) ? -1 : objectType;
-    }
 }
