@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -34,6 +34,8 @@
 #include "translations.h"
 #include "world.h"
 
+enum class PlayerColor : uint8_t;
+
 std::string Difficulty::String( int difficulty )
 {
     switch ( difficulty ) {
@@ -242,6 +244,47 @@ bool Difficulty::allowAIToSplitWeakStacks( const int difficulty )
     return true;
 }
 
+int32_t Difficulty::getGuardianSpellMultiplier( const int difficulty )
+{
+    switch ( difficulty ) {
+    case Difficulty::EASY:
+        return 16;
+    case Difficulty::NORMAL:
+        return 14;
+    case Difficulty::HARD:
+        return 12;
+    case Difficulty::EXPERT:
+        return 10;
+    case Difficulty::IMPOSSIBLE:
+        return 8;
+    default:
+        // Did you add a new difficulty level? Add the logic above!
+        assert( 0 );
+        break;
+    }
+
+    return 20;
+}
+
+bool Difficulty::isObjectVisitInfoSharingAllowedForAI( const int difficulty )
+{
+    switch ( difficulty ) {
+    case Difficulty::EASY:
+    case Difficulty::NORMAL:
+        return false;
+    case Difficulty::HARD:
+    case Difficulty::EXPERT:
+    case Difficulty::IMPOSSIBLE:
+        return true;
+    default:
+        // Did you add a new difficulty level? Add the logic above!
+        assert( 0 );
+        break;
+    }
+
+    return false;
+}
+
 bool Difficulty::allowAIToDevelopCastlesOnDay( const int difficulty, const bool isCampaign, const uint32_t day )
 {
     switch ( difficulty ) {
@@ -266,3 +309,15 @@ bool Difficulty::allowAIToBuildCastleBuilding( const int difficulty, const bool
 
     return true;
 }
+
+bool Difficulty::isBasicAIBattleLogicApplicable( const int32_t difficulty, const bool isControlledByHuman )
+{
+    switch ( difficulty ) {
+    case Difficulty::EASY:
+        return isControlledByHuman;
+    default:
+        break;
+    }
+
+    return !isControlledByHuman;
+}
