@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -34,47 +34,52 @@ Cursor & Cursor::Get()
     return _cursor;
 }
 
-bool Cursor::SetThemes( int name, bool force )
+void Cursor::SetThemes( const int theme, const bool force /* = false */ )
 {
-    if ( force || _theme != name ) {
-        _theme = name;
-
-        // Video pointer cannot be properly rendered in black-white so we have to force to use color cursor.
-        int icnID = ( _monochromeCursorThemes && ( name != Cursor::POINTER_VIDEO ) ) ? ICN::MONO_CURSOR_ADVMBW : ICN::ADVMCO;
-        switch ( 0xF000 & name ) {
-        case 0x3000:
-            icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_SPELBW : ICN::SPELCO;
-            break;
-        case 0x2000:
-            icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_CMSSBW : ICN::CMSECO;
-            break;
-        case 0x4000:
-            icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_ADVENTURE_MAP : ICN::COLOR_CURSOR_ADVENTURE_MAP;
-            break;
-        default:
-            break;
-        }
-        const fheroes2::Sprite & spr = fheroes2::AGG::GetICN( icnID, 0xFF & name );
-        SetOffset( name, { ( spr.width() - spr.x() ) / 2, ( spr.height() - spr.y() ) / 2 } );
-        fheroes2::cursor().update( spr, -_offset.x, -_offset.y );
-
-        // Apply new offset.
-        const fheroes2::Point & currentPos = LocalEvent::Get().GetMouseCursor();
-        Move( currentPos.x, currentPos.y );
-        return true;
+    if ( _theme == theme && !force ) {
+        return;
     }
 
-    return false;
+    _theme = theme;
+
+    // Video pointer cannot be properly rendered in black-white so we have to force to use color cursor.
+    int icnID = ( _monochromeCursorThemes && ( theme != Cursor::POINTER_VIDEO ) ) ? ICN::MONO_CURSOR_ADVMBW : ICN::ADVMCO;
+    switch ( 0xF000 & theme ) {
+    case 0x3000:
+        icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_SPELBW : ICN::SPELCO;
+        break;
+    case 0x2000:
+        icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_CMSSBW : ICN::CMSECO;
+        break;
+    case 0x4000:
+        icnID = _monochromeCursorThemes ? ICN::MONO_CURSOR_ADVENTURE_MAP : ICN::COLOR_CURSOR_ADVENTURE_MAP;
+        break;
+    default:
+        break;
+    }
+    const fheroes2::Sprite & spr = fheroes2::AGG::GetICN( icnID, 0xFF & theme );
+    SetOffset( theme, { ( spr.width() - spr.x() ) / 2, ( spr.height() - spr.y() ) / 2 } );
+
+    fheroes2::Cursor & cursor = fheroes2::cursor();
+    cursor.update( spr, -_offset.x, -_offset.y );
+    // Force the scroll cursor to stay in screen boundaries.
+    cursor.keepInScreenArea( theme >= SCROLL_TOP && theme <= SCROLL_TOPLEFT );
+
+    // Apply new offset.
+    const fheroes2::Point & currentPos = LocalEvent::Get().getMouseCursorPos();
+    Move( currentPos.x, currentPos.y );
 }
 
 void Cursor::setCustomImage( const fheroes2::Image & image, const fheroes2::Point & offset )
 {
     _theme = NONE;
 
-    fheroes2::cursor().update( image, -offset.x, -offset.y );
+    fheroes2::Cursor & cursor = fheroes2::cursor();
+    cursor.update( image, -offset.x, -offset.y );
+    cursor.keepInScreenArea( false );
 
     // Immediately apply new mouse offset.
-    const fheroes2::Point & currentPos = LocalEvent::Get().GetMouseCursor();
+    const fheroes2::Point & currentPos = LocalEvent::Get().getMouseCursorPos();
     _offset = offset;
 
     Move( currentPos.x, currentPos.y );
@@ -97,7 +102,7 @@ void Cursor::Move( int32_t x, int32_t y ) const
     fheroes2::cursor().setPosition( x + _offset.x, y + _offset.y );
 }
 
-void Cursor::SetOffset( int name, const fheroes2::Point & defaultOffset )
+void Cursor::SetOffset( const int name, const fheroes2::Point & defaultOffset )
 {
     switch ( name ) {
     case Cursor::POINTER:
@@ -144,6 +149,14 @@ void Cursor::SetOffset( int name, const fheroes2::Point & defaultOffset )
         _offset = { -20, -5 };
         break;
 
+    case Cursor::SWORD_TOP:
+        _offset = { -defaultOffset.x, -5 };
+        break;
+
+    case Cursor::SWORD_BOTTOM:
+        _offset = { -defaultOffset.x, -25 };
+        break;
+
     case Cursor::SWORD_LEFT:
         _offset = { -5, -7 };
         break;
@@ -252,7 +265,7 @@ CursorRestorer::~CursorRestorer()
     // In case of software emulated cursor when cursor theme is not changed and it is shown after it was hidden
     // we force render the cursor area. It is needed to reduce the cursor show delay.
     if ( isShown && noThemeChange && cursorRenderer.isSoftwareEmulation() ) {
-        const fheroes2::Point & pos = LocalEvent::Get().GetMouseCursor();
+        const fheroes2::Point & pos = LocalEvent::Get().getMouseCursorPos();
         fheroes2::Display::instance().render( { pos.x, pos.y, 1, 1 } );
     }
 }
