@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -34,6 +34,7 @@
 
 #include "agg_image.h"
 #include "castle.h"
+#include "color.h"
 #include "cursor.h"
 #include "direction.h"
 #include "game_delays.h"
@@ -55,6 +56,7 @@
 #include "screen.h"
 #include "settings.h"
 #include "skill.h"
+#include "spell.h"
 #include "ui_constants.h"
 #include "ui_object_rendering.h"
 #include "world.h"
@@ -80,7 +82,8 @@ namespace
     };
 
     void populateStaticTileUnfitObjectInfo( TileUnfitRenderObjectInfo & tileUnfit, std::vector<fheroes2::ObjectRenderingInfo> & imageInfo,
-                                            std::vector<fheroes2::ObjectRenderingInfo> & shadowInfo, const fheroes2::Point & offset, const uint8_t alphaValue )
+                                            std::vector<fheroes2::ObjectRenderingInfo> & shadowInfo, const fheroes2::Point & offset, const uint8_t alphaValue,
+                                            const uint16_t fogDirection )
     {
         for ( auto & objectInfo : imageInfo ) {
             const fheroes2::Point imagePos = objectInfo.tileOffset;
@@ -103,6 +106,11 @@ namespace
                 }
             }
             else {
+                if ( offset.y == 0 && ( fogDirection & Direction::TOP ) == Direction::TOP ) {
+                    // Do not render object over the top map border if there is a nearby fog.
+                    continue;
+                }
+
                 if ( imagePos.x < 0 ) {
                     tileUnfit.topImages[imagePos + offset].emplace_front( objectInfo );
                 }
@@ -146,7 +154,7 @@ namespace
         }
     }
 
-    void populateHeroObjectInfo( TileUnfitRenderObjectInfo & tileUnfit, const Heroes * hero )
+    void populateHeroObjectInfo( TileUnfitRenderObjectInfo & tileUnfit, const Heroes * hero, const uint16_t fogDirection )
     {
         assert( hero != nullptr );
 
@@ -232,7 +240,7 @@ namespace
             }
 
             if ( imagePos.y > 0 && !isHeroInCastle ) {
-                // Hero horse or boat should not be rendered over the bottom map border.
+                // Hero's horse or boat should not be rendered over the bottom map border.
                 if ( ( heroPos.y + imagePos.y ) >= worldHeight ) {
                     continue;
                 }
@@ -254,6 +262,11 @@ namespace
                 }
             }
             else {
+                if ( heroPos.y == 0 && ( fogDirection & Direction::TOP ) == Direction::TOP ) {
+                    // Do not render hero's flag over the top map border if there is a nearby fog.
+                    continue;
+                }
+
                 if ( imagePos.x < 0 ) {
                     tileUnfit.topImages[imagePos + heroPos].emplace_front( objectInfo );
                 }
@@ -299,7 +312,8 @@ namespace
         const Maps::Tile & tileBelow = world.getTile( x, y + 1 );
 
         for ( const auto & lowerPart : tileBelow.getTopObjectParts() ) {
-            if ( lowerPart._uid == uid ) {
+            // NOTICE: Flags are put over the objects as a top object part and have the same UID but they do not make the object taller.
+            if ( lowerPart._uid == uid && lowerPart.icnType != MP2::OBJ_ICN_TYPE_FLAG32 ) {
                 // This is a tall object.
                 return true;
             }
@@ -311,29 +325,19 @@ namespace
 
 Interface::GameArea::GameArea( BaseInterface & interface )
     : _interface( interface )
-    , _minLeftOffset( 0 )
-    , _maxLeftOffset( 0 )
-    , _minTopOffset( 0 )
-    , _maxTopOffset( 0 )
-    , _prevIndexPos( 0 )
-    , scrollDirection( 0 )
-    , updateCursor( false )
-    , _mouseDraggingInitiated( false )
-    , _mouseDraggingMovement( false )
-    , _needRedrawByMouseDragging( false )
-    , _isFastScrollEnabled( false )
-    , _resetMousePositionForFastScroll( false )
 {
     // Do nothing.
 }
 
 void Interface::GameArea::generate( const fheroes2::Size & screenSize, const bool withoutBorders )
 {
-    if ( withoutBorders )
+    if ( withoutBorders ) {
         SetAreaPosition( 0, 0, screenSize.width, screenSize.height );
-    else
+    }
+    else {
         SetAreaPosition( fheroes2::borderWidthPx, fheroes2::borderWidthPx, screenSize.width - fheroes2::radarWidthPx - 3 * fheroes2::borderWidthPx,
                          screenSize.height - 2 * fheroes2::borderWidthPx );
+    }
 }
 
 void Interface::GameArea::SetAreaPosition( int32_t x, int32_t y, int32_t w, int32_t h )
@@ -403,10 +407,10 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 {
     const fheroes2::Rect & tileROI = GetVisibleTileROI();
 
-    int32_t minX = tileROI.x;
-    int32_t minY = tileROI.y;
     int32_t maxX = tileROI.x + tileROI.width;
     int32_t maxY = tileROI.y + tileROI.height;
+    const int32_t worldWidth = world.w();
+    const int32_t worldHeight = world.h();
 
 #ifdef WITH_DEBUG
     const bool renderFog = ( ( flag & LEVEL_FOG ) == LEVEL_FOG ) && !IS_DEVEL();
@@ -418,20 +422,20 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
     for ( int32_t y = 0; y < tileROI.height; ++y ) {
         fheroes2::Point offset( tileROI.x, tileROI.y + y );
 
-        if ( offset.y < 0 || offset.y >= world.h() ) {
+        if ( offset.y < 0 || offset.y >= worldHeight ) {
             for ( ; offset.x < maxX; ++offset.x ) {
                 Maps::redrawEmptyTile( dst, offset, *this );
             }
         }
         else {
             for ( ; offset.x < maxX; ++offset.x ) {
-                if ( offset.x < 0 || offset.x >= world.w() ) {
+                if ( offset.x < 0 || offset.x >= worldWidth ) {
                     Maps::redrawEmptyTile( dst, offset, *this );
                 }
                 else {
                     const Maps::Tile & tile = world.getTile( offset.x, offset.y );
                     // Do not render terrain on the tiles fully covered with the fog.
-                    if ( tile.getFogDirection() != DIRECTION_ALL || !renderFog ) {
+                    if ( !renderFog || tile.getFogDirection() != DIRECTION_ALL ) {
                         DrawTile( dst, getTileSurface( tile ), offset );
                     }
                 }
@@ -439,10 +443,10 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
         }
     }
 
-    minX = std::max( minX, 0 );
-    minY = std::max( minY, 0 );
-    maxX = std::min( maxX, world.w() );
-    maxY = std::min( maxY, world.h() );
+    const int32_t minX = std::max<int32_t>( tileROI.x, 0 );
+    const int32_t minY = std::max<int32_t>( tileROI.y, 0 );
+    maxX = std::min( maxX, worldWidth );
+    maxY = std::min( maxY, worldHeight );
 
     if ( minX >= maxX || minY >= maxY ) {
         // This can't be true! Please check your code changes as we shouldn't have an empty area.
@@ -478,31 +482,36 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 
     // Run through all visible tiles and find all tile-unfit objects.
     // Also cover extra tiles from right and bottom sides because these objects are usually bigger than 1x1 tiles.
-    const int32_t roiToRenderMinX = std::max( minX - 1, 0 );
-    const int32_t roiToRenderMinY = std::max( minY - 1, 0 );
-    const int32_t roiToRenderMaxX = std::min( maxX + 2, world.w() );
-    const int32_t roiToRenderMaxY = std::min( maxY + 2, world.h() );
+    const int32_t roiToRenderMinX = std::max<int32_t>( minX - 1, 0 );
+    const int32_t roiToRenderMinY = std::max<int32_t>( minY - 1, 0 );
+    const int32_t roiToRenderMaxX = std::min( maxX + 2, worldWidth );
+    const int32_t roiToRenderMaxY = std::min( maxY + 2, worldHeight );
 
     const bool isEditor = _interface.isEditor();
 
+    std::vector<fheroes2::Point> ghostAnimationPos;
+
     for ( int32_t posY = roiToRenderMinY; posY < roiToRenderMaxY; ++posY ) {
+        const int32_t offset = posY * worldWidth;
         for ( int32_t posX = roiToRenderMinX; posX < roiToRenderMaxX; ++posX ) {
-            const Maps::Tile & tile = world.getTile( posX, posY );
+            const int32_t tileIndex = offset + posX;
+            const Maps::Tile & tile = world.getTile( tileIndex );
 
             MP2::MapObjectType objectType = tile.getMainObjectType();
 
             // We will skip objects which are fully under the fog.
-            const bool isTileUnderFog = ( tile.getFogDirection() == DIRECTION_ALL ) && renderFog;
+            const uint16_t fogDirection = renderFog ? tile.getFogDirection() : 0;
+            const bool isTileUnderFog = ( fogDirection == DIRECTION_ALL );
 
             switch ( objectType ) {
             case MP2::OBJ_HERO: {
                 if ( isEditor ) {
-                    const uint8_t alphaValue = getObjectAlphaValue( tile.GetIndex(), MP2::OBJ_HERO );
+                    const uint8_t alphaValue = getObjectAlphaValue( tileIndex, MP2::OBJ_HERO );
 
                     auto spriteInfo = getEditorHeroSpritesPerTile( tile );
 
                     std::vector<fheroes2::ObjectRenderingInfo> temp;
-                    populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, temp, tile.GetCenter(), alphaValue );
+                    populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, temp, { posX, posY }, alphaValue, fogDirection );
                     continue;
                 }
 
@@ -510,25 +519,17 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
                     continue;
                 }
 
-                const bool isUpperTileUnderFog = ( posY > 0 ) ? ( world.getTile( posX, posY - 1 ).getFogDirection() == DIRECTION_ALL ) : true;
                 const Heroes * hero = tile.getHero();
 
+                // We do not render heroes and boats that are covered with the fog.
                 // Boats are 2 tiles high so for hero on the boat we have to populate info for boat one tile lower than the fog.
-                if ( isTileUnderFog && ( isUpperTileUnderFog || !hero->isShipMaster() ) ) {
-                    // AI heroes can go out of fog so we have to render them one step earlier than getting out of fog.
-                    if ( hero->isControlAI() ) {
-                        const Route::Path & path = hero->GetPath();
-                        // Check if the next AI hero path point will not be seen on map to skip it.
-                        if ( path.isValidForMovement() && ( world.getTile( path.GetFrontIndex() ).getFogDirection() == DIRECTION_ALL ) ) {
-                            continue;
-                        }
-                    }
-                    else {
-                        continue;
-                    }
+                // While moving enemy heroes can go out of the fog so we need to render them one step earlier than getting out of the fog.
+                if ( isTileUnderFog && ( !hero->isShipMaster() || ( posY > 0 ? ( world.getTile( tileIndex - worldWidth ).getFogDirection() == DIRECTION_ALL ) : true ) )
+                     && ( !hero->isMoveEnabled() || ( world.getTile( hero->GetPath().GetFrontIndex() ).getFogDirection() == DIRECTION_ALL ) ) ) {
+                    continue;
                 }
 
-                populateHeroObjectInfo( tileUnfit, hero );
+                populateHeroObjectInfo( tileUnfit, hero, fogDirection );
 
                 // Update object type as it could be an object under the hero.
                 objectType = tile.getMainObjectType( false );
@@ -541,31 +542,31 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
                     continue;
                 }
 
-                const uint8_t alphaValue = getObjectAlphaValue( tile.GetIndex(), MP2::OBJ_MONSTER );
+                const uint8_t alphaValue = getObjectAlphaValue( tileIndex, MP2::OBJ_MONSTER );
 
                 auto spriteInfo = getMonsterSpritesPerTile( tile, isEditor );
                 auto spriteShadowInfo = getMonsterShadowSpritesPerTile( tile, isEditor );
 
-                populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, spriteShadowInfo, tile.GetCenter(), alphaValue );
+                populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, spriteShadowInfo, { posX, posY }, alphaValue, fogDirection );
 
                 continue;
             }
 
             case MP2::OBJ_BOAT: {
                 // Boats are 2 tiles high so we have to populate info for boat one tile lower than the fog.
-                const bool isUpperTileUnderFog = ( posY > 0 ) ? ( world.getTile( posX, posY - 1 ).getFogDirection() == DIRECTION_ALL ) : true;
+                const bool isUpperTileUnderFog = ( posY > 0 ) ? ( world.getTile( tileIndex - worldWidth ).getFogDirection() == DIRECTION_ALL ) : true;
 
                 if ( !drawHeroes || ( isTileUnderFog && isUpperTileUnderFog ) ) {
                     // Boats can be occupied by heroes so they are considered as the same objects.
                     continue;
                 }
 
-                const uint8_t alphaValue = getObjectAlphaValue( tile.GetIndex(), MP2::OBJ_BOAT );
+                const uint8_t alphaValue = getObjectAlphaValue( tileIndex, MP2::OBJ_BOAT );
 
                 auto spriteInfo = getBoatSpritesPerTile( tile );
                 auto spriteShadowInfo = getBoatShadowSpritesPerTile( tile );
 
-                populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, spriteShadowInfo, tile.GetCenter(), alphaValue );
+                populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, spriteShadowInfo, { posX, posY }, alphaValue, fogDirection );
 
                 continue;
             }
@@ -575,11 +576,21 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
             }
 
             // These are parts of original action objects which must be rendered under heroes.
-            if ( objectType == MP2::OBJ_MINE ) {
+            if ( !isPuzzleDraw && ( objectType == MP2::OBJ_ABANDONED_MINE || ( objectType == MP2::OBJ_MINE && Maps::getMineSpellIdFromTile( tile ) == Spell::HAUNT ) ) ) {
+                // To correctly render ghosts over abandoned mine it is needed to analyze one tile to the top direction under fog.
+                const bool areGhostsUnderFog = renderFog && isTileUnderFog && ( world.getTile( tileIndex - worldWidth ).getFogDirection() == DIRECTION_ALL );
+                if ( areGhostsUnderFog ) {
+                    // Current tile and the top one are both fully under the fog.
+                    continue;
+                }
+
+                ghostAnimationPos.emplace_back( posX, posY );
+            }
+            else if ( objectType == MP2::OBJ_MINE && !isTileUnderFog ) {
                 auto spriteInfo = getMineGuardianSpritesPerTile( tile );
                 if ( !spriteInfo.empty() ) {
                     const uint8_t alphaValue = getObjectAlphaValue( tile.getMainObjectPart()._uid );
-                    populateStaticTileUnfitBackgroundObjectInfo( tileUnfit, spriteInfo, tile.GetCenter(), alphaValue );
+                    populateStaticTileUnfitBackgroundObjectInfo( tileUnfit, spriteInfo, { posX, posY }, alphaValue );
                 }
             }
         }
@@ -587,10 +598,11 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 
     // Render all terrain and background layer object.
     for ( int32_t y = minY; y < maxY; ++y ) {
+        const int32_t offset = y * worldWidth;
         for ( int32_t x = minX; x < maxX; ++x ) {
-            const Maps::Tile & tile = world.getTile( x, y );
+            const Maps::Tile & tile = world.getTile( x + offset );
 
-            if ( tile.getFogDirection() == DIRECTION_ALL && renderFog ) {
+            if ( renderFog && ( tile.getFogDirection() == DIRECTION_ALL ) ) {
                 continue;
             }
 
@@ -605,10 +617,11 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
     renderImagesOnTiles( dst, tileUnfit.bottomBackgroundImages, *this );
 
     for ( int32_t y = minY; y < maxY; ++y ) {
+        const int32_t offset = y * worldWidth;
         for ( int32_t x = minX; x < maxX; ++x ) {
-            const Maps::Tile & tile = world.getTile( x, y );
+            const Maps::Tile & tile = world.getTile( x + offset );
 
-            if ( tile.getFogDirection() == DIRECTION_ALL && renderFog ) {
+            if ( renderFog && ( tile.getFogDirection() == DIRECTION_ALL ) ) {
                 continue;
             }
 
@@ -626,10 +639,11 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
     renderImagesOnTiles( dst, tileUnfit.lowPriorityBottomImages, *this );
 
     for ( int32_t y = minY; y < maxY; ++y ) {
+        const int32_t offset = y * worldWidth;
         for ( int32_t x = minX; x < maxX; ++x ) {
-            const Maps::Tile & tile = world.getTile( x, y );
+            const Maps::Tile & tile = world.getTile( x + offset );
 
-            if ( tile.getFogDirection() == DIRECTION_ALL && renderFog ) {
+            if ( renderFog && ( tile.getFogDirection() == DIRECTION_ALL ) ) {
                 continue;
             }
 
@@ -644,24 +658,18 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
     // High priority images are drawn after any other object on this tile.
     renderImagesOnTiles( dst, tileUnfit.highPriorityBottomImages, *this );
 
-    std::vector<const Maps::ObjectPart *> topLayerTallObjects;
+    std::vector<std::pair<const Maps::ObjectPart *, int32_t>> topLayerTallObjects;
 
     // Expand  ROI to properly render very tall objects (1 tile - left and right; 2 tiles - bottom): Abandoned mine Ghosts, Flag on the Alchemist lab, and others.
-    const int32_t roiExtraObjectsMaxX = std::min( maxX + 1, world.w() );
+    const int32_t roiExtraObjectsMaxX = std::min( maxX + 1, worldWidth );
     for ( int32_t y = minY; y < roiToRenderMaxY; ++y ) {
+        const int32_t offset = y * worldWidth;
         for ( int32_t x = roiToRenderMinX; x < roiExtraObjectsMaxX; ++x ) {
-            const Maps::Tile & tile = world.getTile( x, y );
+            const int32_t tileIndex = x + offset;
+            const Maps::Tile & tile = world.getTile( tileIndex );
 
-            const bool isTileUnderFog = ( tile.getFogDirection() == DIRECTION_ALL ) && renderFog;
-            if ( isTileUnderFog ) {
-                // To correctly render tall extra objects (ghosts over abandoned mine) it is needed to analyze one tile to the bottom direction under fog.
-                const bool isUpperTileUnderFog = ( y > 0 ) ? ( world.getTile( x, y - 1 ).getFogDirection() == DIRECTION_ALL ) : true;
-                if ( isUpperTileUnderFog ) {
-                    // If current tile and the bottom one are both under the fog
-                    continue;
-                }
-
-                redrawTopLayerExtraObjects( tile, dst, isPuzzleDraw, *this );
+            if ( renderFog && ( tile.getFogDirection() == DIRECTION_ALL ) ) {
+                // Do not render parts fully covered with fog.
                 continue;
             }
 
@@ -669,27 +677,30 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
             // For now what we need to do is to run through all level 2 objects and verify that the tile below doesn't have
             // any other level 2 objects with the same UID.
 
-            topLayerTallObjects.clear();
             for ( const auto & part : tile.getTopObjectParts() ) {
                 if ( isTallTopLayerObject( x, y, part._uid ) ) {
-                    topLayerTallObjects.emplace_back( &part );
+                    topLayerTallObjects.emplace_back( &part, tileIndex );
                 }
                 else {
-                    redrawTopLayerObject( tile, dst, isPuzzleDraw, *this, part );
+                    Maps::redrawTopLayerObject( tile, dst, isPuzzleDraw, { x, y }, *this, part );
                 }
             }
-
-            redrawTopLayerExtraObjects( tile, dst, isPuzzleDraw, *this );
-
-            for ( const auto * part : topLayerTallObjects ) {
-                redrawTopLayerObject( tile, dst, isPuzzleDraw, *this, *part );
-            }
         }
     }
 
     // Draw upper part of tile-unfit sprites.
     renderImagesOnTiles( dst, tileUnfit.topImages, *this );
 
+    // Draw the top part of tall objects.
+    for ( const auto & [part, tileIndex] : topLayerTallObjects ) {
+        Maps::redrawTopLayerObject( world.getTile( tileIndex ), dst, isPuzzleDraw, Maps::GetPoint( tileIndex ), *this, *part );
+    }
+
+    // Draw flying ghosts animation for Haunted and Abandoned Mines over all other objects.
+    for ( const auto & pos : ghostAnimationPos ) {
+        Maps::redrawFlyingGhostsOnMap( dst, pos, *this, isEditor );
+    }
+
     // Draw hero's route. It should be drawn on top of everything.
     const bool drawRoutes = ( flag & LEVEL_ROUTES ) != 0;
 
@@ -748,11 +759,12 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 #endif
 
     if ( drawPassabilities ) {
-        const int32_t friendColors = Players::FriendColors();
+        const PlayerColorsSet friendColors = Players::FriendColors();
 
         for ( int32_t y = minY; y < maxY; ++y ) {
+            const int32_t offset = y * worldWidth;
             for ( int32_t x = minX; x < maxX; ++x ) {
-                redrawPassable( world.getTile( x, y ), dst, friendColors, *this, isEditor );
+                redrawPassable( world.getTile( x + offset ), dst, friendColors, *this, isEditor );
             }
         }
     }
@@ -760,8 +772,9 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
         const bool drawTowns = ( flag & LEVEL_TOWNS );
 
         for ( int32_t y = minY; y < maxY; ++y ) {
+            const int32_t offset = y * worldWidth;
             for ( int32_t x = minX; x < maxX; ++x ) {
-                const Maps::Tile & tile = world.getTile( x, y );
+                const Maps::Tile & tile = world.getTile( x + offset );
 
                 if ( tile.getFogDirection() != Direction::UNKNOWN ) {
                     drawFog( tile, dst, *this );
@@ -799,8 +812,8 @@ void Interface::GameArea::renderTileAreaSelect( fheroes2::Image & dst, const int
     const fheroes2::Rect imageRoi{ startX, startY, sizeX, sizeY };
     const fheroes2::Rect overlappedRoi = _windowROI ^ imageRoi;
 
-    const int32_t limitedLineWidth = std::min( 2, overlappedRoi.width );
-    const int32_t limitedLineHeight = std::min( 2, overlappedRoi.height );
+    const int32_t limitedLineWidth = std::min<int32_t>( 2, overlappedRoi.width );
+    const int32_t limitedLineHeight = std::min<int32_t>( 2, overlappedRoi.height );
 
     const uint8_t color = ( isActionObject ? 115 : 181 );
 
@@ -812,7 +825,7 @@ void Interface::GameArea::renderTileAreaSelect( fheroes2::Image & dst, const int
 
 void Interface::GameArea::updateMapFogDirections()
 {
-    const int32_t friendColors = Players::FriendColors();
+    const PlayerColorsSet friendColors = Players::FriendColors();
 
     Maps::updateFogDirectionsInArea( { 0, 0 }, { world.w(), world.h() }, friendColors );
 }
@@ -912,6 +925,8 @@ int Interface::GameArea::GetScrollCursor() const
 
 void Interface::GameArea::SetScroll( const int direction )
 {
+    assert( !isDragScroll() );
+
     if ( ( direction & SCROLL_LEFT ) == SCROLL_LEFT ) {
         if ( _topLeftTileOffset.x > _minLeftOffset ) {
             scrollDirection |= direction;
