@@ -40,9 +40,10 @@ namespace
 {
     fheroes2::Image GetBarBackgroundSprite()
     {
-        fheroes2::Image icon( 34, 34 );
+        fheroes2::Image icon;
         // Sprite ( ICN::HSICONS, 0 ) has no transparency so we can say that 'icon' will also have no transparency.
         icon._disableTransformLayer();
+        icon.resize( 34, 34 );
         icon.reset();
         fheroes2::DrawBorder( icon, fheroes2::GetColorId( 0xD0, 0xC0, 0x48 ) );
         fheroes2::Copy( fheroes2::AGG::GetICN( ICN::HSICONS, 0 ), 26, 21, icon, 1, 1, 32, 32 );
@@ -204,13 +205,15 @@ bool PrimarySkillsBar::ActionBarLeftMouseSingleClick( int & skill )
     }
 
     // The case when we are in Editor mode.
-    auto primarySkillEditDialog = [&skill]( int32_t & skillValue ) {
+    auto primarySkillEditDialog = [skill]( int32_t & skillValue ) {
         std::string header = _( "Set %{skill} Skill" );
         StringReplace( header, "%{skill}", Skill::Primary::String( skill ) );
 
+        const auto [min, max] = Skill::Primary::getSkillValueRange( skill );
+
         const fheroes2::PrimarySkillDialogElement skillUI{ skill, {} };
 
-        return Dialog::SelectCount( std::move( header ), skill == Skill::Primary::POWER ? 1 : 0, 99, skillValue, 1, &skillUI );
+        return Dialog::SelectCount( std::move( header ), min, max, skillValue, 1, &skillUI );
     };
 
     int32_t value;
