@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,8 +21,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2RECRUITS_H
-#define H2RECRUITS_H
+#pragma once
 
 #include <cstdint>
 #include <utility>
@@ -37,13 +36,11 @@ class Recruit
 public:
     Recruit();
     Recruit( const Heroes & hero, const uint32_t surrenderDay );
-    explicit Recruit( const Heroes & hero );
-    Recruit( const Recruit & ) = delete;
-
-    ~Recruit() = default;
-
-    Recruit & operator=( const Recruit & ) = delete;
-    Recruit & operator=( Recruit && ) = default;
+    explicit Recruit( const Heroes & hero )
+        : Recruit( hero, 0 )
+    {
+        // Do nothing.
+    }
 
     int getID() const
     {
@@ -56,22 +53,27 @@ public:
     }
 
 private:
-    int _id;
+    int32_t _id;
     uint32_t _surrenderDay;
 
     friend OStreamBase & operator<<( OStreamBase & stream, const Recruit & recruit );
     friend IStreamBase & operator>>( IStreamBase & stream, Recruit & recruit );
 };
 
-class Recruits : public std::pair<Recruit, Recruit>
+class Recruits final : public std::pair<Recruit, Recruit>
 {
 public:
-    Recruits();
-
     void Reset();
 
-    int GetID1() const;
-    int GetID2() const;
+    int GetID1() const
+    {
+        return first.getID();
+    }
+
+    int GetID2() const
+    {
+        return second.getID();
+    }
 
     Heroes * GetHero1() const;
     Heroes * GetHero2() const;
@@ -87,5 +89,3 @@ public:
 
 OStreamBase & operator<<( OStreamBase & stream, const Recruit & recruit );
 IStreamBase & operator>>( IStreamBase & stream, Recruit & recruit );
-
-#endif
