@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -904,7 +904,7 @@ bool Kingdom::IsTileVisibleFromCrystalBall( const int32_t dest ) const
         assert( hero != nullptr );
 
         if ( hero->GetBagArtifacts().isArtifactBonusPresent( fheroes2::ArtifactBonusType::VIEW_MONSTER_INFORMATION ) ) {
-            const uint32_t crystalBallDistance = hero->GetVisionsDistance();
+            const uint32_t crystalBallDistance = Heroes::GetVisionsDistance();
 
             if ( Maps::GetStraightLineDistance( hero->GetIndex(), dest ) <= crystalBallDistance ) {
                 return true;
@@ -966,7 +966,7 @@ IStreamBase & operator>>( IStreamBase & stream, Kingdom & kingdom )
     stream >> kingdom.modes;
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1109_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1109_RELEASE ) {
-        int temp;
+        int32_t temp;
         stream >> temp;
         kingdom._color = static_cast<PlayerColor>( temp );
     }
@@ -979,7 +979,7 @@ IStreamBase & operator>>( IStreamBase & stream, Kingdom & kingdom )
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE2_1100_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE2_1100_RELEASE ) {
-        int dummy;
+        int32_t dummy;
 
         stream >> dummy;
     }
