@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2023 - 2025                                             *
+ *   Copyright (C) 2023 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -22,6 +22,7 @@
 
 #include <algorithm>
 #include <array>
+#include <cstddef>
 #include <cstdint>
 #include <map>
 #include <string>
@@ -33,6 +34,9 @@
 #include "map_object_info.h"
 #include "resource.h"
 
+class IStreamBase;
+class OStreamBase;
+
 namespace Maps::Map_Format
 {
     struct TileObjectInfo
@@ -52,11 +56,8 @@ namespace Maps::Map_Format
         std::vector<TileObjectInfo> objects;
     };
 
-    // This structure should be used for any object that require simple data to be saved into map.
-    struct StandardObjectMetadata
-    {
-        std::array<int32_t, 3> metadata{ 0 };
-    };
+    constexpr size_t messageCharLimit{ 999 };
+    constexpr size_t nameCharLimit{ 30 };
 
     struct CastleMetadata
     {
@@ -79,10 +80,12 @@ namespace Maps::Map_Format
         // A list of buildings that cannot be built.
         std::vector<uint32_t> bannedBuildings;
 
-        // Spells that must appear in the Magic Guild.
-        std::vector<int32_t> mustHaveSpells;
+        // Spells that must appear in the Mage Guild in relation to position index.
+        // The spell index is written as a decimal number, where the tens digit indicates the spell level (0-4),
+        // and the units digit (0-4) represents its position from left to right.
+        std::map<uint8_t, int32_t> mustHaveSpells;
 
-        // Spells that must NOT appear the Magic Guild.
+        // Spells that must NOT appear the Mage Guild.
         std::vector<int32_t> bannedSpells;
 
         // The number of monsters available to hire in dwellings. A negative value means that no change will be applied.
@@ -260,6 +263,37 @@ namespace Maps::Map_Format
         PlayerColor ownerColor{ 0 };
     };
 
+    struct MonsterMetadata
+    {
+        int32_t count{ 0 };
+
+        // This is not used and reserved for the future.
+        int32_t joinCondition{ 0 };
+
+        // This is not used and reserved for the future.
+        bool isWeeklyGrowthDisabled{ false };
+
+        // Only for random monsters.
+        std::vector<int32_t> selected;
+    };
+
+    struct ArtifactMetadata
+    {
+        // Only for Random Ultimate Artifact.
+        int32_t radius{ 0 };
+
+        // This is not used and reserved for the future.
+        int32_t captureCondition{ 0 };
+
+        // Only for random artifacts and Scroll Spell.
+        std::vector<int32_t> selected;
+    };
+
+    struct ResourceMetadata
+    {
+        int32_t count{ 0 };
+    };
+
     struct DailyEvent
     {
         std::string message;
@@ -277,6 +311,37 @@ namespace Maps::Map_Format
         Funds resources;
     };
 
+    struct TranslationSphinxMetadata final
+    {
+        std::string riddle;
+
+        std::vector<std::string> answers;
+    };
+
+    struct TranslationBaseMapMetadata final
+    {
+        std::string name;
+        std::string description;
+        std::string creatorNotes;
+    };
+
+    struct TranslationFormat final
+    {
+        std::vector<std::string> dailyEvents;
+
+        std::vector<std::string> rumors;
+
+        std::map<uint32_t, std::string> castleMetadata;
+
+        std::map<uint32_t, std::string> heroMetadata;
+
+        std::map<uint32_t, TranslationSphinxMetadata> sphinxMetadata;
+
+        std::map<uint32_t, std::string> signMetadata;
+
+        std::map<uint32_t, std::string> adventureMapEventMetadata;
+    };
+
     struct BaseMapFormat
     {
         uint16_t version{ 1 };
@@ -316,9 +381,12 @@ namespace Maps::Map_Format
         // This parameter is only visible within the Editor, it doesn't affect the gameplay in any way.
         // The parameter is mandatory to fill out by map makers who want to have their creations bundled with the engine.
         std::string creatorNotes;
+
+        // A map can support multiple languages. A language of the map should be chosen based on game's language.
+        std::map<fheroes2::SupportedLanguage, TranslationBaseMapMetadata> translations;
     };
 
-    struct MapFormat : public BaseMapFormat
+    struct MapFormat final : public BaseMapFormat
     {
         // This is used only for campaign maps.
         std::vector<uint32_t> additionalInfo;
@@ -330,8 +398,6 @@ namespace Maps::Map_Format
         std::vector<std::string> rumors;
 
         // These are metadata maps in relation to object UID.
-        std::map<uint32_t, StandardObjectMetadata> standardMetadata;
-
         std::map<uint32_t, CastleMetadata> castleMetadata;
 
         std::map<uint32_t, HeroMetadata> heroMetadata;
@@ -345,10 +411,21 @@ namespace Maps::Map_Format
         std::map<uint32_t, SelectionObjectMetadata> selectionObjectMetadata;
 
         std::map<uint32_t, CapturableObjectMetadata> capturableObjectsMetadata;
+
+        std::map<uint32_t, MonsterMetadata> monsterMetadata;
+
+        std::map<uint32_t, ArtifactMetadata> artifactMetadata;
+
+        std::map<uint32_t, ResourceMetadata> resourceMetadata;
+
+        std::map<fheroes2::SupportedLanguage, TranslationFormat> translationInfo;
     };
 
     bool loadBaseMap( const std::string & path, BaseMapFormat & map );
     bool loadMap( const std::string & path, MapFormat & map );
 
     bool saveMap( const std::string & path, const MapFormat & map );
+
+    bool saveMap( OStreamBase & stream, const MapFormat & map );
+    bool loadMap( IStreamBase & stream, MapFormat & map );
 }
