@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -99,7 +99,7 @@ namespace
         const int width = world.w();
         const int x = tileIndex % width;
         const int y = tileIndex / width;
-        Maps::Tile & tile = world.getTile( tileIndex );
+        const Maps::Tile & tile = world.getTile( tileIndex );
 
         const auto isProtectedBy = [tileIndex, &tile]( const int32_t monsterTileIndex ) {
             const Maps::Tile & monsterTile = world.getTile( monsterTileIndex );
@@ -425,10 +425,10 @@ Maps::Indexes Maps::getAroundIndexes( const int32_t tileIndex, const int32_t wid
     const int32_t centerY = tileIndex / width;
 
     // We avoid getting out of map boundaries.
-    const int32_t minTileX = std::max( centerX - maxDistanceFromTile, 0 );
-    const int32_t minTileY = std::max( centerY - maxDistanceFromTile, 0 );
-    const int32_t maxTileX = std::min( centerX + maxDistanceFromTile + 1, width );
-    const int32_t maxTileY = std::min( centerY + maxDistanceFromTile + 1, height );
+    const int32_t minTileX = std::max<int32_t>( centerX - maxDistanceFromTile, 0 );
+    const int32_t minTileY = std::max<int32_t>( centerY - maxDistanceFromTile, 0 );
+    const int32_t maxTileX = std::min<int32_t>( centerX + maxDistanceFromTile + 1, width );
+    const int32_t maxTileY = std::min<int32_t>( centerY + maxDistanceFromTile + 1, height );
 
     for ( int32_t tileY = minTileY; tileY < maxTileY; ++tileY ) {
         const int32_t indexOffsetY = tileY * width;
@@ -474,13 +474,13 @@ void Maps::ClearFog( const int32_t tileIndex, const int32_t scoutingDistance, co
     const int32_t squaredScoutingRadiusLimit = getSquaredScoutingRadiusLimit( scoutingDistance );
     const PlayerColorsSet alliedColors = Players::GetPlayerFriends( playerColor );
 
-    const int32_t minY = std::max( center.y - scoutingDistance, 0 );
-    const int32_t maxY = std::min( center.y + scoutingDistance, world.h() - 1 );
+    const int32_t minY = std::max<int32_t>( center.y - scoutingDistance, 0 );
+    const int32_t maxY = std::min<int32_t>( center.y + scoutingDistance, world.h() - 1 );
     assert( minY < maxY );
 
     const int32_t worldWidth = world.w();
-    const int32_t minX = std::max( center.x - scoutingDistance, 0 );
-    const int32_t maxX = std::min( center.x + scoutingDistance, worldWidth - 1 );
+    const int32_t minX = std::max<int32_t>( center.x - scoutingDistance, 0 );
+    const int32_t maxX = std::min<int32_t>( center.x + scoutingDistance, worldWidth - 1 );
     assert( minX < maxX );
 
     fheroes2::Point fogRevealMinPos( world.h(), worldWidth );
@@ -533,13 +533,13 @@ int32_t Maps::getFogTileCountToBeRevealed( const int32_t tileIndex, const int32_
     const fheroes2::Point center = Maps::GetPoint( tileIndex );
     const int32_t squaredScoutingRadiusLimit = getSquaredScoutingRadiusLimit( scoutingDistance );
 
-    const int32_t minY = std::max( center.y - scoutingDistance, 0 );
-    const int32_t maxY = std::min( center.y + scoutingDistance, world.h() - 1 );
+    const int32_t minY = std::max<int32_t>( center.y - scoutingDistance, 0 );
+    const int32_t maxY = std::min<int32_t>( center.y + scoutingDistance, world.h() - 1 );
     assert( minY < maxY );
 
     const int32_t worldWidth = world.w();
-    const int32_t minX = std::max( center.x - scoutingDistance, 0 );
-    const int32_t maxX = std::min( center.x + scoutingDistance, worldWidth - 1 );
+    const int32_t minX = std::max<int32_t>( center.x - scoutingDistance, 0 );
+    const int32_t maxX = std::min<int32_t>( center.x + scoutingDistance, worldWidth - 1 );
     assert( minX < maxX );
 
     int32_t tileCount = 0;
