@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -27,6 +27,7 @@
 #include <cassert>
 #include <cstring>
 #include <functional>
+#include <initializer_list>
 #include <list>
 #include <map>
 #include <sstream>
@@ -39,6 +40,7 @@
 #include "game_io.h"
 #include "game_over.h"
 #include "logging.h"
+#include "map_format_helper.h"
 #include "map_format_info.h"
 #include "maps_tiles.h"
 #include "maps_tiles_helper.h"
@@ -64,6 +66,8 @@ namespace
         // create a list of unique maps (based on the map file name) and filter it by the preferred number of players
         std::map<std::string, Maps::FileInfo, std::less<>> uniqueMaps;
 
+        const auto currentLanguage = fheroes2::getCurrentLanguage();
+
         // Maps made by the original French version Editor or hacked maps could contain
         // special ASCII characters that are not supposed to be there.
         // While reading the original maps we attempt to fix these characters
@@ -81,7 +85,7 @@ namespace
                 }
             }
             else {
-                if ( !fi.readResurrectionMap( mapFile, isForEditor ) ) {
+                if ( !fi.readResurrectionMap( mapFile, isForEditor, currentLanguage ) ) {
                     continue;
                 }
             }
@@ -188,6 +192,8 @@ void Maps::FileInfo::Reset()
     worldMonth = 0;
 
     mainLanguage = fheroes2::SupportedLanguage::English;
+
+    translations = {};
 }
 
 bool Maps::FileInfo::readMP2Map( std::string filePath, const bool isForEditor )
@@ -382,7 +388,7 @@ bool Maps::FileInfo::readMP2Map( std::string filePath, const bool isForEditor )
     return true;
 }
 
-bool Maps::FileInfo::readResurrectionMap( std::string filePath, const bool isForEditor )
+bool Maps::FileInfo::readResurrectionMap( std::string filePath, const bool isForEditor, const fheroes2::SupportedLanguage currentLanguage )
 {
     Reset();
 
@@ -391,6 +397,18 @@ bool Maps::FileInfo::readResurrectionMap( std::string filePath, const bool isFor
         return false;
     }
 
+    if ( !isForEditor ) {
+        // Since we are loading this map for the game, we need to set the language of the map.
+        if ( !Maps::setInGameLanguage( map, currentLanguage ) ) {
+            // The currently chosen language is not available in the map.
+            // Try to use the default language then - English.
+            // Even if it fails, the map's first language is going to be used after.
+            if ( currentLanguage != fheroes2::SupportedLanguage::English ) {
+                Maps::setInGameLanguage( map, fheroes2::SupportedLanguage::English );
+            }
+        }
+    }
+
     if ( !loadResurrectionMap( map, std::move( filePath ) ) ) {
         return false;
     }
@@ -518,6 +536,11 @@ bool Maps::FileInfo::loadResurrectionMap( const Map_Format::BaseMapFormat & map,
 
     mainLanguage = map.mainLanguage;
 
+    translations = {};
+    for ( const auto & [language, info] : map.translations ) {
+        translations.emplace_back( language );
+    }
+
     return true;
 }
 
@@ -753,3 +776,28 @@ MapsFileInfoList Maps::getResurrectionMapFileInfos( const bool isForEditor, cons
 
     return validMaps;
 }
+
+bool Maps::tryGetMatchingFile( const std::string & fileName, std::string & matchingFilePath )
+{
+    static const auto fileNameToPath = []() {
+        std::map<std::string, std::string> result;
+
+        const ListFiles files = Settings::FindFiles( "maps", "", false );
+
+        for ( const std::string & file : files ) {
+            result.try_emplace( StringLower( System::GetFileName( file ) ), file );
+        }
+
+        return result;
+    }();
+
+    const auto result = fileNameToPath.find( fileName );
+
+    if ( result != fileNameToPath.end() ) {
+        matchingFilePath = result->second;
+
+        return true;
+    }
+
+    return false;
+}
