@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -39,7 +39,7 @@
 class IStreamBase;
 class OStreamBase;
 
-enum class GameVersion : int
+enum class GameVersion : int32_t
 {
     SUCCESSION_WARS = 0,
     PRICE_OF_LOYALTY = 1,
@@ -76,7 +76,7 @@ namespace Maps
 
         bool readMP2Map( std::string filePath, const bool isForEditor );
 
-        bool readResurrectionMap( std::string filePath, const bool isForEditor );
+        bool readResurrectionMap( std::string filePath, const bool isForEditor, const fheroes2::SupportedLanguage currentLanguage );
 
         bool loadResurrectionMap( const Map_Format::BaseMapFormat & map, std::string filePath );
 
@@ -144,10 +144,6 @@ namespace Maps
 
         void Reset();
 
-        static bool sortByFileName( const FileInfo & lhs, const FileInfo & rhs );
-
-        static bool sortByMapName( const FileInfo & lhs, const FileInfo & rhs );
-
         // Only Resurrection Maps contain supported language.
         std::optional<fheroes2::SupportedLanguage> getSupportedLanguage() const
         {
@@ -179,6 +175,39 @@ namespace Maps
             LOSS_OUT_OF_TIME = 3
         };
 
+        // This comparator performs the case-insensitive comparison
+        struct CompareByFileName
+        {
+            bool operator()( const FileInfo & lhs, const FileInfo & rhs ) const;
+            bool operator()( const FileInfo & lhs, const std::string & rhs ) const;
+            bool operator()( const std::string & lhs, const FileInfo & rhs ) const;
+        };
+
+        // This comparator performs the case-insensitive comparison
+        struct CompareByMapName
+        {
+            bool operator()( const FileInfo & lhs, const FileInfo & rhs ) const;
+        };
+
+        // This comparator will place the newer FileInfo instances (with a bigger timestamp) first
+        struct CompareByTimestamp
+        {
+            bool operator()( const FileInfo & lhs, const FileInfo & rhs ) const
+            {
+                return lhs.timestamp > rhs.timestamp;
+            }
+
+            bool operator()( const FileInfo & lhs, uint32_t rhs ) const
+            {
+                return lhs.timestamp > rhs;
+            }
+
+            bool operator()( uint32_t lhs, const FileInfo & rhs ) const
+            {
+                return lhs > rhs.timestamp;
+            }
+        };
+
         std::string filename;
         std::string name;
         std::string description;
@@ -223,6 +252,13 @@ namespace Maps
         // because it is unknown what language was used for these maps.
         fheroes2::SupportedLanguage mainLanguage{ fheroes2::SupportedLanguage::English };
 
+        // A list of supported translations for the map. Only for Resurrection maps.
+        // This entry is not a part of the save file as once a map has been loaded it is not possible to change texts.
+        // It might be possible in the future but it would require a huge rework of the code.
+        //
+        // It is very unlikely that a player suddenly decides to switch a language in the middle of a playthrough.
+        std::vector<fheroes2::SupportedLanguage> translations;
+
     private:
         void FillUnions( const PlayerColorsSet side1Colors, const PlayerColorsSet side2Colors );
     };
@@ -240,4 +276,6 @@ namespace Maps
 
     // Only for RESURRECTION map files.
     MapsFileInfoList getResurrectionMapFileInfos( const bool isForEditor, const uint8_t humanPlayerCount );
+
+    bool tryGetMatchingFile( const std::string & fileName, std::string & matchingFilePath );
 }
