@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -346,28 +346,6 @@ namespace
         return true;
     }
 
-    bool isSpriteRoad( const MP2::ObjectIcnType objectIcnType, const uint8_t imageIndex )
-    {
-        switch ( objectIcnType ) {
-        case MP2::OBJ_ICN_TYPE_ROAD: {
-            static const std::set<uint8_t> allowedIndecies{ 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 16, 17, 18, 19, 20, 21, 26, 28, 29, 30, 31 };
-            return ( allowedIndecies.count( imageIndex ) == 1 );
-        }
-        case MP2::OBJ_ICN_TYPE_OBJNTOWN: {
-            static const std::set<uint8_t> allowedIndecies{ 13, 29, 45, 61, 77, 93, 109, 125, 141, 157, 173, 189 };
-            return ( allowedIndecies.count( imageIndex ) == 1 );
-        }
-        case MP2::OBJ_ICN_TYPE_OBJNTWRD: {
-            static const std::set<uint8_t> allowedIndecies{ 13, 29 };
-            return ( allowedIndecies.count( imageIndex ) == 1 );
-        }
-        default:
-            break;
-        }
-
-        return false;
-    }
-
     bool isObjectPartShadow( const Maps::ObjectPart & ta )
     {
         return isValidShadowSprite( MP2::getIcnIdFromObjectIcnType( ta.icnType ), ta.icnIndex );
@@ -514,6 +492,12 @@ void Maps::Tile::setBoat( const int direction, const PlayerColor color )
     assert( isWater() );
 
     setMainObjectType( MP2::OBJ_BOAT );
+
+    // Make sure that the object part is being cleared before setting a new one.
+    _mainObjectPart = {};
+
+    // A boat is an object so it must be on the object layer.
+    _mainObjectPart.layerType = ObjectLayerType::OBJECT_LAYER;
     _mainObjectPart.icnType = MP2::OBJ_ICN_TYPE_BOAT32;
 
     switch ( direction ) {
@@ -547,6 +531,9 @@ void Maps::Tile::setBoat( const int direction, const PlayerColor color )
         break;
     }
 
+    // Boats are not passable.
+    assert( !_mainObjectPart.isPassabilityTransparent() );
+
 #ifdef WITH_DEBUG
     const uint32_t newUid = getNewObjectUID();
 
@@ -811,6 +798,28 @@ bool Maps::Tile::doesObjectExist( const uint32_t uid ) const
                         [uid]( const auto & part ) { return part._uid == uid && !part.isPassabilityTransparent(); } );
 }
 
+bool Maps::Tile::isSpriteRoad( const MP2::ObjectIcnType objectIcnType, const uint8_t imageIndex )
+{
+    switch ( objectIcnType ) {
+    case MP2::OBJ_ICN_TYPE_ROAD: {
+        static const std::set<uint8_t> allowedIndecies{ 0, 2, 3, 4, 5, 6, 7, 9, 12, 13, 14, 16, 17, 18, 19, 20, 21, 26, 28, 29, 30, 31 };
+        return ( allowedIndecies.count( imageIndex ) == 1 );
+    }
+    case MP2::OBJ_ICN_TYPE_OBJNTOWN: {
+        static const std::set<uint8_t> allowedIndecies{ 13, 29, 45, 61, 77, 93, 109, 125, 141, 157, 173, 189 };
+        return ( allowedIndecies.count( imageIndex ) == 1 );
+    }
+    case MP2::OBJ_ICN_TYPE_OBJNTWRD: {
+        static const std::set<uint8_t> allowedIndecies{ 13, 29 };
+        return ( allowedIndecies.count( imageIndex ) == 1 );
+    }
+    default:
+        break;
+    }
+
+    return false;
+}
+
 void Maps::Tile::pushGroundObjectPart( ObjectPart part )
 {
     if ( isSpriteRoad( part.icnType, part.icnIndex ) ) {
@@ -1051,6 +1060,33 @@ bool Maps::Tile::isSuitableForDisembarkation() const
            && std::all_of( _groundObjectPart.begin(), _groundObjectPart.end(), isObjectPartPassable );
 }
 
+bool Maps::Tile::isSuitableForSummoningBoat() const
+{
+    // Tiles with heroes are a special case because heroes are moving objects and, strictly speaking, are not part of the map itself, so the checks below do not work with
+    // them
+    if ( _mainObjectType == MP2::OBJ_HERO ) {
+        return false;
+    }
+
+    // Tiles with events are not suitable for summoning a boat (at least for now)
+    if ( _mainObjectType == MP2::OBJ_EVENT ) {
+        return false;
+    }
+
+    // It is impossible to summon a boat on land
+    if ( !isWater() ) {
+        return false;
+    }
+
+    const auto isObjectPartPassable = []( const Maps::ObjectPart & part ) {
+        // The part of the Lighthouse flag that is located on the adjacent tile doesn't affect the passability
+        return part.icnType == MP2::OBJ_ICN_TYPE_FLAG32 || part.isPassabilityTransparent() || isObjectPartShadow( part );
+    };
+
+    return ( _mainObjectPart.icnType == MP2::OBJ_ICN_TYPE_UNKNOWN || isObjectPartPassable( _mainObjectPart ) )
+           && std::all_of( _groundObjectPart.begin(), _groundObjectPart.end(), isObjectPartPassable );
+}
+
 bool Maps::Tile::isPassabilityTransparent() const
 {
     for ( const auto & part : _groundObjectPart ) {
@@ -1265,7 +1301,7 @@ void Maps::Tile::updateFlag( const PlayerColor color, const uint8_t objectSprite
     }
 }
 
-void Maps::Tile::_updateRoadFlag()
+void Maps::Tile::updateRoadFlag()
 {
     _isTileMarkedAsRoad = isSpriteRoad( _mainObjectPart.icnType, _mainObjectPart.icnIndex );
 
@@ -1475,7 +1511,7 @@ void Maps::Tile::removeObjects( const MP2::ObjectIcnType objectIcnType )
         _mainObjectPart = {};
     }
 
-    _updateRoadFlag();
+    updateRoadFlag();
 
     // TODO: update tile's object type after objects' removal.
 }
@@ -1551,7 +1587,7 @@ void Maps::Tile::updateTileObjectIcnIndex( Maps::Tile & tile, const uint32_t uid
         tile._mainObjectPart.icnIndex = newIndex;
     }
 
-    tile._updateRoadFlag();
+    tile.updateRoadFlag();
 }
 
 void Maps::Tile::updateObjectType()
