@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,8 +21,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2MONSTER_H
-#define H2MONSTER_H
+#pragma once
 
 #include <cstdint>
 
@@ -51,7 +50,7 @@ public:
         LEVEL_4
     };
 
-    enum monster_t : int32_t
+    enum MonsterType : int32_t
     {
         UNKNOWN,
 
@@ -139,10 +138,18 @@ public:
     {
         // Do nothing.
     }
-    explicit Monster( const Spell & );
-    Monster( int race, uint32_t dw );
+
+    explicit Monster( const Spell & sp );
+    Monster( const int race, const uint32_t dw );
+
+    Monster( const Monster & ) = default;
+    Monster( Monster && ) = default;
+
     virtual ~Monster() = default;
 
+    Monster & operator=( const Monster & ) = default;
+    Monster & operator=( Monster && ) = default;
+
     bool operator==( const Monster & m ) const
     {
         return id == m.id;
@@ -153,7 +160,7 @@ public:
         return id != m.id;
     }
 
-    int GetID() const
+    int32_t GetID() const
     {
         return id;
     }
@@ -168,7 +175,6 @@ public:
 
     virtual uint32_t GetAttack() const;
     virtual uint32_t GetDefense() const;
-    virtual int GetColor() const;
     virtual int GetMorale() const;
     virtual int GetLuck() const;
     virtual int GetRace() const;
@@ -286,9 +292,11 @@ public:
 
     bool isAbilityPresent( const fheroes2::MonsterAbilityType abilityType ) const;
 
+    bool isWeaknessPresent( const fheroes2::MonsterWeaknessType weaknessType ) const;
+
     double GetMonsterStrength( int attack = -1, int defense = -1 ) const;
 
-    int ICNMonh() const;
+    int32_t ICNMonh() const;
 
     uint32_t GetSpriteIndex() const
     {
@@ -319,7 +327,5 @@ protected:
 
     static Monster FromDwelling( int race, uint32_t dw );
 
-    int id;
+    int32_t id;
 };
-
-#endif
