@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -92,7 +92,7 @@ namespace
 
 const char * Artifact::GetName() const
 {
-    return _( fheroes2::getArtifactData( id ).name );
+    return _( fheroes2::getArtifactData( id ).untranslatedName );
 }
 
 bool Artifact::isUltimate() const
@@ -431,13 +431,13 @@ int32_t Artifact::getSpellId() const
     return Spell::NONE;
 }
 
-int Artifact::Rand( ArtLevel lvl )
+int32_t Artifact::Rand( ArtLevel lvl )
 {
-    std::vector<int> v;
+    std::vector<int32_t> v;
     v.reserve( 25 );
 
     // if possibly: make unique on map
-    for ( int art = UNKNOWN + 1; art < ARTIFACT_COUNT; ++art ) {
+    for ( int32_t art = UNKNOWN + 1; art < ARTIFACT_COUNT; ++art ) {
         const Artifact artifact{ art };
 
         if ( artifact.isValid() && ( lvl & artifact.Level() ) && !( artifactGlobalStatus[art] & ART_RNDDISABLED ) && !( artifactGlobalStatus[art] & ART_RNDUSED ) ) {
@@ -446,7 +446,7 @@ int Artifact::Rand( ArtLevel lvl )
     }
 
     if ( v.empty() ) {
-        for ( int art = UNKNOWN + 1; art < ARTIFACT_COUNT; ++art ) {
+        for ( int32_t art = UNKNOWN + 1; art < ARTIFACT_COUNT; ++art ) {
             const Artifact artifact{ art };
 
             if ( artifact.isValid() && ( lvl & artifact.Level() ) && !( artifactGlobalStatus[art] & ART_RNDDISABLED ) ) {
@@ -455,7 +455,7 @@ int Artifact::Rand( ArtLevel lvl )
         }
     }
 
-    int res = !v.empty() ? Rand::Get( v ) : Artifact::UNKNOWN;
+    const int32_t res = !v.empty() ? Rand::Get( v ) : Artifact::UNKNOWN;
     artifactGlobalStatus[res] |= ART_RNDUSED;
 
     return res;
@@ -501,7 +501,7 @@ Artifact Artifact::getArtifactFromMapSpriteIndex( const uint32_t index )
 
 const char * Artifact::getDiscoveryDescription( const Artifact & art )
 {
-    return _( fheroes2::getArtifactData( art.GetID() ).discoveryEventDescription );
+    return _( fheroes2::getArtifactData( art.GetID() ).untranslatedDiscoveryEventDescription );
 }
 
 OStreamBase & operator<<( OStreamBase & stream, const Artifact & art )
