@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -20,9 +20,10 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifndef H2ARTIFACT_H
-#define H2ARTIFACT_H
 
+#pragma once
+
+#include <cstddef>
 #include <cstdint>
 #include <set>
 #include <string>
@@ -33,17 +34,23 @@
 #include "image.h"
 #include "interface_itemsbar.h"
 #include "math_base.h"
-#include "mp2.h"
 #include "ui_tool.h"
 
+class IStreamBase;
+class OStreamBase;
+
 class Heroes;
 class StatusBar;
-class StreamBase;
+
+namespace MP2
+{
+    enum MapObjectType : uint16_t;
+}
 
 class Artifact
 {
 public:
-    enum level_t
+    enum ArtLevel
     {
         ART_NONE = 0,
         ART_LEVEL_TREASURE = 0x01,
@@ -57,7 +64,7 @@ public:
 
     // All artifact IDs are by value 1 bigger than in the original game.
     // This is done to support new artifact addition and also align with the rest of object types.
-    enum type_t : int
+    enum : int32_t
     {
         UNKNOWN = 0,
 
@@ -146,10 +153,12 @@ public:
 
         MAGIC_BOOK,
 
+        // This entry is used in Editor for the special victory conditions.
+        EDITOR_ANY_ULTIMATE_ARTIFACT,
+
         // These four artifacts are used in original editor for the Ultimate and Random artifacts.
         // They should not exist and used on the adventure map after the game is properly started.
         // We do not use these or create extra entries for the Ultimate and Random artifacts in editor.
-        UNUSED_83,
         UNUSED_84,
         UNUSED_85,
         UNUSED_86,
@@ -179,9 +188,8 @@ public:
         ARTIFACT_COUNT
     };
 
-    Artifact( int art = UNKNOWN )
+    Artifact( int32_t art = UNKNOWN )
         : id( art > UNKNOWN && art < ARTIFACT_COUNT ? art : UNKNOWN )
-        , ext( 0 )
     {
         // Do nothing.
     }
@@ -210,7 +218,7 @@ public:
 
     bool isValid() const
     {
-        return id != UNKNOWN && id < ARTIFACT_COUNT && ( id < UNUSED_83 || id > UNUSED_86 );
+        return id != UNKNOWN && id != EDITOR_ANY_ULTIMATE_ARTIFACT && id < ARTIFACT_COUNT && ( id < UNUSED_84 || id > UNUSED_86 );
     }
 
     void Reset()
@@ -250,21 +258,18 @@ public:
         return fheroes2::getArtifactData( id ).getDescription( ext );
     }
 
-    static int Rand( level_t );
+    static int32_t Rand( ArtLevel );
     static Artifact getArtifactFromMapSpriteIndex( const uint32_t index );
     static const char * getDiscoveryDescription( const Artifact & );
 
 private:
-    friend StreamBase & operator<<( StreamBase &, const Artifact & );
-    friend StreamBase & operator>>( StreamBase &, Artifact & );
+    friend OStreamBase & operator<<( OStreamBase & stream, const Artifact & art );
+    friend IStreamBase & operator>>( IStreamBase & stream, Artifact & art );
 
-    int id;
-    int ext;
+    int32_t id;
+    int32_t ext{ 0 };
 };
 
-StreamBase & operator<<( StreamBase &, const Artifact & );
-StreamBase & operator>>( StreamBase &, Artifact & );
-
 uint32_t GoldInsteadArtifact( const MP2::MapObjectType objectType );
 
 namespace fheroes2
@@ -293,10 +298,13 @@ struct ArtifactSetData
 class BagArtifacts : public std::vector<Artifact>
 {
 public:
+    // Maximum number of artifacts that can be placed in an artifact bag
+    static constexpr size_t maxCapacity{ 14 };
+
     BagArtifacts();
 
     bool ContainSpell( const int spellId ) const;
-    bool isPresentArtifact( const Artifact & ) const;
+    bool isPresentArtifact( const Artifact & art ) const;
 
     bool isArtifactBonusPresent( const fheroes2::ArtifactBonusType type ) const;
     bool isArtifactCursePresent( const fheroes2::ArtifactCurseType type ) const;
@@ -316,7 +324,7 @@ public:
     Artifact getFirstArtifactWithBonus( const fheroes2::ArtifactBonusType bonus ) const;
     Artifact getFirstArtifactWithCurse( const fheroes2::ArtifactCurseType curse ) const;
 
-    bool PushArtifact( const Artifact & );
+    bool PushArtifact( const Artifact & art );
 
     // Removes the first found instance of the specified artifact from the artifact bag
     void RemoveArtifact( const Artifact & art );
@@ -325,11 +333,11 @@ public:
     bool ContainUltimateArtifact() const;
 
     // Automatically exchange artifacts between two heroes. The taker should get the best possible artifacts.
-    void exchangeArtifacts( BagArtifacts & giftBag, const Heroes & taker, const Heroes & giver );
+    static void exchangeArtifacts( Heroes & taker, Heroes & giver );
 
     double getArtifactValue() const;
     uint32_t CountArtifacts() const;
-    uint32_t Count( const Artifact & ) const;
+    uint32_t Count( const Artifact & art ) const;
 
     std::set<ArtifactSetData> assembleArtifactSetIfPossible();
 
@@ -342,7 +350,7 @@ public:
     using Interface::ItemsActionBar<Artifact>::RedrawItem;
     using Interface::ItemsActionBar<Artifact>::ActionBarRightMouseHold;
 
-    ArtifactsBar( const Heroes * hero, const bool mini, const bool ro, const bool change, const bool allowOpeningMagicBook, StatusBar * bar );
+    ArtifactsBar( Heroes * hero, const bool mini, const bool ro, const bool change, const bool allowOpeningMagicBook, StatusBar * bar );
 
     void RedrawBackground( const fheroes2::Rect &, fheroes2::Image & ) override;
     void RedrawItem( Artifact &, const fheroes2::Rect &, bool, fheroes2::Image & ) override;
@@ -365,7 +373,7 @@ protected:
     fheroes2::MovableSprite spcursor;
 
 private:
-    const Heroes * _hero;
+    Heroes * _hero;
     fheroes2::Image backsf;
     const bool use_mini_sprite;
     const bool read_only;
@@ -378,5 +386,3 @@ private:
 
     void messageMagicBookAbortTrading() const;
 };
-
-#endif
