@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -20,8 +20,8 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifndef H2RESOURCE_H
-#define H2RESOURCE_H
+
+#pragma once
 
 #include <cstdint>
 #include <limits>
@@ -94,10 +94,8 @@ struct Funds
     Funds operator+( const Funds & pm ) const;
     Funds operator*( uint32_t mul ) const;
     Funds operator-( const Funds & pm ) const;
-    Funds operator/( const int32_t div ) const;
     Funds & operator+=( const Funds & pm );
     Funds & operator*=( uint32_t mul );
-    Funds & operator/=( const int32_t div );
     Funds & operator-=( const Funds & pm );
     Funds & operator=( const Cost & cost );
 
@@ -212,5 +210,3 @@ namespace Resource
         }
     }
 }
-
-#endif
