@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -73,7 +73,8 @@ namespace
         { gettext_noop( "Mass Haste" ), 10, 0, 0, 61, 2, gettext_noop( "Increases the speed of all of your creatures by %{count}." ) },
         { gettext_noop( "spell|Slow" ), 3, 0, 0, 1, 0, gettext_noop( "Slows target to half movement rate." ) },
         { gettext_noop( "Mass Slow" ), 15, 0, 0, 62, 0, gettext_noop( "Slows all enemies to half movement rate." ) },
-        { gettext_noop( "spell|Blind" ), 6, 0, 0, 21, 0, gettext_noop( "Clouds the affected creatures' eyes, preventing them from moving." ) },
+        { gettext_noop( "spell|Blind" ), 6, 0, 0, 21, 50,
+          gettext_noop( "Clouds the affected creatures' eyes, preventing them from moving and reduces the damage when retaliating by %{count} percent." ) },
         { gettext_noop( "Bless" ), 3, 0, 0, 7, 0, gettext_noop( "Causes the selected creatures to inflict maximum damage." ) },
         { gettext_noop( "Mass Bless" ), 12, 0, 0, 63, 0, gettext_noop( "Causes all of your units to inflict maximum damage." ) },
         { gettext_noop( "Stoneskin" ), 3, 0, 0, 31, 3, gettext_noop( "Magically increases the defense skill of the selected creatures." ) },
@@ -85,7 +86,7 @@ namespace
         { gettext_noop( "Holy Shout" ), 12, 0, 0, 23, 20, gettext_noop( "Damages all undead in the battle. This is an improved version of Holy Word." ) },
         { gettext_noop( "Anti-Magic" ), 7, 0, 0, 17, 0, gettext_noop( "Prevents any magic against the selected creatures." ) },
         { gettext_noop( "Dispel Magic" ), 5, 0, 0, 18, 0, gettext_noop( "Removes all magic spells from a single target." ) },
-        { gettext_noop( "Mass Dispel" ), 12, 0, 0, 18, 0, gettext_noop( "Removes all magic spells from all creatures." ) },
+        { gettext_noop( "Mass Dispel" ), 12, 0, 0, 73, 0, gettext_noop( "Removes all magic spells from all creatures." ) },
         { gettext_noop( "Magic Arrow" ), 3, 0, 0, 38, 10, gettext_noop( "Causes a magic arrow to strike the selected target." ) },
         { gettext_noop( "Berserker" ), 12, 0, 0, 19, 0, gettext_noop( "Causes a creature to attack its nearest neighbor." ) },
         { gettext_noop( "Armageddon" ), 20, 0, 0, 16, 50, gettext_noop( "Holy terror strikes the battlefield, causing severe damage to all creatures." ) },
@@ -127,7 +128,8 @@ namespace
           gettext_noop(
               "Summons the nearest unoccupied, friendly boat to an adjacent shore location. A friendly boat is one which you just built or were the most recent player to occupy." ) },
         { gettext_noop( "Dimension Door" ), 10, 225, 69, 47, 0, gettext_noop( "Allows the caster to magically transport to a nearby location." ) },
-        { gettext_noop( "Town Gate" ), 10, 225, 69, 48, 0, gettext_noop( "Returns the caster to any town or castle currently owned." ) },
+        { gettext_noop( "Town Gate" ), 10, 225, 69, 48, 0,
+          gettext_noop( "Returns the caster to the nearest town or castle currently owned. This spell cannot be cast if the hero is already in a town or a castle." ) },
         { gettext_noop( "Town Portal" ), 20, 225, 69, 49, 0, gettext_noop( "Returns the hero to the town or castle of choice, provided it is controlled by you." ) },
         { gettext_noop( "Visions" ), 6, 0, 0, 50, 3, gettext_noop( "Visions predicts the likely outcome of an encounter with a neutral army camp." ) },
         { gettext_noop( "Haunt" ), 8, 0, 0, 51, 4,
@@ -215,10 +217,14 @@ uint32_t Spell::spellPoints( const HeroBase * hero ) const
     return static_cast<uint32_t>( spellCost );
 }
 
-double Spell::getStrategicValue( double armyStrength, uint32_t currentSpellPoints, int spellPower ) const
+double Spell::getStrategicValue( const double armyStrength, const uint32_t currentSpellPoints, const int spellPower ) const
 {
     const uint32_t spellCost = spellPoints();
-    const uint32_t casts = spellCost ? std::min( 10U, currentSpellPoints / spellCost ) : 0;
+    const uint32_t casts = spellCost ? std::min<uint32_t>( 10U, currentSpellPoints / spellCost ) : 0;
+    if ( casts == 0 ) {
+        // The spell cannot be casted.
+        return 0;
+    }
 
     // use quadratic formula to diminish returns from subsequent spell casts, (up to x5 when spell has 10 uses)
     const double amountModifier = ( casts == 1 ) ? 1 : casts - ( 0.05 * casts * casts );
@@ -513,42 +519,37 @@ uint32_t Spell::weightForRace( const int race ) const
 
 Spell Spell::getRandomSpell( const int level )
 {
-    assert( level > 0 && level < 6 );
+    assert( level >= 1 && level <= 5 );
 
-    std::vector<Spell> validSpells;
-    validSpells.reserve( Spell::SPELL_COUNT );
-
-    for ( int32_t spellId = NONE; spellId < PETRIFY; ++spellId ) {
-        const Spell spell( spellId );
-
-        if ( level != spell.Level() ) {
-            continue;
-        }
-
-        validSpells.push_back( spell );
-    }
+    const std::vector<int32_t> validSpells = getAllSpellIdsSuitableForSpellBook( level );
 
     assert( !validSpells.empty() );
 
-    return validSpells.empty() ? Spell::NONE : Rand::Get( validSpells );
+    return validSpells.empty() ? Spell::NONE : Spell( Rand::Get( validSpells ) );
 }
 
-std::vector<int> Spell::getAllSpellIdsSuitableForSpellBook( const int spellLevel /* = -1 */ )
+std::vector<int32_t> Spell::getAllSpellIdsSuitableForSpellBook( const int spellLevel /* = -1 */, const std::set<int32_t> & spellsToExclude /* = {} */ )
 {
-    std::vector<int> result;
-    result.reserve( SPELL_COUNT );
+    if ( spellLevel < -1 || spellLevel == 0 || spellLevel > 5 ) {
+        // Have you add a new spell level? Check your logic!
+        assert( 0 );
 
-    for ( int spellId = 0; spellId < SPELL_COUNT; ++spellId ) {
-        if ( spellId == NONE || ( spellId >= RANDOM && spellId <= PETRIFY ) ) {
+        return {};
+    }
+
+    std::vector<int32_t> result;
+    if ( spellLevel == -1 ) {
+        // Reserve memory only for vector with all spells.
+        result.reserve( SPELL_COUNT );
+    }
+
+    for ( int32_t spellId = NONE + 1; spellId < SPELL_COUNT; ++spellId ) {
+        if ( const int level = Spell( spellId ).Level(); ( spellLevel != -1 && level != spellLevel ) || level < 1 ) {
             continue;
         }
 
-        if ( spellLevel > 0 ) {
-            const Spell spell( spellId );
-
-            if ( spell.Level() != spellLevel ) {
-                continue;
-            }
+        if ( spellsToExclude.count( spellId ) != 0 ) {
+            continue;
         }
 
         result.push_back( spellId );
