@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2025                                             *
+ *   Copyright (C) 2019 - 2026                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -131,7 +131,7 @@ namespace
 
         const Maps::FileInfo & mapInfo = Settings::Get().getCurrentMapInfo();
 
-        // do not let the player get a random artifact that allows him to win the game
+        // Do not let the player get a random artifact that allows him to win the game.
         if ( ( mapInfo.ConditionWins() & GameOver::WINS_ARTIFACT ) == GameOver::WINS_ARTIFACT && !mapInfo.WinsFindUltimateArtifact() ) {
             fheroes2::ExcludeArtifactFromRandom( mapInfo.WinsFindArtifactID() );
         }
@@ -145,19 +145,20 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
 
     StreamFile fs;
     if ( !fs.open( filename, "rb" ) ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file not found " << filename )
+        ERROR_LOG( "Map file " << filename << " is corrupted or missing." )
         return false;
     }
 
     // Read magic number.
     if ( fs.getBE32() != 0x5C000000 ) {
         // It is not a MP2 or MX2 file.
+        ERROR_LOG( "File " << filename << " is not a valid map." )
         return false;
     }
 
     const size_t totalFileSize = fs.size();
     if ( totalFileSize < MP2::MP2_MAP_INFO_SIZE ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+        ERROR_LOG( "Map file " << filename << " is corrupted." )
         return false;
     }
 
@@ -207,7 +208,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
     const int32_t worldSize = width * height;
 
     if ( totalFileSize < MP2::MP2_MAP_INFO_SIZE + static_cast<size_t>( worldSize ) * MP2::MP2_TILE_STRUCTURE_SIZE + MP2::MP2_ADDON_COUNT_SIZE ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+        ERROR_LOG( "Map file " << filename << " is corrupted." )
         return false;
     }
 
@@ -220,7 +221,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
 
     if ( totalFileSize < MP2::MP2_MAP_INFO_SIZE + static_cast<size_t>( worldSize ) * MP2::MP2_TILE_STRUCTURE_SIZE + addonCount * MP2::MP2_ADDON_STRUCTURE_SIZE
                              + MP2::MP2_ADDON_COUNT_SIZE ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+        ERROR_LOG( "Map file " << filename << " is corrupted." )
         return false;
     }
 
@@ -319,7 +320,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
     fs.seek( afterAddonInfoPos );
 
     if ( totalFileSize < afterAddonInfoPos + static_cast<size_t>( MP2::MP2_CASTLE_COUNT * MP2::MP2_CASTLE_POSITION_SIZE ) ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+        ERROR_LOG( "Map file " << filename << " is corrupted." )
         return false;
     }
 
@@ -381,7 +382,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
     if ( totalFileSize
          < afterAddonInfoPos
                + static_cast<size_t>( MP2::MP2_CASTLE_COUNT * MP2::MP2_CASTLE_POSITION_SIZE + MP2::MP2_CAPTURE_OBJECT_COUNT * MP2::MP2_CAPTURE_OBJECT_POSITION_SIZE ) ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+        ERROR_LOG( "Map file " << filename << " is corrupted." )
         return false;
     }
 
@@ -446,7 +447,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
         const uint32_t h = fs.get();
 
         if ( fs.tell() == fs.size() ) {
-            DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file " << filename << " is corrupted" )
+            ERROR_LOG( "Map file " << filename << " is corrupted." )
             return false;
         }
 
@@ -714,10 +715,20 @@ bool World::loadResurrectionMap( const std::string & filename )
 
     Maps::Map_Format::MapFormat map;
     if ( !Maps::Map_Format::loadMap( filename, map ) ) {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map file '" << filename << "' is corrupted or missing." )
+        ERROR_LOG( "Map file '" << filename << "' is corrupted or missing." )
         return false;
     }
 
+    const auto currentLanguage = fheroes2::getCurrentLanguage();
+    if ( !Maps::loadTranslation( map, currentLanguage ) ) {
+        // The current game language is not supported by the map.
+        // Try to set the default language - English.
+        // Even if it fails, the first language of the map is going to be used.
+        if ( currentLanguage != fheroes2::SupportedLanguage::English ) {
+            Maps::loadTranslation( map, fheroes2::SupportedLanguage::English );
+        }
+    }
+
     width = map.width;
     height = map.width;
 
@@ -746,15 +757,18 @@ bool World::loadResurrectionMap( const std::string & filename )
     const auto & treasuresObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_TREASURES );
     const auto & powerUpsObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_POWER_UPS );
     const auto & minesObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MINES );
+    const auto & monsterObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::MONSTERS );
 
 #if defined( WITH_DEBUG )
-    std::set<uint32_t> standardMetadataUIDs;
+    std::set<uint32_t> resourceMetadataUIDs;
     std::set<uint32_t> castleMetadataUIDs;
     std::set<uint32_t> heroMetadataUIDs;
     std::set<uint32_t> sphinxMetadataUIDs;
     std::set<uint32_t> signMetadataUIDs;
     std::set<uint32_t> adventureMapEventMetadataUIDs;
     std::set<uint32_t> selectionObjectMetadataUIDs;
+    std::set<uint32_t> monsterMetadataUIDs;
+    std::set<uint32_t> artifactMetadataUIDs;
 #endif
 
     const auto areSpellsValid = []( const Maps::Map_Format::SelectionObjectMetadata & metadata, const int spellLevel ) {
@@ -875,15 +889,49 @@ bool World::loadResurrectionMap( const std::string & filename )
             }
             else if ( object.group == Maps::ObjectGroup::MONSTERS ) {
 #if defined( WITH_DEBUG )
-                standardMetadataUIDs.emplace( object.id );
+                monsterMetadataUIDs.emplace( object.id );
 #endif
-                assert( map.standardMetadata.find( object.id ) != map.standardMetadata.end() );
-                auto & objectInfo = map.standardMetadata[object.id];
+                assert( map.monsterMetadata.find( object.id ) != map.monsterMetadata.end() );
+                auto & objectInfo = map.monsterMetadata[object.id];
 
                 std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
+                tileData[0] = static_cast<uint32_t>( objectInfo.count );
 
-                for ( size_t idx = 0; idx < objectInfo.metadata.size(); ++idx ) {
-                    tileData[idx] = static_cast<uint32_t>( objectInfo.metadata[idx] );
+                switch ( monsterObjects[object.index].objectType ) {
+                case MP2::OBJ_RANDOM_MONSTER:
+                case MP2::OBJ_RANDOM_MONSTER_MEDIUM:
+                case MP2::OBJ_RANDOM_MONSTER_STRONG:
+                case MP2::OBJ_RANDOM_MONSTER_VERY_STRONG:
+                case MP2::OBJ_RANDOM_MONSTER_WEAK: {
+                    auto & selected = objectInfo.selected;
+                    if ( selected.empty() ) {
+                        // Nothing to do. Use the default behavior.
+                        break;
+                    }
+
+                    // Verify that the input data is correct.
+                    if ( monsterObjects[object.index].objectType == MP2::OBJ_RANDOM_MONSTER ) {
+                        selected.erase( std::remove_if( selected.begin(), selected.end(), []( const int value ) { return !Monster{ value }.isValid(); } ),
+                                        selected.end() );
+                    }
+                    else {
+                        const auto level = Monster{ static_cast<int>( object.index ) + 1 }.GetRandomUnitLevel();
+
+                        selected.erase( std::remove_if( selected.begin(), selected.end(),
+                                                        [level]( const int value ) {
+                                                            const Monster temp{ value };
+                                                            return !temp.isValid() || temp.GetRandomUnitLevel() != level;
+                                                        } ),
+                                        selected.end() );
+                    }
+
+                    if ( !objectInfo.selected.empty() ) {
+                        tileData[1] = Rand::Get( objectInfo.selected );
+                    }
+                    break;
+                }
+                default:
+                    break;
                 }
             }
             else if ( object.group == Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS ) {
@@ -1046,7 +1094,7 @@ bool World::loadResurrectionMap( const std::string & filename )
                             return false;
                         }
 
-                        const std::vector<int32_t> & skills = GameStatic::getSecondarySkillsForWitchsHut();
+                        const std::vector<int32_t> skills = GameStatic::getSecondarySkillsForWitchsHut();
 
                         return std::all_of( metadata.selectedItems.begin(), metadata.selectedItems.end(),
                                             [&skills]( const int32_t skillId ) { return std::find( skills.begin(), skills.end(), skillId ) != skills.end(); } );
@@ -1092,42 +1140,43 @@ bool World::loadResurrectionMap( const std::string & filename )
             }
             else if ( object.group == Maps::ObjectGroup::ADVENTURE_ARTIFACTS ) {
 #if defined( WITH_DEBUG )
-                standardMetadataUIDs.emplace( object.id );
+                artifactMetadataUIDs.emplace( object.id );
 #endif
 
-                assert( map.standardMetadata.find( object.id ) != map.standardMetadata.end() );
-                auto & objectInfo = map.standardMetadata[object.id];
+                assert( map.artifactMetadata.find( object.id ) != map.artifactMetadata.end() );
+                auto & artifactInfo = map.artifactMetadata[object.id];
 
                 std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
 
-                for ( size_t idx = 0; idx < objectInfo.metadata.size(); ++idx ) {
-                    tileData[idx] = static_cast<uint32_t>( objectInfo.metadata[idx] );
-                }
-
                 assert( object.index < artifactObjects.size() );
-                if ( artifactObjects[object.index].objectType == MP2::OBJ_ARTIFACT && artifactObjects[object.index].metadata[0] == Artifact::SPELL_SCROLL ) {
+                const auto & objectInfo = artifactObjects[object.index];
+
+                if ( objectInfo.objectType == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT ) {
+                    tileData[0] = static_cast<uint32_t>( artifactInfo.radius );
+                }
+                else if ( objectInfo.objectType == MP2::OBJ_ARTIFACT && objectInfo.metadata[0] == Artifact::SPELL_SCROLL ) {
+                    assert( artifactInfo.selected.size() == 1 );
+
                     // This is a hack we need to do since in the original game spells start from 0.
                     // TODO: fix this hack.
-                    assert( tileData[0] > 0 );
-                    tileData[0] = tileData[0] - 1U;
+                    assert( artifactInfo.selected.front() > 0 );
+                    tileData[0] = static_cast<uint32_t>( artifactInfo.selected.front() ) - 1U;
                 }
             }
             else if ( object.group == Maps::ObjectGroup::ADVENTURE_TREASURES ) {
                 assert( object.index < treasuresObjects.size() );
-
                 const auto & objectInfo = treasuresObjects[object.index];
 
-                if ( objectInfo.objectType == MP2::OBJ_RESOURCE && map.standardMetadata.find( object.id ) != map.standardMetadata.end() ) {
-                    // Some maps may have resource objects being set by older Editor versions.
-                    // Therefore, we cannot have a strict check whether metadata for this object exists like we do for other objects.
+                if ( objectInfo.objectType == MP2::OBJ_RESOURCE && map.resourceMetadata.find( object.id ) != map.resourceMetadata.end() ) {
+                    // Some maps might be corrupted or broken due to issues in the older versions of the Editor.
 #if defined( WITH_DEBUG )
-                    standardMetadataUIDs.emplace( object.id );
+                    resourceMetadataUIDs.emplace( object.id );
 #endif
 
                     std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
 
                     tileData[0] = objectInfo.metadata[0];
-                    tileData[1] = map.standardMetadata[object.id].metadata[0];
+                    tileData[1] = map.resourceMetadata[object.id].count;
                 }
             }
             else if ( object.group == Maps::ObjectGroup::ADVENTURE_POWER_UPS ) {
@@ -1183,16 +1232,18 @@ bool World::loadResurrectionMap( const std::string & filename )
     }
 
 #if defined( WITH_DEBUG )
-    assert( standardMetadataUIDs.size() == map.standardMetadata.size() );
+    assert( resourceMetadataUIDs.size() == map.resourceMetadata.size() );
     assert( castleMetadataUIDs.size() == map.castleMetadata.size() );
     assert( heroMetadataUIDs.size() == map.heroMetadata.size() );
     assert( sphinxMetadataUIDs.size() == map.sphinxMetadata.size() );
     assert( signMetadataUIDs.size() == map.signMetadata.size() );
     assert( adventureMapEventMetadataUIDs.size() == map.adventureMapEventMetadata.size() );
     assert( selectionObjectMetadataUIDs.size() == map.selectionObjectMetadata.size() );
+    assert( monsterMetadataUIDs.size() == map.monsterMetadata.size() );
+    assert( artifactMetadataUIDs.size() == map.artifactMetadata.size() );
 
-    for ( const uint32_t uid : standardMetadataUIDs ) {
-        assert( map.standardMetadata.find( uid ) != map.standardMetadata.end() );
+    for ( const uint32_t uid : resourceMetadataUIDs ) {
+        assert( map.resourceMetadata.find( uid ) != map.resourceMetadata.end() );
     }
 
     for ( const uint32_t uid : castleMetadataUIDs ) {
@@ -1218,6 +1269,14 @@ bool World::loadResurrectionMap( const std::string & filename )
     for ( const uint32_t uid : selectionObjectMetadataUIDs ) {
         assert( map.selectionObjectMetadata.find( uid ) != map.selectionObjectMetadata.end() );
     }
+
+    for ( const uint32_t uid : monsterMetadataUIDs ) {
+        assert( map.monsterMetadata.find( uid ) != map.monsterMetadata.end() );
+    }
+
+    for ( const uint32_t uid : artifactMetadataUIDs ) {
+        assert( map.artifactMetadata.find( uid ) != map.artifactMetadata.end() );
+    }
 #endif
 
     // Load daily events.
@@ -1261,7 +1320,7 @@ bool World::loadResurrectionMap( const std::string & filename )
     if ( map.lossConditionType == Maps::FileInfo::LOSS_HERO ) {
         auto iter = hiredHeroTileId.find( map.lossConditionMetadata[0] );
         if ( iter == hiredHeroTileId.end() ) {
-            VERBOSE_LOG( "A hero at tile " << map.lossConditionMetadata[0] << " does not exist." )
+            ERROR_LOG( "A Loss Condition hero at tile " << map.lossConditionMetadata[0] << " does not exist." )
             return false;
         }
     }
@@ -1269,7 +1328,7 @@ bool World::loadResurrectionMap( const std::string & filename )
         const Castle * castle
             = vec_castles.Get( { static_cast<int32_t>( map.lossConditionMetadata[0] % map.width ), static_cast<int32_t>( map.lossConditionMetadata[0] / map.width ) } );
         if ( castle == nullptr ) {
-            VERBOSE_LOG( "A castle at tile " << map.lossConditionMetadata[0] << " does not exist." )
+            ERROR_LOG( "A Loss Condition castle at tile " << map.lossConditionMetadata[0] << " does not exist." )
             return false;
         }
     }
@@ -1277,7 +1336,7 @@ bool World::loadResurrectionMap( const std::string & filename )
     if ( map.victoryConditionType == Maps::FileInfo::VICTORY_KILL_HERO ) {
         auto iter = hiredHeroTileId.find( map.victoryConditionMetadata[0] );
         if ( iter == hiredHeroTileId.end() ) {
-            VERBOSE_LOG( "A hero at tile " << map.victoryConditionMetadata[0] << " does not exist." )
+            ERROR_LOG( "A Victory Condition hero at tile " << map.victoryConditionMetadata[0] << " does not exist." )
             return false;
         }
     }
@@ -1285,7 +1344,7 @@ bool World::loadResurrectionMap( const std::string & filename )
         const Castle * castle = vec_castles.Get(
             { static_cast<int32_t>( map.victoryConditionMetadata[0] % map.width ), static_cast<int32_t>( map.victoryConditionMetadata[0] / map.width ) } );
         if ( castle == nullptr ) {
-            VERBOSE_LOG( "A castle at tile " << map.victoryConditionMetadata[0] << " does not exist." )
+            ERROR_LOG( "A Victory Condition castle at tile " << map.victoryConditionMetadata[0] << " does not exist." )
             return false;
         }
     }
@@ -1294,7 +1353,7 @@ bool World::loadResurrectionMap( const std::string & filename )
 
     updateArtifactStats();
 
-    if ( !ProcessNewMP2Map( filename, false ) ) {
+    if ( !_processNewResurrectionMap( filename ) ) {
         return false;
     }
 
@@ -1336,34 +1395,51 @@ bool World::ProcessNewMP2Map( const std::string & filename, const bool checkPoLO
         }
     }
 
-    // add heroes to kingdoms
+    // Add heroes and castles to kingdoms.
     vec_kingdoms.AddHeroes( vec_heroes );
-
-    // add castles to kingdoms
     vec_kingdoms.AddCastles( vec_castles );
 
-    setHeroIdsForMapConditions();
-
-    // Search for a tile with a predefined Ultimate Artifact
-    const auto ultArtTileIter
-        = std::find_if( vec_tiles.begin(), vec_tiles.end(), []( const Maps::Tile & tile ) { return tile.getMainObjectType() == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT; } );
-    int32_t ultimateArtifactTileId = -1;
-    int32_t ultimateArtifactRadius = 0;
-    if ( ultArtTileIter != vec_tiles.end() ) {
-        ultimateArtifactTileId = ultArtTileIter->GetIndex();
-        ultimateArtifactRadius = static_cast<int32_t>( ultArtTileIter->metadata()[0] );
-
-        // Remove the predefined Ultimate Artifact object
-        ultArtTileIter->removeObjectPartsByUID( ultArtTileIter->getMainObjectPart()._uid );
+    if ( !setHeroIdsForMapConditions() ) {
+        return false;
     }
 
-    setUltimateArtifact( ultimateArtifactTileId, ultimateArtifactRadius );
+    // Set up Ultimate Artifact.
+    setUltimateArtifact();
 
     PostLoad( true, false );
 
     vec_kingdoms.ApplyPlayWithStartingHero();
 
-    addDebugHero();
+    tryAddDebugHero();
+
+    return true;
+}
+
+bool World::_processNewResurrectionMap( const std::string & filename )
+{
+    for ( Maps::Tile & tile : vec_tiles ) {
+        if ( !updateTileMetadata( tile, tile.getMainObjectType(), false ) ) {
+            ERROR_LOG( "Failed to load Resurrection map '" << filename << "'." )
+            return false;
+        }
+    }
+
+    // Add heroes and castles to kingdoms.
+    vec_kingdoms.AddHeroes( vec_heroes );
+    vec_kingdoms.AddCastles( vec_castles );
+
+    if ( !setHeroIdsForMapConditions() ) {
+        return false;
+    }
+
+    // Set up Ultimate Artifact.
+    setUltimateArtifact();
+
+    PostLoad( true, false );
+
+    vec_kingdoms.ApplyPlayWithStartingHero();
+
+    tryAddDebugHero();
 
     return true;
 }
@@ -1503,9 +1579,37 @@ bool World::updateTileMetadata( Maps::Tile & tile, const MP2::MapObjectType obje
     return true;
 }
 
-void World::setUltimateArtifact( const int32_t tileId, const int32_t radius )
+void World::setUltimateArtifact()
 {
-    assert( radius >= 0 );
+    int32_t tileId = -1;
+    int32_t radius = 0;
+
+    const auto existingUltimateArtIter
+        = std::find_if( vec_tiles.begin(), vec_tiles.end(), []( const auto & tile ) { return tile.getMainObjectType() == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT; } );
+    if ( existingUltimateArtIter != vec_tiles.end() ) {
+        // An Ultimate Artifact exists on this map.
+#if defined( WITH_DEBUG )
+        // We need to make sure that only 1 Ultimate Artifact exists on the map.
+        auto artifactCount
+            = std::count_if( vec_tiles.begin(), vec_tiles.end(), []( const auto & tile ) { return tile.getMainObjectType() == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT; } );
+        assert( artifactCount == 1 );
+#endif
+
+        tileId = existingUltimateArtIter->GetIndex();
+        radius = static_cast<int32_t>( existingUltimateArtIter->metadata()[0] );
+
+        assert( radius >= 0 );
+
+        // Remove the predefined Ultimate Artifact object.
+        const uint32_t uid = existingUltimateArtIter->getMainObjectPart()._uid;
+        existingUltimateArtIter->removeObjectPartsByUID( uid );
+
+        // Also, remove the shadow of the Artifact.
+        if ( Maps::isValidDirection( tileId, Direction::LEFT ) ) {
+            const int32_t neighborTileIndex = Maps::GetDirectionIndex( tileId, Direction::LEFT );
+            vec_tiles[neighborTileIndex].removeObjectPartsByUID( uid );
+        }
+    }
 
     const auto checkTileForSuitabilityForUltArt = [this]( const int32_t idx ) {
         const int32_t x = idx % width;
@@ -1537,12 +1641,12 @@ void World::setUltimateArtifact( const int32_t tileId, const int32_t radius )
         if ( !pool.empty() ) {
             const int32_t pos = Rand::Get( pool );
 
-            ultimate_artifact.Set( pos, getUltimateArtifact() );
+            _ultimateArtifact.Set( pos, getUltimateArtifact() );
 
-            DEBUG_LOG( DBG_GAME, DBG_INFO, "Ultimate Artifact index: " << pos )
+            DEBUG_LOG( DBG_GAME, DBG_INFO, "Ultimate Artifact has been placed at tile " << pos )
         }
         else {
-            DEBUG_LOG( DBG_GAME, DBG_WARN, "no suitable tile to place the Ultimate Artifact was found" )
+            DEBUG_LOG( DBG_GAME, DBG_WARN, "No suitable tile to place the Ultimate Artifact was found" )
         }
 
         return;
@@ -1564,17 +1668,21 @@ void World::setUltimateArtifact( const int32_t tileId, const int32_t radius )
                                     [&checkTileForSuitabilityForUltArt]( const int32_t idx ) { return !checkTileForSuitabilityForUltArt( idx ); } ),
                     pool.end() );
 
+        // If this assertion blows up then the map is corrupted.
+        // The Editor has the same Ultimate Artifact placement conditions as the game.
+        assert( !pool.empty() );
+
         if ( !pool.empty() ) {
             pos = Rand::Get( pool );
         }
     }
 
-    ultimate_artifact.Set( pos, getUltimateArtifact() );
+    _ultimateArtifact.Set( pos, getUltimateArtifact() );
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, "Predefined Ultimate Artifact tile index: " << tileId << ", radius: " << radius << ", final tile index: " << pos )
 }
 
-void World::addDebugHero()
+void World::tryAddDebugHero()
 {
     if ( !IS_DEVEL() ) {
         return;
@@ -1599,7 +1707,7 @@ void World::addDebugHero()
     }
 }
 
-void World::setHeroIdsForMapConditions()
+bool World::setHeroIdsForMapConditions()
 {
     const Maps::FileInfo & mapInfo = Settings::Get().getCurrentMapInfo();
 
@@ -1608,13 +1716,11 @@ void World::setHeroIdsForMapConditions()
 
         const Heroes * hero = GetHeroes( pos );
         if ( hero == nullptr ) {
-            heroIdAsWinCondition = Heroes::UNKNOWN;
-
             ERROR_LOG( "The hero whose defeat is a game win condition was not found at the position ['" << pos.x << ", " << pos.y << "']." )
+            return false;
         }
-        else {
-            heroIdAsWinCondition = hero->GetID();
-        }
+
+        heroIdAsWinCondition = hero->GetID();
     }
 
     if ( GameOver::LOSS_HERO & mapInfo.ConditionLoss() ) {
@@ -1622,14 +1728,14 @@ void World::setHeroIdsForMapConditions()
 
         Heroes * hero = GetHeroes( pos );
         if ( hero == nullptr ) {
-            heroIdAsLossCondition = Heroes::UNKNOWN;
-
             ERROR_LOG( "The hero whose defeat is a game loss condition was not found at the position ['" << pos.x << ", " << pos.y << "']." )
+            return false;
         }
-        else {
-            heroIdAsLossCondition = hero->GetID();
 
-            hero->SetModes( Heroes::NOTDISMISS | Heroes::CUSTOM );
-        }
+        heroIdAsLossCondition = hero->GetID();
+
+        hero->SetModes( Heroes::NOTDISMISS | Heroes::CUSTOM );
     }
+
+    return true;
 }
