@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2020 - 2025                                             *
+ *   Copyright (C) 2020 - 2026                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,6 +21,7 @@
 #include "world_pathfinding.h"
 
 #include <algorithm>
+#include <array>
 #include <cassert>
 #include <cmath>
 #include <cstdlib>
@@ -372,7 +373,7 @@ void WorldPathfinder::reset()
     if ( const size_t worldSize = world.getSize(); _cache.size() != worldSize ) {
         _cache.resize( worldSize );
 
-        const Directions & directions = Direction::All();
+        const auto & directions = Direction::allNeighboringDirections;
         _mapOffset.resize( directions.size() );
 
         for ( size_t i = 0; i < directions.size(); ++i ) {
@@ -406,7 +407,7 @@ void WorldPathfinder::processWorldMap()
 
 void WorldPathfinder::checkAdjacentNodes( std::vector<int> & nodesToExplore, const int currentNodeIdx )
 {
-    const Directions & directions = Direction::All();
+    const auto & directions = Direction::allNeighboringDirections;
     const WorldNode & currentNode = _cache[currentNodeIdx];
     const uint32_t maxMovePoints = getMaxMovePoints( world.getTile( currentNodeIdx ).isWater() );
 
@@ -584,7 +585,7 @@ void AIWorldPathfinder::reEvaluateIfNeeded( const Heroes & hero )
     // need to reserve SP can be neglected.
     const bool isDimensionDoorSpellAvailable = hero.HaveSpell( dimensionDoor );
 
-    const int32_t townGateCastleIndex = [this, &hero]() {
+    const int32_t townGateCastleIndex = [this, &hero]() -> int32_t {
         static const Spell townGate( Spell::TOWNGATE );
 
         if ( !hero.CanCastSpell( townGate ) ) {
@@ -886,7 +887,7 @@ std::pair<int32_t, bool> AIWorldPathfinder::getFogDiscoveryTile( const Heroes &
     reEvaluateIfNeeded( hero );
 
     const auto findBestTile = [this, scoutingDistance = hero.GetScoutingDistance()]( const auto nearbyTilePredicate ) {
-        const Directions & directions = Direction::All();
+        const auto & directions = Direction::allNeighboringDirections;
 
         struct TileCharacteristics
         {
@@ -970,9 +971,9 @@ int AIWorldPathfinder::getNearestTileToMove( const Heroes & hero )
 
     const int start = hero.GetIndex();
 
-    Directions directions = Direction::All();
+    auto directions{ Direction::allNeighboringDirections };
     // We have to shuffle directions to avoid cases when heroes repeat the same steps again and again.
-    Rand::Shuffle( directions );
+    Rand::shuffle( directions.begin(), directions.end(), Rand::CurrentThreadRandomDevice() );
 
     for ( size_t i = 0; i < directions.size(); ++i ) {
         if ( !Maps::isValidDirection( start, directions[i] ) ) {
@@ -1263,7 +1264,7 @@ std::list<Route::Step> AIWorldPathfinder::buildDimensionDoorPath( const int targ
     const bool isHeroOnWater = world.getTile( _pathStart ).isWater();
     const int32_t distanceLimit = Spell::CalculateDimensionDoorDistance() / 2;
     const uint32_t maxCasts = std::min( { remainingSpellPoints / _dimensionDoorSPCost, _remainingMovePoints / dimensionDoorMovementCost, difficultyLimit } );
-    const Directions & directions = Direction::All();
+    const auto & directions = Direction::allNeighboringDirections;
 
     std::list<Route::Step> path;
 
@@ -1336,7 +1337,7 @@ std::list<Route::Step> AIWorldPathfinder::buildDimensionDoorPath( const int targ
     return {};
 }
 
-std::list<Route::Step> AIWorldPathfinder::buildPath( const int targetIndex ) const
+std::list<Route::Step> AIWorldPathfinder::buildPath( const int targetIndex, const bool accountNearestObject ) const
 {
     assert( _cache.size() == world.getSize() && Maps::isValidAbsIndex( _pathStart ) && Maps::isValidAbsIndex( targetIndex ) );
 
@@ -1359,7 +1360,7 @@ std::list<Route::Step> AIWorldPathfinder::buildPath( const int targetIndex ) con
     while ( currentNode != _pathStart ) {
         assert( currentNode != -1 );
 
-        if ( !isTileAvailableForWalkThrough( currentNode, fromWater ) ) {
+        if ( accountNearestObject && !isTileAvailableForWalkThrough( currentNode, fromWater ) ) {
             lastValidNode = currentNode;
         }
 
